Package: cgmguru
Type: Package
Title: Advanced Continuous Glucose Monitoring Analysis with
        High-Performance C++ Backend
Version: 1.1.0
Authors@R: c(person(given = c("Sang", "Ho"),
                        family = "Park",
                        role = c("aut", "cre"),
                        email = "shstat1729@gmail.com"),
                 person(given = "Rosa",
                        family = "Oh",
                        role = c("aut", "ctb")),
                 person(given = "Sang-Man",
                        family = "Jin",
                        role = c("aut", "ctb")))
Maintainer: Sang Ho Park <shstat1729@gmail.com>
Description: Tools for advanced analysis of continuous glucose monitoring (CGM)
    time-series, implementing GRID (Glucose Rate Increase Detector) and GRID-based
    algorithms for postprandial peak detection, and detection of hypoglycemic and
    hyperglycemic episodes (Levels 1/2/Extended) aligned with international consensus
    CGM metrics. Core algorithms are implemented in optimized C++ using 'Rcpp' to
    provide accurate and fast analysis on large datasets.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
LinkingTo: Rcpp
Imports: Rcpp
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, iglu, dplyr, covr,
        ggplot2, microbenchmark
VignetteBuilder: knitr
URL: https://github.com/shstat1729/cgmguru
BugReports: https://github.com/shstat1729/cgmguru/issues
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-06-09 00:28:06 UTC; sanghopark
Author: Sang Ho Park [aut, cre],
  Rosa Oh [aut, ctb],
  Sang-Man Jin [aut, ctb]
Repository: CRAN
Date/Publication: 2026-06-09 02:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-06-09 02:51:13 UTC; unix
Archs: cgmguru.so.dSYM

# Generated by roxygen2: do not edit by hand

export(detect_all_events)
export(detect_between_maxima)
export(detect_hyperglycemic_events)
export(detect_hypoglycemic_events)
export(excursion)
export(find_local_maxima)
export(find_max_after_hours)
export(find_max_before_hours)
export(find_min_after_hours)
export(find_min_before_hours)
export(find_new_maxima)
export(grid)
export(interpolate_cgm)
export(maxima_grid)
export(mod_grid)
export(orderfast)
export(sensor_wear)
export(start_finder)
export(transform_df)
importFrom(Rcpp,evalCpp)
useDynLib(cgmguru, .registration = TRUE)

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Create start points for demonstration (using row index)
start_index <- seq(1, nrow(example_data_5_subject), by = 100)
start_points <- data.frame(start_index = start_index)

# Find maximum glucose in previous 2 hours
max_before <- find_max_before_hours(example_data_5_subject, start_points, hours = 2)
print(paste("Found", length(max_before$max_index), "maximum points"))

# Find maximum glucose in previous 1 hour
max_before_1h <- find_max_before_hours(example_data_5_subject, start_points, hours = 1)

# Analysis on larger dataset
large_start_index <- seq(1, nrow(example_data_hall), by = 200)
large_start_points <- data.frame(start_index = large_start_index)
large_max_before <- find_max_before_hours(example_data_hall, large_start_points, hours = 2)
print(paste("Found", length(large_max_before$max_index), "maximum points in larger dataset"))

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Level 1 Hypoglycemia Event (≥15 consecutive min of <70 mg/dL)
hypo_lv1 <- detect_hypoglycemic_events(
  example_data_5_subject,
  type = "lv1"
)
print(hypo_lv1$events_total)

# Level 2 Hypoglycemia Event (≥15 consecutive min of <54 mg/dL)
hypo_lv2 <- detect_hypoglycemic_events(
  example_data_5_subject,
  type = "lv2"
)

# Extended Hypoglycemia Event (default parameters)
hypo_extended <- detect_hypoglycemic_events(example_data_5_subject, type = "extended")

# Analysis on larger dataset
large_hypo <- detect_hypoglycemic_events(
  example_data_hall,
  type = "lv1"
)
print(paste("Total hypoglycemic events:", sum(large_hypo$events_total$total_events)))

# Compare Level 1 vs Level 2 hypoglycemia
cat("Level 1 events:", sum(hypo_lv1$events_total$total_events), "\n")
cat("Level 2 events:", sum(hypo_lv2$events_total$total_events), "\n")

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Shuffle without replacement, then order and compare to baseline
set.seed(123)
shuffled <- example_data_5_subject[sample(seq_len(nrow(example_data_5_subject)),
                                          replace = FALSE), ]
baseline <- orderfast(example_data_5_subject)
ordered_shuffled <- orderfast(shuffled)

# Compare results
print(paste("Identical after ordering:", identical(baseline, ordered_shuffled)))
head(baseline[, c("id", "time", "gl")])
head(ordered_shuffled[, c("id", "time", "gl")])

# Order larger dataset
ordered_large <- orderfast(example_data_hall)
print(paste("Ordered", nrow(ordered_large), "rows in larger dataset"))

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Combined analysis on smaller dataset
maxima_result <- maxima_grid(example_data_5_subject, threshold = 130, gap = 60, hours = 2)
print(maxima_result$episode_counts)
print(maxima_result$results)

# More sensitive analysis
sensitive_maxima <- maxima_grid(example_data_5_subject, threshold = 120, gap = 30, hours = 1)
print(sensitive_maxima$episode_counts)
print(sensitive_maxima$results)

# Analysis on larger dataset
large_maxima <- maxima_grid(example_data_hall, threshold = 130, gap = 60, hours = 2)
print(large_maxima$episode_counts)
print(large_maxima$results)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4,
  message = FALSE,
  warning = FALSE
)

library(cgmguru)

# Availability flags used in chunk eval options
iglu_available <- requireNamespace("iglu", quietly = TRUE)
iglu_api_ok <- iglu_available

## ----data-load, eval=iglu_available-------------------------------------------
data(example_data_5_subject, package = "iglu")
data(example_data_hall, package = "iglu")

# Base-R summaries (no external dependencies)
summary_5 <- data.frame(
  rows = nrow(example_data_5_subject),
  subjects = length(unique(example_data_5_subject$id)),
  time_min = min(example_data_5_subject$time),
  time_max = max(example_data_5_subject$time),
  gl_min = min(example_data_5_subject$gl, na.rm = TRUE),
  gl_max = max(example_data_5_subject$gl, na.rm = TRUE)
)

summary_5

## ----data-load-missing, eval=!iglu_available----------------------------------
# cat("Note: The 'iglu' package is not available; vignette examples are skipped.\n")

## ----iglu-episodes-5, eval=iglu_available-------------------------------------
iglu_episodes_5 <- iglu::episode_calculation(
  data = example_data_5_subject
)
print(iglu_episodes_5)

## ----iglu-episodes-hall, eval=iglu_api_ok-------------------------------------
iglu_episodes_hall <- iglu::episode_calculation(
  data = example_data_hall
)
print(iglu_episodes_hall)

## ----cgmguru-all-5, eval=iglu_available---------------------------------------
all_events_5 <- detect_all_events(example_data_5_subject, reading_minutes = 5)
print(all_events_5)

## ----cgmguru-all-hall, eval=iglu_available------------------------------------
all_events_hall <- detect_all_events(example_data_hall, reading_minutes = 5)
print(all_events_hall)

## ----speed-microbenchmark, cache=FALSE, eval=iglu_available-------------------
library(microbenchmark)
library(iglu)

# example_data_5_subject
bench_5 <- microbenchmark(
  episode_calculation = iglu::episode_calculation(example_data_5_subject),
  detect_all_events   = cgmguru::detect_all_events(example_data_5_subject, reading_minutes = 5),
  times = 10,
  unit = "ms"
)
print(bench_5)

# example_data_hall (all subjects)
bench_hall <- microbenchmark(
  episode_calculation = iglu::episode_calculation(example_data_hall),
  detect_all_events   = cgmguru::detect_all_events(example_data_hall, reading_minutes = 5),
  times = 10,
  unit = "ms"
)
print(bench_hall)

## ----iglu-api-mismatch, eval=iglu_available && !iglu_api_ok-------------------
# cat("Note: Installed 'iglu' version has a different 'episode_calculation' API; iglu examples are skipped.\n")

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Find local maxima
maxima_result <- find_local_maxima(example_data_5_subject)
print(paste("Found", nrow(maxima_result$local_maxima_vector), "local maxima"))

# Find maxima on larger dataset
large_maxima <- find_local_maxima(example_data_hall)
print(paste("Found", nrow(large_maxima$local_maxima_vector), "local maxima in larger dataset"))

# View first few maxima
head(maxima_result$local_maxima_vector)
# View merged results
head(maxima_result$merged_results)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", fig.width = 10, fig.height = 6)
library(cgmguru)
library(iglu)
library(ggplot2)
library(dplyr)
set.seed(123)

## ----eval=FALSE---------------------------------------------------------------
# ?grid
# ?detect_all_events

## ----load-data----------------------------------------------------------------
# Load example datasets
data(example_data_5_subject)  # 5 subjects, 13,866 readings
data(example_data_hall)       # 19 subjects, 34,890 readings

# Display basic information about the datasets
cat("Dataset 1 (example_data_5_subject):\n")
cat("  Rows:", nrow(example_data_5_subject), "\n")
cat("  Subjects:", length(unique(example_data_5_subject$id)), "\n")
cat("  Time range:", as.character(range(example_data_5_subject$time)), "\n")
cat("  Glucose range:", range(example_data_5_subject$gl), "mg/dL\n\n")

cat("Dataset 2 (example_data_hall):\n")
cat("  Rows:", nrow(example_data_hall), "\n")
cat("  Subjects:", length(unique(example_data_hall$id)), "\n")
cat("  Time range:", as.character(range(example_data_hall$time)), "\n")
cat("  Glucose range:", range(example_data_hall$gl), "mg/dL\n")

# Show first few rows
head(example_data_5_subject)

## ----grid-analysis------------------------------------------------------------
# Perform GRID analysis on the smaller dataset
grid_result <- grid(example_data_5_subject, gap = 15, threshold = 130)

# Display results
cat("GRID Analysis Results:\n")
cat("  Detected grid points:", nrow(grid_result$grid_vector), "\n")
cat("  Episode counts:\n")
print(grid_result$episode_counts)

# Show first few detected grid points
cat("\nFirst few detected grid points:\n")
head(grid_result$grid_vector)

## ----hyperglycemic-events-----------------------------------------------------
# Level 1 Hyperglycemic events (≥15 consecutive minutes >180 mg/dL)
hyper_lv1 <- detect_hyperglycemic_events(
  example_data_5_subject, 
  type = "lv1"
)

# Level 2 Hyperglycemic events (≥15 consecutive minutes >250 mg/dL)
hyper_lv2 <- detect_hyperglycemic_events(
  example_data_5_subject, 
  type = "lv2"
)

# Extended Hyperglycemic events (default parameters)
hyper_extended <- detect_hyperglycemic_events(example_data_5_subject, type = "extended")

cat("Hyperglycemic Event Detection Results:\n")
cat("Level 1 Events (>180 mg/dL):\n")
print(hyper_lv1$events_total)

cat("\nLevel 2 Events (>250 mg/dL):\n")
print(hyper_lv2$events_total)

cat("\nExtended Events (default):\n")
print(hyper_extended$events_total)

# Show detailed events for first subject
cat("\nDetailed Level 1 Events for Subject", hyper_lv1$events_detailed$id[1], ":\n")
head(hyper_lv1$events_detailed[hyper_lv1$events_detailed$id == hyper_lv1$events_detailed$id[1], ])

## ----hypoglycemic-events------------------------------------------------------
# Level 1 Hypoglycemic events (≤70 mg/dL)
hypo_lv1 <- detect_hypoglycemic_events(
  example_data_5_subject, 
  type = "lv1"
)

# Level 2 Hypoglycemic events (≤54 mg/dL)
hypo_lv2 <- detect_hypoglycemic_events(
  example_data_5_subject, 
  type = "lv2"
)

cat("Hypoglycemic Event Detection Results:\n")
cat("Level 1 Events (≤70 mg/dL):\n")
print(hypo_lv1$events_total)

cat("\nLevel 2 Events (≤54 mg/dL):\n")
print(hypo_lv2$events_total)

## ----all-events---------------------------------------------------------------
# Detect all events with 5-minute reading intervals
all_events <- detect_all_events(example_data_5_subject, reading_minutes = 5)

cat("Comprehensive Event Detection Results:\n")
print(all_events)

## ----local-maxima-------------------------------------------------------------
# Find local maxima
maxima_result <- find_local_maxima(example_data_5_subject)

cat("Local Maxima Detection Results:\n")
cat("  Total local maxima found:", nrow(maxima_result$local_maxima_vector), "\n")
cat("  Merged results:", nrow(maxima_result$merged_results), "\n")

# Show first few maxima
head(maxima_result$local_maxima_vector)

## ----maxima-grid--------------------------------------------------------------
# Combined maxima and GRID analysis
maxima_grid_result <- maxima_grid(
  example_data_5_subject, 
  threshold = 130, 
  gap = 60, 
  hours = 2
)

cat("Maxima-GRID Combined Analysis Results:\n")
cat("  Detected maxima:", nrow(maxima_grid_result$results), "\n")
cat("  Episode counts:\n")
print(maxima_grid_result$episode_counts)

# Show first few results
head(maxima_grid_result$results)

## ----excursion-analysis-------------------------------------------------------
# Excursion analysis
excursion_result <- excursion(example_data_5_subject, gap = 15)

cat("Excursion Analysis Results:\n")
cat("  Excursion vector length:", length(excursion_result$excursion_vector), "\n")
cat("  Episode counts:\n")
print(excursion_result$episode_counts)

# Show episode start information
head(excursion_result$episode_start)

## ----complete-pipeline--------------------------------------------------------
# Use the larger dataset for comprehensive analysis
cat("Running complete analysis pipeline on example_data_hall...\n")

# Step 1: GRID analysis
cat("Step 1: GRID Analysis\n")
grid_pipeline <- grid(example_data_hall, gap = 15, threshold = 130)
cat("  Detected", nrow(grid_pipeline$grid_vector), "grid points\n")

# Step 2: Local maxima detection
cat("Step 2: Local Maxima Detection\n")
maxima_pipeline <- find_local_maxima(example_data_hall)
cat("  Found", nrow(maxima_pipeline$local_maxima_vector), "local maxima\n")

# Step 3: Modified GRID analysis
cat("Step 3: Modified GRID Analysis\n")
mod_grid_pipeline <- mod_grid(
  example_data_hall, 
  grid_pipeline$grid_vector, 
  hours = 2, 
  gap = 15
)
cat("  Modified grid points:", nrow(mod_grid_pipeline$mod_grid_vector), "\n")

# Step 4: Find maximum points after modified GRID points
cat("Step 4: Finding Maximum Points After GRID Points\n")
max_after_pipeline <- find_max_after_hours(
  example_data_hall,
  mod_grid_pipeline$mod_grid_vector,
  hours = 2
)
cat("  Maximum points found:", length(max_after_pipeline$max_index), "\n")

# Step 5: Find new maxima
cat("Step 5: Finding New Maxima\n")
new_maxima_pipeline <- find_new_maxima(
  example_data_hall,
  max_after_pipeline$max_index,
  maxima_pipeline$local_maxima_vector
)
cat("  New maxima identified:", nrow(new_maxima_pipeline), "\n")

# Step 6: Transform dataframes
cat("Step 6: Transforming Dataframes\n")
transformed_pipeline <- transform_df(
  grid_pipeline$episode_start, 
  new_maxima_pipeline
)
cat("  Transformed dataframe rows:", nrow(transformed_pipeline), "\n")

# Step 7: Detect between maxima
cat("Step 7: Detecting Between Maxima\n")
between_maxima_pipeline <- detect_between_maxima(
  example_data_hall, 
  transformed_pipeline
)
cat("  Between maxima analysis completed\n")

cat("\nComplete pipeline executed successfully!\n")

## ----time-based-analysis------------------------------------------------------
# Create a subset for demonstration
subset_data <- example_data_5_subject[example_data_5_subject$id == unique(example_data_5_subject$id)[1], ][1:100, ]

# Create start points for time-based analysis
start_points <- subset_data[seq(1, nrow(subset_data), by = 20), ]

cat("Time-Based Analysis Functions:\n")

# Find maximum after 1 hour
max_after <- find_max_after_hours(subset_data, start_points, hours = 1)
cat("  Max after 1 hour:", length(max_after$max_index), "points\n")

# Find maximum before 1 hour
max_before <- find_max_before_hours(subset_data, start_points, hours = 1)
cat("  Max before 1 hour:", length(max_before$max_index), "points\n")

# Find minimum after 1 hour
min_after <- find_min_after_hours(subset_data, start_points, hours = 1)
cat("  Min after 1 hour:", length(min_after$min_index), "points\n")

# Find minimum before 1 hour
min_before <- find_min_before_hours(subset_data, start_points, hours = 1)
cat("  Min before 1 hour:", length(min_before$min_index), "points\n")

## ----ordering-utility---------------------------------------------------------
# Create sample data with mixed order
sample_data <- data.frame(
  id = c("b", "a", "c", "a", "b"),
  time = as.POSIXct(c("2023-01-01 10:00:00", "2023-01-01 09:00:00", 
                      "2023-01-01 11:00:00", "2023-01-01 08:00:00", 
                      "2023-01-01 12:00:00"), tz = "UTC"),
  gl = c(120, 100, 140, 90, 130)
)

cat("Original data (unordered):\n")
print(sample_data)

# Order the data
ordered_data <- orderfast(sample_data)

cat("\nOrdered data:\n")
print(ordered_data)

## ----visualization, fig.width=12, fig.height=8--------------------------------
# Select one subject for visualization
subject_id <- unique(example_data_5_subject$id)[1]
subject_data <- example_data_5_subject[example_data_5_subject$id == subject_id, ]

# Create a comprehensive plot
p1 <- ggplot(subject_data, aes(x = time, y = gl)) +
  geom_line(color = "blue", alpha = 0.7, size = 0.5) +
  geom_hline(yintercept = 180, color = "red", linetype = "dashed", alpha = 0.8) +
  geom_hline(yintercept = 250, color = "darkred", linetype = "dashed", alpha = 0.8) +
  geom_hline(yintercept = 70, color = "orange", linetype = "dashed", alpha = 0.8) +
  geom_hline(yintercept = 54, color = "darkorange", linetype = "dashed", alpha = 0.8) +
  labs(title = paste("CGM Data for Subject", subject_id, "with Clinical Thresholds"),
       subtitle = "Red lines: Hyperglycemia thresholds (180, 250 mg/dL)\nOrange lines: Hypoglycemia thresholds (70, 54 mg/dL)",
       x = "Time", 
       y = "Glucose (mg/dL)") +
  theme_minimal() +
  theme(plot.title = element_text(size = 14, face = "bold"),
        plot.subtitle = element_text(size = 10))

print(p1)

# Create a summary plot showing event counts across subjects
event_summary <- hyper_lv1$events_total
event_summary$subject <- paste("Subject", event_summary$id)

p2 <- ggplot(event_summary, aes(x = subject, y = total_events)) +
  geom_col(fill = "steelblue", alpha = 0.7) +
  geom_text(aes(label = total_events), vjust = -0.5) +
  labs(title = "Level 1 Hyperglycemic Events by Subject",
       x = "Subject",
       y = "Number of Events") +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust = 1))

print(p2)

## ----performance-comparison---------------------------------------------------
# Function to measure execution time
measure_time <- function(expr) {
  start_time <- Sys.time()
  result <- eval(expr)
  end_time <- Sys.time()
  return(list(result = result, time = as.numeric(end_time - start_time, units = "secs")))
}

cat("Performance Comparison:\n")

# Test on smaller dataset
cat("Small dataset (5 subjects, 13,866 readings):\n")
small_time <- measure_time(grid(example_data_5_subject, gap = 15, threshold = 130))
cat("  GRID analysis time:", round(small_time$time, 3), "seconds\n")

small_maxima_time <- measure_time(find_local_maxima(example_data_5_subject))
cat("  Local maxima time:", round(small_maxima_time$time, 3), "seconds\n")

# Test on larger dataset
cat("\nLarge dataset (19 subjects, 34,890 readings):\n")
large_time <- measure_time(grid(example_data_hall, gap = 15, threshold = 130))
cat("  GRID analysis time:", round(large_time$time, 3), "seconds\n")

large_maxima_time <- measure_time(find_local_maxima(example_data_hall))
cat("  Local maxima time:", round(large_maxima_time$time, 3), "seconds\n")

# Calculate efficiency
efficiency_ratio <- (large_time$time / large_time$result$episode_counts$total_episodes) / 
                    (small_time$time / small_time$result$episode_counts$total_episodes)
cat("\nEfficiency ratio (large/small):", round(efficiency_ratio, 2))

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4,
  message = FALSE,
  warning = FALSE
)

library(cgmguru)

# Availability flags used in chunk eval options
iglu_available <- requireNamespace("iglu", quietly = TRUE)
iglu_api_ok <- iglu_available

## ----data-load, eval=iglu_available-------------------------------------------
data(example_data_5_subject, package = "iglu")
data(example_data_hall, package = "iglu")

# Base-R summaries (no external dependencies)
summary_5 <- data.frame(
  rows = nrow(example_data_5_subject),
  subjects = length(unique(example_data_5_subject$id)),
  time_min = min(example_data_5_subject$time),
  time_max = max(example_data_5_subject$time),
  gl_min = min(example_data_5_subject$gl, na.rm = TRUE),
  gl_max = max(example_data_5_subject$gl, na.rm = TRUE)
)

summary_5

## ----data-load-missing, eval=!iglu_available----------------------------------
# cat("Note: The 'iglu' package is not available; vignette examples are skipped.\n")

## ----iglu-episodes-5, eval=iglu_available-------------------------------------
iglu_episodes_5 <- iglu::episode_calculation(
  data = example_data_5_subject
)
print(iglu_episodes_5)

## ----iglu-episodes-hall, eval=iglu_api_ok-------------------------------------
iglu_episodes_hall <- iglu::episode_calculation(
  data = example_data_hall
)
print(iglu_episodes_hall)

## ----cgmguru-all-5, eval=iglu_available---------------------------------------
all_events_5 <- detect_all_events(example_data_5_subject)
print(all_events_5)

## ----cgmguru-all-hall, eval=iglu_available------------------------------------
all_events_hall <- detect_all_events(example_data_hall)
print(all_events_hall)

## ----speed-microbenchmark, cache=FALSE, eval=iglu_available-------------------
library(microbenchmark)
library(iglu)

# example_data_5_subject
bench_5 <- microbenchmark(
  episode_calculation = iglu::episode_calculation(example_data_5_subject),
  detect_all_events   = cgmguru::detect_all_events(example_data_5_subject),
  times = 10,
  unit = "ms"
)
print(bench_5)

# example_data_hall (all subjects)
bench_hall <- microbenchmark(
  episode_calculation = iglu::episode_calculation(example_data_hall),
  detect_all_events   = cgmguru::detect_all_events(example_data_hall),
  times = 10,
  unit = "ms"
)
print(bench_hall)

## ----iglu-api-mismatch, eval=iglu_available && !iglu_api_ok-------------------
# cat("Note: Installed 'iglu' version has a different 'episode_calculation' API; iglu examples are skipped.\n")

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(cgmguru)
library(iglu)

## ----data-load----------------------------------------------------------------
data(example_data_5_subject)
data(example_data_hall)

## ----grid-analysis------------------------------------------------------------
grid_result <- grid(example_data_5_subject, gap = 15, threshold = 130)

# View the number of GRID events detected per subject
print(grid_result$episode_counts)

# View the start points of GRID events
print(head(grid_result$episode_start))

# See the identified points in the time series
grid_points <- head(grid_result$grid_vector)
print(grid_points)

## ----grid-sensitive-----------------------------------------------------------
sensitive_result <- grid(example_data_5_subject, gap = 10, threshold = 120)

print(head(sensitive_result$episode_counts))

## ----grid-large---------------------------------------------------------------
large_grid <- grid(example_data_hall, gap = 15, threshold = 130)
print(paste("Detected", sum(large_grid$episode_counts$episode_counts), "episodes"))

## ----plot-grid, fig.height=4--------------------------------------------------
library(ggplot2)
subid <- example_data_5_subject$id[1]
subdata <- example_data_5_subject[example_data_5_subject$id == subid, ]
substarts <- grid_result$episode_start[grid_result$episode_start$id == subid, ]

plot <- ggplot(subdata, aes(x = time, y = gl)) +
  geom_line() +
  geom_point(data = substarts, aes(x = time, y = gl), color = 'red', size = 2) +
  labs(title = paste("GRID Events for Subject", subid), y = "Glucose (mg/dL)")
plot

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Basic GRID analysis on smaller dataset
grid_result <- grid(example_data_5_subject, gap = 15, threshold = 130)
print(grid_result$episode_counts)
print(grid_result$episode_start)
print(grid_result$grid_vector)

# More sensitive GRID analysis
sensitive_result <- grid(example_data_5_subject, gap = 10, threshold = 120)

# GRID analysis on larger dataset
large_grid <- grid(example_data_hall, gap = 15, threshold = 130)
print(paste("Detected", sum(large_grid$episode_counts$episode_counts), "episodes"))
print(large_grid$episode_start)
print(large_grid$grid_vector)

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Create a binary vector indicating episode starts
binary_vector <- c(0, 0, 1, 1, 0, 1, 0, 0, 1, 1)
df <- data.frame(episode_starts = binary_vector)

# Find R-based index where episodes start
start_points <- start_finder(df)
print(paste("Start index:", paste(start_points$start_index, collapse = ", ")))

# Use with actual GRID results
grid_result <- grid(example_data_5_subject, gap = 15, threshold = 130)
grid_starts <- start_finder(grid_result$grid_vector)
print(paste("GRID episode starts:", length(grid_starts$start_index)))

# Analysis on larger dataset
large_grid <- grid(example_data_hall, gap = 15, threshold = 130)
large_starts <- start_finder(large_grid$grid_vector)
print(paste("GRID episode starts in larger dataset:", length(large_starts$start_index)))

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Detect all glycemic events with 5-minute reading intervals
all_events <- detect_all_events(example_data_5_subject, reading_minutes = 5)
print(all_events)

# Detect all events on larger dataset
large_all_events <- detect_all_events(example_data_hall, reading_minutes = 5)
print(paste("Total event types analyzed:", nrow(large_all_events)))

# Filter for specific event types
hyperglycemia_events <- all_events[all_events$type == "hyper", ]
hypoglycemia_events <- all_events[all_events$type == "hypo", ]

print("Hyperglycemia events:")
print(hyperglycemia_events)
print("Hypoglycemia events:")
print(hypoglycemia_events)

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Calculate glucose excursions
excursion_result <- excursion(example_data_5_subject, gap = 15)
print(paste("Excursion vector length:", length(excursion_result$excursion_vector)))
print(excursion_result$episode_counts)

# Excursion analysis with different gap
excursion_30min <- excursion(example_data_5_subject, gap = 30)

# Analysis on larger dataset
large_excursion <- excursion(example_data_hall, gap = 15)
print(paste("Excursion vector length in larger dataset:", length(large_excursion$excursion_vector)))
print(paste("Total episodes:", sum(large_excursion$episode_counts$episode_counts)))

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Complete pipeline example with smaller dataset
threshold <- 130
gap <- 60
hours <- 2
# 1) Find GRID points
grid_result <- grid(example_data_5_subject, gap = gap, threshold = threshold)

# 2) Find modified GRID points before 2 hours minimum
mod_grid <- mod_grid(example_data_5_subject,
                     start_finder(grid_result$grid_vector),
                     hours = hours,
                     gap = gap)

# 3) Find maximum point 2 hours after mod_grid point
mod_grid_maxima <- find_max_after_hours(example_data_5_subject,
                                        start_finder(mod_grid$mod_grid_vector),
                                        hours = hours)

# 4) Identify local maxima around episodes/windows
local_maxima <- find_local_maxima(example_data_5_subject)

# 5) Among local maxima, find maximum point after two hours
final_maxima <- find_new_maxima(example_data_5_subject,
                                mod_grid_maxima$max_index,
                                local_maxima$local_maxima_vector)

# 6) Map GRID points to maximum points (within 4 hours)
transform_maxima <- transform_df(grid_result$episode_start, final_maxima)

# 7) Redistribute overlapping maxima between GRID points
final_between_maxima <- detect_between_maxima(example_data_5_subject, transform_maxima)

# Complete pipeline example with larger dataset (example_data_hall)
hall_threshold <- 130
hall_gap <- 60
hall_hours <- 2

# 1) Find GRID points on larger dataset
hall_grid_result <- grid(example_data_hall, gap = hall_gap, threshold = hall_threshold)

# 2) Find modified GRID points
hall_mod_grid <- mod_grid(example_data_hall,
                         start_finder(hall_grid_result$grid_vector),
                         hours = hall_hours,
                         gap = hall_gap)

# 3) Find maximum points after mod_grid
hall_mod_grid_maxima <- find_max_after_hours(example_data_hall,
                                            start_finder(hall_mod_grid$mod_grid_vector),
                                            hours = hall_hours)

# 4) Identify local maxima
hall_local_maxima <- find_local_maxima(example_data_hall)

# 5) Find new maxima
hall_final_maxima <- find_new_maxima(example_data_hall,
                                     hall_mod_grid_maxima$max_index,
                                     hall_local_maxima$local_maxima_vector)

# 6) Transform data
hall_transform_maxima <- transform_df(hall_grid_result$episode_start, hall_final_maxima)

# 7) Detect between maxima
hall_final_between_maxima <- detect_between_maxima(example_data_hall, hall_transform_maxima)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", fig.width = 10, fig.height = 6, warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
library(ggplot2)
library(dplyr)

# Use package root as working directory so we can reference ../examples
pkg_root <- normalizePath("..", winslash = "/", mustWork = TRUE)
examples_dir <- file.path(pkg_root, "inst", "examples")
stopifnot(dir.exists(examples_dir))

## ----ex-grid------------------------------------------------------------------
source(file.path(examples_dir, "grid.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-detect-all-events-----------------------------------------------------
source(file.path(examples_dir, "detect_all_events.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-detect-hyper----------------------------------------------------------
source(file.path(examples_dir, "detect_hyperglycemic_events.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-detect-hypo-----------------------------------------------------------
source(file.path(examples_dir, "detect_hypoglycemic_events.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-excursion-------------------------------------------------------------
source(file.path(examples_dir, "excursion.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-local-maxima----------------------------------------------------------
source(file.path(examples_dir, "find_local_maxima.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-max-after-------------------------------------------------------------
source(file.path(examples_dir, "find_max_after_hours.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-max-before------------------------------------------------------------
source(file.path(examples_dir, "find_max_before_hours.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-min-after-------------------------------------------------------------
source(file.path(examples_dir, "find_min_after_hours.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-min-before------------------------------------------------------------
source(file.path(examples_dir, "find_min_before_hours.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-maxima-grid-----------------------------------------------------------
source(file.path(examples_dir, "maxima_grid.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-orderfast-------------------------------------------------------------
source(file.path(examples_dir, "orderfast.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-start-finder----------------------------------------------------------
source(file.path(examples_dir, "start_finder.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----ex-transform-df----------------------------------------------------------
source(file.path(examples_dir, "transform_df.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)

## ----eval=FALSE---------------------------------------------------------------
# help(package = "cgmguru")

## ----eval=FALSE---------------------------------------------------------------
# browseVignettes("cgmguru")

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Level 1 Hyperglycemia Event (≥15 consecutive min of >180 mg/dL)
hyper_lv1 <- detect_hyperglycemic_events(
  example_data_5_subject,
  type = "lv1"
)
print(hyper_lv1$events_total)

# Level 2 Hyperglycemia Event (≥15 consecutive min of >250 mg/dL)
hyper_lv2 <- detect_hyperglycemic_events(
  example_data_5_subject,
  type = "lv2"
)
print(hyper_lv2$events_total)
# Extended Hyperglycemia Event (default parameters)
hyper_extended <- detect_hyperglycemic_events(example_data_5_subject, type = "extended")

# Analysis on larger dataset
large_hyper <- detect_hyperglycemic_events(
  example_data_hall,
  type = "lv1"
)
print(paste("Total hyperglycemic episodes:", sum(large_hyper$events_total$total_episodes)))

# View detailed events for first subject
if(nrow(hyper_lv1$events_detailed) > 0) {
  first_subject <- hyper_lv1$events_detailed$id[1]
  subject_events <- hyper_lv1$events_detailed[hyper_lv1$events_detailed$id == first_subject, ]
  head(subject_events)
}

#' @title GRID Algorithm for Glycemic Event Detection
#' @name grid
#' @description
#' Implements the GRID (Glucose Rate Increase Detector) algorithm for detecting rapid glucose rate increases in continuous glucose monitoring (CGM) data.
#' This algorithm identifies rapid glucose changes using specific rate-based criteria, and is commonly applied for meal detection.
#' Meals are detected when the CGM value is \eqn{\geq} 7.2 mmol/L (\eqn{\geq} 130 mg/dL) and the rate-of-change is \eqn{\geq} 5.3 mmol/L/h [\eqn{\geq} 95 mg/dL/h] for the last two consecutive readings, or \eqn{\geq} 5.0 mmol/L/h [\eqn{\geq} 90 mg/dL/h] for two of the last three readings.
#'
#'
#' @references
#' Harvey, R. A., et al. (2014). Design of the glucose rate increase detector: a meal detection module for the health monitoring system. Journal of Diabetes Science and Technology, 8(2), 307-320.
#'
#' Adolfsson, Peter, et al. "Increased time in range and fewer missed bolus injections after introduction of a smart connected insulin pen." Diabetes technology & therapeutics 22.10 (2020): 709-718.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param gap Gap threshold in minutes for event detection (default: 15).
#'   This parameter defines the minimum time interval between consecutive GRID events. For example, if gap is set to 60, only one GRID event can be detected within any one-hour window; subsequent events within the gap interval are not counted as new events.
#' @param threshold GRID slope threshold in mg/dL/hour for event classification (default: 130)
#' @usage grid(df, gap = 15, threshold = 130)
#' @section Algorithm:
#' - Flags points where \code{gl >= 130 mg/dL} and rate-of-change meets the GRID criteria (see references).
#' - Enforces a minimum \code{gap} in minutes between detected events to avoid duplicates.
#' @section Units and sampling:
#' - \code{gl} is mg/dL; \code{time} is POSIXct; \code{gap} is minutes.
#' - The effective sampling interval is derived from \code{time} deltas.
#' - This function operates on the rows supplied in \code{df}. It does not use
#'   \code{\link{interpolate_cgm}} or the full-day event preprocessing grid.
#' @seealso \link{mod_grid}, \link{maxima_grid}, \link{find_local_maxima}, \link{detect_between_maxima}
#' @family GRID pipeline
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{grid_vector}: A tibble with the results of the GRID analysis. Contains a \code{grid} column (0/1 values; 1 denotes a detected GRID event) and all relevant input columns.
#'   \item \code{episode_counts}: A tibble summarizing the number of GRID events per subject (\code{id}) as \code{episode_counts}.
#'   \item \code{episode_start}: A tibble listing the start of each GRID episode, with columns:
#'     \itemize{
#'       \item \code{id}: Subject ID.
#'       \item \code{time}: The timestamp (POSIXct) at which the GRID event was detected.
#'       \item \code{gl}: The glucose value (mg/dL; integer or numeric) at the GRID event.
#'       \item \code{index}: R-based (1-indexed) row number(s) in the original dataframe where the GRID event occurs. The occurrence time equals \code{df$time[index]} and glucose equals \code{df$gl[index]}.
#'     }
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Basic GRID analysis on smaller dataset
#' grid_result <- grid(example_data_5_subject, gap = 15, threshold = 130)
#' print(grid_result$episode_counts)
#' print(grid_result$episode_start)
#' print(grid_result$grid_vector)
#'
#' # More sensitive GRID analysis
#' sensitive_result <- grid(example_data_5_subject, gap = 10, threshold = 120)
#'
#' # GRID analysis on larger dataset
#' large_grid <- grid(example_data_hall, gap = 15, threshold = 130)
#' print(paste("Detected", sum(large_grid$episode_counts$episode_counts), "episodes"))
#' print(large_grid$episode_start)
#' print(large_grid$grid_vector)
#'

NULL

#' @title Combined Maxima Detection and GRID Analysis
#' @name maxima_grid
#' @description
#' Fast method for postprandial glucose peak detection combining GRID algorithm with local maxima analysis.
#' Detects meal-induced glucose peaks by identifying GRID events (rapid glucose increases) and mapping
#' them to corresponding local maxima within a search window. Local maxima are defined as points where
#' glucose values increase or remain constant for two consecutive points before the peak, and decrease
#' or remain constant for two consecutive points after the peak.
#'
#' The 7-step algorithm:
#' (1) finds GRID points indicating meal starts
#' (2) identifies modified GRID points after minimum duration
#' (3) locates maximum glucose within the subsequent time window
#' (4) detects all local maxima using the two-consecutive-point criteria
#' (5) refines peaks from local maxima candidates
#' (6) maps GRID points to peaks within 4-hour constraint
#' (7) redistributes overlapping peaks.
#'
#' @references
#' Park, Sang Ho, et al. "Identification of clinically meaningful automatically detected postprandial glucose excursions in individuals with type 1 diabetes using personal continuous glucose monitoring." Diabetes Research and Clinical Practice (2025): 112951.
#'
#' Park, Soojin, et al. "High-Amplitude and Prolonged Glucose Excursions as a Key Determinant of Discordance Between Glucose Management Indicator and Glycated Hemoglobin in Type 1 Diabetes." Diabetes Care (2026): dc252820. https://doi.org/10.2337/dc25-2820
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param threshold GRID slope threshold in mg/dL/hour for event classification (default: 130)
#' @param gap Gap threshold in minutes for event detection (default: 60).
#'   This parameter defines the minimum time interval between consecutive GRID events.
#' @param hours Time window in hours for maxima analysis (default: 2)
#' @usage maxima_grid(df, threshold = 130, gap = 60, hours = 2)
#' @section Algorithm (7 steps):
#' 1) GRID -> 2) modified GRID -> 3) window maxima -> 4) local maxima -> 5) refine peaks ->
#' 6) map GRID to peaks (\eqn{\leq} 4h) -> 7) redistribute overlapping peaks.
#' @section Input grid:
#' This function operates on the rows supplied in \code{df}. It does not use
#' \code{\link{interpolate_cgm}} or the full-day event preprocessing grid unless
#' the caller explicitly supplies interpolated data.
#' @seealso \link{grid}, \link{mod_grid}, \link{find_local_maxima}, \link{find_new_maxima}, \link{transform_df}
#' @family GRID pipeline
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{results}: Tibble with combined maxima and GRID analysis results, with columns:
#'     \itemize{
#'       \item \code{id}: Subject identifier
#'       \item \code{grid_time}: Timestamp of GRID event detection (POSIXct)
#'       \item \code{grid_gl}: Glucose value at GRID event (mg/dL)
#'       \item \code{maxima_time}: Timestamp of peak glucose (POSIXct)
#'       \item \code{maxima_glucose}: Peak glucose value (mg/dL)
#'       \item \code{time_to_peak_min}: Time from GRID event to peak in minutes
#'       \item \code{grid_index}: R-based (1-indexed) row number of GRID event; \code{grid_time == df$time[grid_index]}, \code{grid_gl == df$gl[grid_index]}
#'       \item \code{maxima_index}: R-based (1-indexed) row number of peak; \code{maxima_time == df$time[maxima_index]}, \code{maxima_glucose == df$gl[maxima_index]}
#'     }
#'   \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Combined analysis on smaller dataset
#' maxima_result <- maxima_grid(example_data_5_subject, threshold = 130, gap = 60, hours = 2)
#' print(maxima_result$episode_counts)
#' print(maxima_result$results)
#'
#' # More sensitive analysis
#' sensitive_maxima <- maxima_grid(example_data_5_subject, threshold = 120, gap = 30, hours = 1)
#' print(sensitive_maxima$episode_counts)
#' print(sensitive_maxima$results)
#'
#' # Analysis on larger dataset
#' large_maxima <- maxima_grid(example_data_hall, threshold = 130, gap = 60, hours = 2)
#' print(large_maxima$episode_counts)
#' print(large_maxima$results)
NULL

#' @title Detect Hyperglycemic Events
#' @name detect_hyperglycemic_events
#' @encoding UTF-8
#' @description
#' Identifies and segments hyperglycemic events in CGM data based on international consensus
#' CGM metrics (Battelino et al., 2023). Use \code{type} to select one of
#' three event definitions:
#' \itemize{
#'   \item \strong{Level 1}: \eqn{\geq} 15 consecutive min of \eqn{>} 180 mg/dL, ends with \eqn{\geq} 15 consecutive min \eqn{\leq} 180 mg/dL
#'   \item \strong{Level 2}: \eqn{\geq} 15 consecutive min of \eqn{>} 250 mg/dL, ends with \eqn{\geq} 15 consecutive min \eqn{\leq} 250 mg/dL
#'   \item \strong{Extended}: \eqn{>} 250 mg/dL lasting \eqn{\geq} 90 cumulative min within a 120-min period, ends when glucose returns to \eqn{\leq} 180 mg/dL
#'     for \eqn{\geq} 15 consecutive min after
#' }
#' Events are counted only after glucose remains at or below the end threshold
#' for the specified end length. In \code{events_detailed}, \code{end_time},
#' \code{end_glucose}, and \code{end_index} report the last hyperglycemic
#' reading immediately before that confirmed recovery period starts.
#'
#' @references
#' Battelino, T., et al. (2023). Continuous glucose monitoring and metrics for clinical trials: an international consensus statement. The Lancet Diabetes & Endocrinology, 11(1), 42-57.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param ... Custom event criteria supplied by name. Prefer \code{type} for
#'   standard Level 1, Level 2, and Extended events. Supported custom criteria
#'   are:
#'   \itemize{
#'     \item \code{dur_length}: Minimum event duration in minutes required for
#'       an event to qualify.
#'     \item \code{end_length}: Required recovery duration in minutes before
#'       event termination is confirmed.
#'     \item \code{start_gl}: Glucose threshold in mg/dL used to qualify
#'       hyperglycemic readings. Hyperglycemic readings are above this value.
#'     \item \code{end_gl}: Glucose threshold in mg/dL used to confirm
#'       recovery. Hyperglycemic events end after glucose remains at or below
#'       this value for \code{end_length} minutes.
#'   }
#' @param type Hyperglycemia event definition. One of \code{"extended"}
#'   (default), \code{"lv1"}, \code{"lv2"}, or \code{"lv1_excl"}.
#' @param reading_minutes Time interval between readings in minutes (optional).
#'   If omitted or \code{NULL}, the interval is calculated automatically per id
#'   as the median positive time difference in the data.
#' @param sort_time Logical. If \code{TRUE}, sort rows within each id by
#'   \code{time} in C++ before interpolation. Defaults to \code{FALSE}.
#' @param inter_gap Maximum gap in minutes to interpolate across. Defaults to
#'   45; larger gaps split event-detection segments.
#' @param return_interpolated Logical. If \code{TRUE}, include the interpolated
#'   grid data used for event detection in the returned list. Defaults to
#'   \code{TRUE}.
#' @usage detect_hyperglycemic_events(df, ..., type = "extended",
#'  reading_minutes = NULL, sort_time = FALSE, inter_gap = 45,
#'  return_interpolated = TRUE)
#' @section Methods:
#' Hyperglycemic events can be detected using either the recommended
#' \code{type} argument or named custom threshold and duration criteria.
#'
#' \strong{1. Preset method using \code{type} (recommended):}
#' Use \code{type} when you want the standard Level 1, Level 2, or Extended
#' hyperglycemia definitions without manually entering thresholds.
#' \itemize{
#'   \item \code{type = "lv1"} uses \code{start_gl = 180}, \code{dur_length = 15},
#'     \code{end_length = 15}, and \code{end_gl = 180}.
#'   \item \code{type = "lv2"} uses \code{start_gl = 250}, \code{dur_length = 15},
#'     \code{end_length = 15}, and \code{end_gl = 250}.
#'   \item \code{type = "extended"} uses \code{start_gl = 250},
#'     \code{dur_length = 120}, \code{end_length = 15}, and \code{end_gl = 180}.
#'   \item \code{type = "lv1_excl"} returns Level 1 episodes that do not
#'     overlap Level 2 episodes.
#' }
#'
#' \strong{2. Custom criteria method:}
#' Supply \code{start_gl}, \code{dur_length}, \code{end_length}, and
#' \code{end_gl} directly when using a custom definition, for example
#' \code{detect_hyperglycemic_events(df, start_gl = 180, dur_length = 15,
#' end_length = 15, end_gl = 180)} for Level 1 hyperglycemia. If an explicit
#' \code{type} is supplied together with custom numeric criteria, the function
#' returns results based on \code{type}; the custom criteria are ignored and a
#' warning is issued.
#' @section Units and sampling:
#' - \code{reading_minutes} can be a scalar (all rows) or a vector per-row.
#' - If \code{reading_minutes} is omitted or \code{NULL}, it is calculated
#'   automatically per id from timestamp spacing.
#' - Event classification uses cgmguru's independent C++ implementation of an
#'   iglu-compatible, midnight-aligned full-day grid. Data are linearly
#'   interpolated at the id-specific interval up to \code{inter_gap}; larger
#'   gaps are masked, removed from the event-classification data, and split
#'   segments.
#' - This preprocessing is specific to event calculation and does not affect
#'   \code{\link{grid}}, \code{\link{maxima_grid}}, or \code{\link{excursion}}.
#' @seealso \link{detect_all_events}
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{events_total}: Tibble with summary statistics per subject (id, total_episodes, avg_ep_per_day)
#'   \item \code{events_detailed}: Tibble with detailed event information (id, start_time, start_glucose, end_time, end_glucose, start_index, end_index). End fields report the last dysglycemic reading before confirmed recovery starts. \code{start_index} and \code{end_index} are 1-based row positions in the returned \code{interpolated_data}.
#'   \item \code{interpolated_data}: Included by default with columns
#'     \code{id}, \code{time}, and \code{gl}; set
#'     \code{return_interpolated = FALSE} to omit it.
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Level 1 Hyperglycemia Event (>=15 consecutive min of >180 mg/dL and event
#' # ends when there is >=15 consecutive min with a CGM sensor value of <=180 mg/dL)
#' hyper_lv1 <- detect_hyperglycemic_events(example_data_5_subject, type = "lv1")
#' print(hyper_lv1$events_total)
#'
#' # Level 2 Hyperglycemia Event (>=15 consecutive min of >250 mg/dL and event
#' # ends when there is >=15 consecutive min with a CGM sensor value of <=250 mg/dL)
#' hyper_lv2 <- detect_hyperglycemic_events(example_data_5_subject, type = "lv2")
#' print(hyper_lv2$events_total)
#'
#' # Extended Hyperglycemia Event (>250 mg/dL lasting >=90 cumulative min within a
#' # 120-min period, ends when glucose returns to <=180 mg/dL for >=15 consecutive
#' # min after)
#' hyper_extended <- detect_hyperglycemic_events(example_data_5_subject, type = "extended")
#' print(hyper_extended$events_total)
#'
#' # Custom criteria method for the same standard definitions
#' hyper_lv1_custom <- detect_hyperglycemic_events(
#'   example_data_5_subject,
#'   start_gl = 180,
#'   dur_length = 15,
#'   end_length = 15,
#'   end_gl = 180
#' )
#' hyper_lv2_custom <- detect_hyperglycemic_events(
#'   example_data_5_subject,
#'   start_gl = 250,
#'   dur_length = 15,
#'   end_length = 15,
#'   end_gl = 250
#' )
#' hyper_extended_custom <- detect_hyperglycemic_events(
#'   example_data_5_subject,
#'   start_gl = 250,
#'   dur_length = 120,
#'   end_length = 15,
#'   end_gl = 180
#' )
#'
#' # Compare event rates across levels
#' cat("Level 1 episodes:", sum(hyper_lv1$events_total$total_episodes), "\n")
#' cat("Level 2 episodes:", sum(hyper_lv2$events_total$total_episodes), "\n")
#' cat("Extended episodes:", sum(hyper_extended$events_total$total_episodes), "\n")
#'
#' # Analysis on larger dataset with Level 1 criteria
#' large_hyper <- detect_hyperglycemic_events(example_data_hall, type = "lv1")
#' print(large_hyper$events_total)
#'
#' # Analysis on larger dataset with Level 2 criteria
#' large_hyper_lv2 <- detect_hyperglycemic_events(example_data_hall, type = "lv2")
#' print(large_hyper_lv2$events_total)
#'
#' # Analysis on larger dataset with Extended criteria
#' large_hyper_extended <- detect_hyperglycemic_events(example_data_hall, type = "extended")
#' print(large_hyper_extended$events_total)
#'
#' # View detailed events for specific subject
#' if(nrow(hyper_lv1$events_detailed) > 0) {
#'   first_subject <- hyper_lv1$events_detailed$id[1]
#'   subject_events <- hyper_lv1$events_detailed[hyper_lv1$events_detailed$id == first_subject, ]
#'   head(subject_events)
#' }
NULL

#' @title Detect Hypoglycemic Events
#' @name detect_hypoglycemic_events
#' @encoding UTF-8
#' @description
#' Identifies and segments hypoglycemic events in CGM data based on international consensus
#' CGM metrics (Battelino et al., 2023). Use \code{type} to select one of
#' three event definitions:
#' \itemize{
#'   \item \strong{Level 1}: \eqn{\geq} 15 consecutive min of \eqn{<} 70 mg/dL, ends with \eqn{\geq} 15 consecutive min \eqn{\geq} 70 mg/dL
#'   \item \strong{Level 2}: \eqn{\geq} 15 consecutive min of \eqn{<} 54 mg/dL, ends with \eqn{\geq} 15 consecutive min \eqn{\geq} 54 mg/dL
#'   \item \strong{Extended}: \eqn{>} 120 consecutive min of \eqn{<} 70 mg/dL, ends with \eqn{\geq} 15 consecutive min \eqn{\geq} 70 mg/dL
#' }
#' Events are counted only after glucose remains at or above the recovery
#' threshold for the specified end length. In \code{events_detailed},
#' \code{end_time}, \code{end_glucose}, and \code{end_index} report the last
#' hypoglycemic reading immediately before that confirmed recovery period starts.
#'
#' @references
#' Battelino, T., et al. (2023). Continuous glucose monitoring and metrics for clinical trials: an international consensus statement. The Lancet Diabetes & Endocrinology, 11(1), 42-57.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param ... Custom event criteria supplied by name. Prefer \code{type} for
#'   standard Level 1, Level 2, and Extended events. Supported custom criteria
#'   are:
#'   \itemize{
#'     \item \code{dur_length}: Minimum event duration in minutes required for
#'       an event to qualify.
#'     \item \code{end_length}: Required recovery duration in minutes before
#'       event termination is confirmed.
#'     \item \code{start_gl}: Glucose threshold in mg/dL used to qualify
#'       hypoglycemic readings. Hypoglycemic readings are below this value, and
#'       recovery is confirmed after glucose remains at or above this value for
#'       \code{end_length} minutes.
#'   }
#' @param type Hypoglycemia event definition. One of \code{"extended"}
#'   (default), \code{"lv1"}, \code{"lv2"}, or \code{"lv1_excl"}.
#' @param reading_minutes Time interval between readings in minutes (optional).
#'   If omitted or \code{NULL}, the interval is calculated automatically per id
#'   as the median positive time difference in the data.
#' @param sort_time Logical. If \code{TRUE}, sort rows within each id by
#'   \code{time} in C++ before interpolation. Defaults to \code{FALSE}.
#' @param inter_gap Maximum gap in minutes to interpolate across. Defaults to
#'   45; larger gaps split event-detection segments.
#' @param return_interpolated Logical. If \code{TRUE}, include the interpolated
#'   grid data used for event detection in the returned list. Defaults to
#'   \code{TRUE}.
#' @usage detect_hypoglycemic_events(df, ..., type = "extended",
#'  reading_minutes = NULL, sort_time = FALSE, inter_gap = 45,
#'  return_interpolated = TRUE)
#' @section Methods:
#' Hypoglycemic events can be detected using either the recommended
#' \code{type} argument or named custom threshold and duration criteria.
#'
#' \strong{1. Preset method using \code{type} (recommended):}
#' Use \code{type} when you want the standard Level 1, Level 2, or Extended
#' hypoglycemia definitions without manually entering thresholds.
#' \itemize{
#'   \item \code{type = "lv1"} uses \code{start_gl = 70},
#'     \code{dur_length = 15}, and \code{end_length = 15}.
#'   \item \code{type = "lv2"} uses \code{start_gl = 54},
#'     \code{dur_length = 15}, and \code{end_length = 15}.
#'   \item \code{type = "extended"} uses \code{start_gl = 70},
#'     \code{dur_length = 120}, and \code{end_length = 15}.
#'   \item \code{type = "lv1_excl"} returns Level 1 episodes that do not
#'     overlap Level 2 episodes.
#' }
#'
#' \strong{2. Custom criteria method:}
#' Supply \code{start_gl}, \code{dur_length}, and \code{end_length} directly
#' when using a custom definition, for example
#' \code{detect_hypoglycemic_events(df, start_gl = 70, dur_length = 15,
#' end_length = 15)} for Level 1 hypoglycemia. If an explicit \code{type} is
#' supplied together with custom numeric criteria, the function returns results
#' based on \code{type}; the custom criteria are ignored and a warning is
#' issued.
#' @section Units and sampling:
#' - \code{reading_minutes} can be a scalar (all rows) or a vector per-row.
#' - If \code{reading_minutes} is omitted or \code{NULL}, it is calculated
#'   automatically per id from timestamp spacing.
#' - Event classification uses cgmguru's independent C++ implementation of an
#'   iglu-compatible, midnight-aligned full-day grid. Data are linearly
#'   interpolated at the id-specific interval up to \code{inter_gap}; larger
#'   gaps are masked, removed from the event-classification data, and split
#'   segments.
#' - This preprocessing is specific to event calculation and does not affect
#'   \code{\link{grid}}, \code{\link{maxima_grid}}, or \code{\link{excursion}}.
#' @seealso \link{detect_all_events}
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{events_total}: Tibble with summary statistics per subject (id, total_episodes, avg_ep_per_day)
#'   \item \code{events_detailed}: Tibble with detailed event information (id, start_time, start_glucose, end_time, end_glucose, start_index, end_index, duration_below_54_minutes). End fields report the last dysglycemic reading before confirmed recovery starts. \code{start_index} and \code{end_index} are 1-based row positions in the returned \code{interpolated_data}.
#'   \item \code{interpolated_data}: Included by default with columns
#'     \code{id}, \code{time}, and \code{gl}; set
#'     \code{return_interpolated = FALSE} to omit it.
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Level 1 Hypoglycemia Event (>=15 consecutive min of <70 mg/dL and event
#' # ends when there is >=15 consecutive min with a CGM sensor value of >=70 mg/dL)
#' hypo_lv1 <- detect_hypoglycemic_events(example_data_5_subject, type = "lv1")
#' print(hypo_lv1$events_total)
#'
#' # Level 2 Hypoglycemia Event (>=15 consecutive min of <54 mg/dL and event
#' # ends when there is >=15 consecutive min with a CGM sensor value of >=54 mg/dL)
#' hypo_lv2 <- detect_hypoglycemic_events(example_data_5_subject, type = "lv2")
#'
#' # Extended Hypoglycemia Event (>120 consecutive min of <70 mg/dL and event
#' # ends when there is >=15 consecutive min with a CGM sensor value of >=70 mg/dL)
#' hypo_extended <- detect_hypoglycemic_events(example_data_5_subject, type = "extended")
#' print(hypo_extended$events_total)
#'
#' # Custom criteria method for the same standard definitions
#' hypo_lv1_custom <- detect_hypoglycemic_events(
#'   example_data_5_subject,
#'   start_gl = 70,
#'   dur_length = 15,
#'   end_length = 15
#' )
#' hypo_lv2_custom <- detect_hypoglycemic_events(
#'   example_data_5_subject,
#'   start_gl = 54,
#'   dur_length = 15,
#'   end_length = 15
#' )
#' hypo_extended_custom <- detect_hypoglycemic_events(
#'   example_data_5_subject,
#'   start_gl = 70,
#'   dur_length = 120,
#'   end_length = 15
#' )
#'
#' # Compare event rates across levels
#' cat("Level 1 episodes:", sum(hypo_lv1$events_total$total_episodes), "\n")
#' cat("Level 2 episodes:", sum(hypo_lv2$events_total$total_episodes), "\n")
#' cat("Extended episodes:", sum(hypo_extended$events_total$total_episodes), "\n")
#'
#' # Analysis on larger dataset with Level 1 criteria
#' large_hypo <- detect_hypoglycemic_events(example_data_hall, type = "lv1")
#' print(large_hypo$events_total)
#'
#' # Analysis on larger dataset with Level 2 criteria
#' large_hypo_lv2 <- detect_hypoglycemic_events(example_data_hall, type = "lv2")
#' print(large_hypo_lv2$events_total)
#'
#' # Analysis on larger dataset with Extended criteria
#' large_hypo_extended <- detect_hypoglycemic_events(example_data_hall, type = "extended")
#' print(large_hypo_extended$events_total)
NULL

#' @title Detect All Glycemic Events
#' @name detect_all_events
#' @description
#' Comprehensive function to detect all types of glycemic events aligned with
#' international consensus CGM metrics (Battelino et al., 2023). This function
#' provides a unified interface for detecting multiple event types including
#' Level 1/2/Extended hypo- and hyperglycemia, and Level 1 excluded events.
#' Events are counted only after the required recovery condition is confirmed;
#' duration summaries use the event boundary immediately before recovery starts.
#' Event preprocessing uses cgmguru's independent C++ implementation of an
#' iglu-compatible day-based grid: each subject is interpolated from the first
#' observed day's midnight plus one reading interval, rather than from the first
#' observed timestamp. Larger gaps are masked and removed before event
#' classification, preserving gap-based segment boundaries. This preprocessing
#' is specific to event calculation and does not affect \code{\link{grid}},
#' \code{\link{maxima_grid}}, or \code{\link{excursion}}.
#'
#' @references
#' Battelino, T., et al. (2023). Continuous glucose monitoring and metrics for clinical trials: an international consensus statement. The Lancet Diabetes & Endocrinology, 11(1), 42-57.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param reading_minutes Time interval between readings in minutes (optional).
#'   Can be a single integer/numeric value (applied to all subjects), a vector
#'   matching data length, or \code{NULL}. If omitted or \code{NULL}, the
#'   interval is calculated automatically per id as the median positive time
#'   difference in the data.
#' @param sort_time Logical. If \code{TRUE}, sort rows within each id by
#'   \code{time} in C++ before interpolation. Defaults to \code{FALSE}.
#' @param inter_gap Maximum gap in minutes to interpolate across. Defaults to
#'   45; larger gaps split event-detection segments.
#' @param return_interpolated Logical. Retained for backward compatibility and
#'   ignored; \code{detect_all_events()} does not return interpolated data.
#' @usage detect_all_events(df, reading_minutes = NULL, sort_time = FALSE,
#'  inter_gap = 45, return_interpolated = FALSE)
#' @section Event types:
#' - Hypoglycemia: lv1 (\eqn{<} 70 mg/dL, \eqn{\geq} 15 min), lv2 (\eqn{<} 54 mg/dL, \eqn{\geq} 15 min), extended (\eqn{<} 70 mg/dL, \eqn{\geq} 120 min).
#' - Hyperglycemia: lv1 (\eqn{>} 180 mg/dL, \eqn{\geq} 15 min), lv2 (\eqn{>} 250 mg/dL, \eqn{\geq} 15 min), extended (\eqn{>} 250 mg/dL, \eqn{\geq} 90 min in 120 min, end \eqn{\leq} 180 mg/dL for \eqn{\geq} 15 min).
#' @seealso \link{detect_hyperglycemic_events}, \link{detect_hypoglycemic_events}
#'
#' @return A list with two tibbles:
#' \itemize{
#'   \item \code{events_long_df}: One row per subject, event type, and event
#'     level. Contains the full event summary: \code{id}, \code{type},
#'     \code{level}, \code{total_episodes}, \code{avg_ep_per_day}, and
#'     \code{avg_episode_duration_below_54}.
#'   \item \code{summary_df}: One row per subject. CGM summary metric columns
#'     are calculated on the interpolated glucose grid, and event summaries are
#'     included as wide \code{*_event_count} columns only.
#' }
#' \code{summary_df} includes:
#' \itemize{
#'   \item \code{id}: Subject identifier
#'   \item \code{TIR}: Time in range 70-180 mg/dL, percent
#'   \item \code{TITR}: Time in tight range 70-140 mg/dL, percent
#'   \item \code{TBR70}, \code{TBR54}: Time below 70 and 54 mg/dL, percent
#'   \item \code{TAR180}, \code{TAR250}: Time above 180 and 250 mg/dL, percent
#'   \item \code{CV}: Glucose standard deviation divided by mean glucose
#'   \item \code{SD}: Glucose standard deviation
#'   \item \code{mean_glucose}: Mean glucose in mg/dL
#'   \item \code{GMI}: \eqn{3.31 + 0.02392 * mean_glucose}
#'   \item \code{uGMI}: \eqn{1 / (15.36 / mean_glucose + 0.0425)}
#'   \item \code{GRI}: Glycemia Risk Index,
#'     \eqn{3.0 * VLow + 2.4 * Low + 1.6 * VHigh + 0.8 * High}, where
#'     \code{VLow} is percent time \eqn{<}54 mg/dL, \code{Low} is 54-\eqn{<}70
#'     mg/dL, \code{VHigh} is \eqn{>}250 mg/dL, and \code{High} is
#'     \eqn{>}180-\eqn{\leq}250 mg/dL
#'   \item \code{sensor_wear}: Percent of expected CGM readings observed,
#'     calculated from the original timestamps using the same automatic range
#'     method as \code{iglu::active_percent()}
#' }
#' Event counts are returned in wide columns for each hypo/hyper level
#' combination, for example \code{hypo_lv1_event_count}. The same pattern is
#' repeated for \code{lv1}, \code{lv2}, \code{extended}, and \code{lv1_excl}
#' within both \code{hypo} and \code{hyper}.
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Detect all glycemic events; reading_minutes is calculated automatically
#' # from the timestamp spacing when omitted
#' all_outputs <- detect_all_events(example_data_5_subject)
#' print(all_outputs$summary_df)
#' print(all_outputs$events_long_df)
#'
#' # Detect all events on larger dataset
#' large_outputs <- detect_all_events(example_data_hall)
#' print(paste("Total subjects analyzed:", nrow(large_outputs$summary_df)))
NULL

#' @title Find Local Maxima in Glucose Time Series
#' @name find_local_maxima
#' @description
#' Identifies local maxima (peaks) in glucose concentration time series data.
#' Uses a difference-based algorithm to detect peaks where glucose values
#' increase or remain constant for two consecutive points before the peak point,
#' and decrease or remain constant for two consecutive points after the peak point.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @usage find_local_maxima(df)
#' @seealso \link{grid}, \link{mod_grid}, \link{find_new_maxima}
#' @family GRID pipeline
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{local_maxima_vector}: Tibble with R-based (1-indexed) row numbers of local maxima (\code{local_maxima}). The corresponding occurrence time is \code{df$time[local_maxima]} and glucose is \code{df$gl[local_maxima]}.
#'   \item \code{merged_results}: Tibble with local maxima details (\code{id}, \code{time}, \code{gl})
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Find local maxima
#' maxima_result <- find_local_maxima(example_data_5_subject)
#' print(paste("Found", nrow(maxima_result$local_maxima_vector), "local maxima"))
#'
#' # Find maxima on larger dataset
#' large_maxima <- find_local_maxima(example_data_hall)
#' print(paste("Found", nrow(large_maxima$local_maxima_vector), "local maxima in larger dataset"))
#'
#' # View first few maxima
#' head(maxima_result$local_maxima_vector)
#'
#' # View merged results
#' head(maxima_result$merged_results)
NULL

#' @title Find Maximum Glucose After Specified Hours
#' @name find_max_after_hours
#' @description
#' Identifies the maximum glucose value occurring within a specified time window
#' after a given start point. This function is useful for analyzing glucose
#' patterns following specific events or time points.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param start_point_df A dataframe with column \code{start_index} (R-based index into \code{df})
#' @param hours Number of hours to look ahead from the start point
#' @usage find_max_after_hours(df, start_point_df, hours)
#' @section Notes:
#' - \code{start_index} must be valid row numbers in \code{df} (1-indexed).
#' - The search window is (0, \code{hours}] hours after each start index.
#' @seealso \link{mod_grid}, \link{find_local_maxima}, \link{find_new_maxima}, \link{transform_df}
#' @family GRID pipeline
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{max_index}: Tibble with R-based (1-indexed) row numbers of maximum glucose (\code{max_index}).
#'     The corresponding occurrence time is \code{df$time[max_index]} and glucose is \code{df$gl[max_index]}.
#'   \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
#'   \item \code{episode_start}: Tibble with all episode starts with columns:
#'     \itemize{
#'       \item \code{id}: Subject identifier
#'       \item \code{time}: Timestamp at which the maximum occurs; equivalent to \code{df$time[index]}
#'       \item \code{gl}: Glucose value at the maximum; equivalent to \code{df$gl[index]}
#'       \item \code{index}: R-based (1-indexed) row number(s) in \code{df} denoting where the maximum occurs
#'     }
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Create start points for demonstration (using row index)
#' start_index <- seq(1, nrow(example_data_5_subject), by = 100)
#' start_points <- data.frame(start_index = start_index)
#'
#' # Find maximum glucose in next 2 hours
#' max_after <- find_max_after_hours(example_data_5_subject, start_points, hours = 2)
#' print(paste("Found", length(max_after$max_index), "maximum points"))
#'
#' # Find maximum glucose in next 1 hour
#' max_after_1h <- find_max_after_hours(example_data_5_subject, start_points, hours = 1)
#'
#' # Analysis on larger dataset
#' large_start_index <- seq(1, nrow(example_data_hall), by = 200)
#' large_start_points <- data.frame(start_index = large_start_index)
#' large_max_after <- find_max_after_hours(example_data_hall, large_start_points, hours = 2)
#' print(paste("Found", length(large_max_after$max_index), "maximum points in larger dataset"))
NULL

#' @title Find Maximum Glucose Before Specified Hours
#' @name find_max_before_hours
#' @description
#' Identifies the maximum glucose value occurring within a specified time window
#' before a given start point. This function is useful for analyzing glucose
#' patterns preceding specific events or time points.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param start_point_df A dataframe with column \code{start_index} (R-based index into \code{df})
#' @param hours Number of hours to look back from the start point
#' @usage find_max_before_hours(df, start_point_df, hours)
#' @section Notes:
#' - The search window is [\code{hours}, 0) hours before each start index.
#' @seealso \link{mod_grid}, \link{find_local_maxima}, \link{find_new_maxima}
#' @family GRID pipeline
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{max_index}: Tibble with R-based (1-indexed) row numbers of maximum glucose (\code{max_index}).
#'     The corresponding occurrence time is \code{df$time[max_index]} and glucose is \code{df$gl[max_index]}.
#'   \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
#'   \item \code{episode_start}: Tibble with all episode starts with columns:
#'     \itemize{
#'       \item \code{id}: Subject identifier
#'       \item \code{time}: Timestamp at which the maximum occurs; equivalent to \code{df$time[index]}
#'       \item \code{gl}: Glucose value at the maximum; equivalent to \code{df$gl[index]}
#'       \item \code{index}: R-based (1-indexed) row number(s) in \code{df} denoting where the maximum occurs
#'     }
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Create start points for demonstration (using row index)
#' start_index <- seq(1, nrow(example_data_5_subject), by = 100)
#' start_points <- data.frame(start_index = start_index)
#'
#' # Find maximum glucose in previous 2 hours
#' max_before <- find_max_before_hours(example_data_5_subject, start_points, hours = 2)
#' print(paste("Found", length(max_before$max_index), "maximum points"))
#'
#' # Find maximum glucose in previous 1 hour
#' max_before_1h <- find_max_before_hours(example_data_5_subject, start_points, hours = 1)
#'
#' # Analysis on larger dataset
#' large_start_index <- seq(1, nrow(example_data_hall), by = 200)
#' large_start_points <- data.frame(start_index = large_start_index)
#' large_max_before <- find_max_before_hours(example_data_hall, large_start_points, hours = 2)
#' print(paste("Found", length(large_max_before$max_index), "maximum points in larger dataset"))
NULL

#' @title Find Minimum Glucose After Specified Hours
#' @name find_min_after_hours
#' @description
#' Identifies the minimum glucose value occurring within a specified time window
#' after a given start point. This function is useful for analyzing glucose
#' patterns following specific events or time points.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param start_point_df A dataframe with column \code{start_index} (R-based index into \code{df})
#' @param hours Number of hours to look ahead from the start point
#' @usage find_min_after_hours(df, start_point_df, hours)
#' @seealso \link{mod_grid}, \link{find_local_maxima}
#' @family GRID pipeline
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{min_index}: Tibble with R-based (1-indexed) row numbers of minimum glucose (\code{min_index}). The corresponding occurrence time is \code{df$time[min_index]} and glucose is \code{df$gl[min_index]}.
#'   \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
#'   \item \code{episode_start}: Tibble with all episode starts with columns:
#'     \itemize{
#'       \item \code{id}: Subject identifier
#'       \item \code{time}: Timestamp at which the minimum occurs; equivalent to \code{df$time[index]}
#'       \item \code{gl}: Glucose value at the minimum; equivalent to \code{df$gl[index]}
#'       \item \code{index}: R-based (1-indexed) row number(s) in \code{df} denoting where the minimum occurs
#'     }
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Create start points for demonstration (using row index)
#' start_index <- seq(1, nrow(example_data_5_subject), by = 100)
#' start_points <- data.frame(start_index = start_index)
#'
#' # Find minimum glucose in next 2 hours
#' min_after <- find_min_after_hours(example_data_5_subject, start_points, hours = 2)
#' print(paste("Found", length(min_after$min_index), "minimum points"))
#'
#' # Find minimum glucose in next 1 hour
#' min_after_1h <- find_min_after_hours(example_data_5_subject, start_points, hours = 1)
#'
#' # Analysis on larger dataset
#' large_start_index <- seq(1, nrow(example_data_hall), by = 200)
#' large_start_points <- data.frame(start_index = large_start_index)
#' large_min_after <- find_min_after_hours(example_data_hall, large_start_points, hours = 2)
#' print(paste("Found", length(large_min_after$min_index), "minimum points in larger dataset"))
NULL

#' @title Find Minimum Glucose Before Specified Hours
#' @name find_min_before_hours
#' @description
#' Identifies the minimum glucose value occurring within a specified time window
#' before a given start point. This function is useful for analyzing glucose
#' patterns preceding specific events or time points.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param start_point_df A dataframe with column \code{start_index} (R-based index into \code{df})
#' @param hours Number of hours to look back from the start point
#' @usage find_min_before_hours(df, start_point_df, hours)
#' @seealso \link{mod_grid}, \link{find_local_maxima}
#' @family GRID pipeline
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{min_index}: Tibble with R-based (1-indexed) row numbers of minimum glucose (\code{min_index}). The corresponding occurrence time is \code{df$time[min_index]} and glucose is \code{df$gl[min_index]}.
#'   \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
#'   \item \code{episode_start}: Tibble with all episode starts with columns:
#'     \itemize{
#'       \item \code{id}: Subject identifier
#'       \item \code{time}: Timestamp at which the minimum occurs; equivalent to \code{df$time[index]}
#'       \item \code{gl}: Glucose value at the minimum; equivalent to \code{df$gl[index]}
#'       \item \code{index}: R-based (1-indexed) row number(s) in \code{df} denoting where the minimum occurs
#'     }
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Create start points for demonstration (using row index)
#' start_index <- seq(1, nrow(example_data_5_subject), by = 100)
#' start_points <- data.frame(start_index = start_index)
#'
#' # Find minimum glucose in previous 2 hours
#' min_before <- find_min_before_hours(example_data_5_subject, start_points, hours = 2)
#' print(paste("Found", length(min_before$min_index), "minimum points"))
#'
#' # Find minimum glucose in previous 1 hour
#' min_before_1h <- find_min_before_hours(example_data_5_subject, start_points, hours = 1)
#'
#' # Analysis on larger dataset
#' large_start_index <- seq(1, nrow(example_data_hall), by = 200)
#' large_start_points <- data.frame(start_index = large_start_index)
#' large_min_before <- find_min_before_hours(example_data_hall, large_start_points, hours = 2)
#' print(paste("Found", length(large_min_before$min_index), "minimum points in larger dataset"))
NULL

#' @title Find New Maxima Around Grid Points
#' @name find_new_maxima
#' @description
#' Identifies new maxima in the vicinity of previously identified grid points,
#' useful for refining maxima detection in GRID analysis. This function helps
#' improve the accuracy of peak detection by searching around known event points.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param mod_grid_max_point_df A dataframe with column \code{index} (candidate maxima index)
#' @param local_maxima_df A dataframe with column \code{local_maxima} (index of local peaks)
#' @usage find_new_maxima(df, mod_grid_max_point_df, local_maxima_df)
#' @seealso \link{find_local_maxima}, \link{find_max_after_hours}, \link{transform_df}
#' @family GRID pipeline
#'
#' @return A tibble with updated maxima information containing columns (\code{id}, \code{time}, \code{gl}, \code{index})
#' The \code{index} column contains R-based (1-indexed) row number(s) in \code{df}; thus, \code{time == df$time[index]} and \code{gl == df$gl[index]}.
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # First, get grid points and local maxima
#' grid_result <- grid(example_data_5_subject, gap = 15, threshold = 130)
#' maxima_result <- find_local_maxima(example_data_5_subject)
#'
#' # Create modified grid points (simplified for example)
#' mod_grid_indices <- data.frame(index = grid_result$episode_start$index[1:10])
#'
#' # Find new maxima around grid points
#' new_maxima <- find_new_maxima(example_data_5_subject,
#'                               mod_grid_indices,
#'                               maxima_result$local_maxima_vector)
#' print(paste("Found", nrow(new_maxima), "new maxima"))
#'
#' # Analysis on larger dataset
#' large_grid <- grid(example_data_hall, gap = 15, threshold = 130)
#' large_maxima <- find_local_maxima(example_data_hall)
#' large_mod_grid <- data.frame(index = large_grid$episode_start$index[1:20])
#' large_new_maxima <- find_new_maxima(example_data_hall,
#'                                     large_mod_grid,
#'                                     large_maxima$local_maxima_vector)
#' print(paste("Found", nrow(large_new_maxima), "new maxima in larger dataset"))
NULL

#' @title Modified GRID Analysis
#' @name mod_grid
#' @description
#' Constructs a modified GRID series by reapplying the GRID logic with a designated
#' gap (e.g., 60 minutes) and analysis window in hours (e.g., 2 hours). It
#' reassigns GRID events under these constraints to produce a modified grid
#' suitable for downstream maxima mapping and episode analysis.
#'
#' @references
#' Park, Sang Ho, et al. "Identification of clinically meaningful automatically detected postprandial glucose excursions in individuals with type 1 diabetes using personal continuous glucose monitoring." Diabetes Research and Clinical Practice (2025): 112951.
#'
#' Park, Soojin, et al. "High-Amplitude and Prolonged Glucose Excursions as a Key Determinant of Discordance Between Glucose Management Indicator and Glycated Hemoglobin in Type 1 Diabetes." Diabetes Care (2026): dc252820. https://doi.org/10.2337/dc25-2820
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param grid_point_df A dataframe with column \code{start_index} (start points for re-applied GRID)
#' @param hours Time window in hours for analysis (default: 2)
#' @param gap Gap threshold in minutes for event detection (default: 15).
#'   This parameter defines the minimum time interval between consecutive GRID events.
#' @usage mod_grid(df, grid_point_df, hours = 2, gap = 15)
#' @section Units and sampling:
#' - \code{gap} is minutes; \code{hours} is hours; \code{time} is POSIXct.
#' @seealso \link{grid}, \link{find_max_after_hours}, \link{find_new_maxima}
#' @family GRID pipeline
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{mod_grid_vector}: Tibble with modified GRID results (\code{mod_grid})
#'   \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
#'   \item \code{episode_start}: Tibble with all episode starts with columns:
#'     \itemize{
#'       \item \code{id}: Subject identifier
#'       \item \code{time}: Timestamp at which the event occurs; equivalent to \code{df$time[index]}
#'       \item \code{gl}: Glucose value at the event; equivalent to \code{df$gl[index]}
#'       \item \code{index}: R-based (1-indexed) row number(s) in \code{df} denoting where the event occurs
#'     }
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # First, get grid points
#' grid_result <- grid(example_data_5_subject, gap = 60, threshold = 130)
#'
#' # Perform modified GRID analysis
#' mod_result <- mod_grid(example_data_5_subject, grid_result$grid_vector, hours = 2, gap = 60)
#' print(paste("Modified grid points:", nrow(mod_result$mod_grid_vector)))
#'
#' # Modified analysis with different parameters
#' mod_result_1h <- mod_grid(example_data_5_subject, grid_result$grid_vector, hours = 1, gap = 40)
#'
#' # Analysis on larger dataset
#' large_grid <- grid(example_data_hall, gap = 60, threshold = 130)
#' large_mod_result <- mod_grid(example_data_hall, large_grid$grid_vector, hours = 2, gap = 60)
#' print(paste("Modified grid points in larger dataset:", nrow(large_mod_result$mod_grid_vector)))
NULL

#' @title Detect Events Between Maxima
#' @name detect_between_maxima
#' @description
#' Identifies and analyzes events occurring between detected maxima points,
#' providing detailed episode information for GRID analysis. This function
#' helps characterize the glucose dynamics between identified peaks.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param transform_df A dataframe containing summary information from previous transformations
#' @usage detect_between_maxima(df, transform_df)
#' @seealso \link{grid}, \link{mod_grid}, \link{find_new_maxima}, \link{transform_df}
#' @family GRID pipeline
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{results}: Tibble with events between maxima (\code{id}, \code{grid_time}, \code{grid_gl}, \code{maxima_time}, \code{maxima_glucose}, \code{time_to_peak})
#'   \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Complete pipeline to get transform_df
#' grid_result <- grid(example_data_5_subject, gap = 60, threshold = 130)
#' maxima_result <- find_local_maxima(example_data_5_subject)
#' mod_result <- mod_grid(example_data_5_subject, grid_result$grid_vector, hours = 2, gap = 60)
#' max_after <- find_max_after_hours(example_data_5_subject, mod_result$mod_grid_vector, hours = 2)
#' new_maxima <- find_new_maxima(example_data_5_subject,
#'                               max_after$max_index,
#'                               maxima_result$local_maxima_vector)
#' transformed <- transform_df(grid_result$episode_start, new_maxima)
#'
#' # Detect events between maxima
#' between_events <- detect_between_maxima(example_data_5_subject, transformed)
#' print(paste("Events between maxima:", length(between_events)))
#'
#' # Analysis on larger dataset
#' large_grid <- grid(example_data_hall, gap = 60, threshold = 130)
#' large_maxima <- find_local_maxima(example_data_hall)
#' large_mod <- mod_grid(example_data_hall, large_grid$grid_vector, hours = 2, gap = 60)
#' large_max_after <- find_max_after_hours(example_data_hall, large_mod$mod_grid_vector, hours = 2)
#' large_new_maxima <- find_new_maxima(example_data_hall,
#'                                     large_max_after$max_index,
#'                                     large_maxima$local_maxima_vector)
#' large_transformed <- transform_df(large_grid$episode_start, large_new_maxima)
#' large_between <- detect_between_maxima(example_data_hall, large_transformed)
#' print(paste("Events between maxima in larger dataset:", length(large_between)))
NULL

#' @title Calculate Glucose Excursions
#' @name excursion
#' @description
#' Calculates glucose excursions in CGM data. An excursion is defined as
#' a \eqn{>} 70 mg/dL (\eqn{>} 3.9 mmol/L) rise within 2 hours, not preceded by a value
#' \eqn{<} 70 mg/dL (\eqn{<} 3.9 mmol/L).
#'
#' @references
#' Edwards, S., et al. (2022). Use of connected pen as a diagnostic tool to evaluate missed bolus dosing behavior in people with type 1 and type 2 diabetes. Diabetes Technology & Therapeutics, 24(1), 61-66.
#'
#' @param df A dataframe containing continuous glucose monitoring (CGM) data.
#'   Must include columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier (string or factor)
#'     \item \code{time}: Time of measurement (POSIXct)
#'     \item \code{gl}: Glucose value (integer or numeric, mg/dL)
#'   }
#' @param gap Gap threshold in minutes for excursion calculation (default: 15).
#'   This parameter defines the minimum time interval between consecutive GRID events.
#' @usage excursion(df, gap = 15)
#' @section Notes:
#' - \code{gap} is minutes; change to enforce minimum separation between excursions.
#' - This function operates on the rows supplied in \code{df}. It does not use
#'   \code{\link{interpolate_cgm}} or the full-day event preprocessing grid.
#' @seealso \link{grid}
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{excursion_vector}: Tibble with excursion results (\code{excursion})
#'   \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
#'   \item \code{episode_start}: Tibble with all episode starts with columns:
#'     \itemize{
#'       \item \code{id}: Subject identifier
#'       \item \code{time}: Timestamp at which the event occurs; equivalent to \code{df$time[index]}
#'       \item \code{gl}: Glucose value at the event; equivalent to \code{df$gl[index]}
#'       \item \code{index}: R-based (1-indexed) row number(s) in \code{df} denoting where the event occurs
#'     }
#' }
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Calculate glucose excursions
#' excursion_result <- excursion(example_data_5_subject, gap = 15)
#' print(paste("Excursion vector length:", length(excursion_result$excursion_vector)))
#' print(excursion_result$episode_counts)
#'
#' # Excursion analysis with different gap
#' excursion_30min <- excursion(example_data_5_subject, gap = 30)
#'
#' # Analysis on larger dataset
#' large_excursion <- excursion(example_data_hall, gap = 15)
#' print(paste("Excursion vector length in larger dataset:", length(large_excursion$excursion_vector)))
#' print(paste("Total episodes:", sum(large_excursion$episode_counts$episode_counts)))
NULL

#' @title Find Start Points for Event Analysis
#' @name start_finder
#' @description
#' Finds R-based (1-indexed) positions where the value is 1 in an integer vector
#' of 0s and 1s, specifically identifying episode start points. This function
#' looks for positions where a 1 follows a 0 or is at the beginning of the vector,
#' which is useful for identifying the start of glycemic events or episodes.
#'
#' @param df A dataframe with the first column containing an integer vector of 0s and 1s
#' @usage start_finder(df)
#' @section Notes:
#' - Returns R-based \code{start_index} positions relative to the provided input vector/dataframe.
#' - If used on vectors derived from a CGM \code{df}, index map directly to \code{df} rows.
#' @seealso \link{grid}, \link{mod_grid}
#' @family GRID pipeline
#'
#' @return A tibble containing start_index with R-based (1-indexed) positions where episodes start
#' Note: These index refer to positions in the provided input vector/dataframe, not necessarily rows of the original CGM \code{df} unless that vector was derived directly from \code{df} in row order.
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Create a binary vector indicating episode starts
#' binary_vector <- c(0, 0, 1, 1, 0, 1, 0, 0, 1, 1)
#' df <- data.frame(episode_starts = binary_vector)
#'
#' # Find R-based index where episodes start
#' start_points <- start_finder(df)
#' print(paste("Start index:", paste(start_points$start_index, collapse = ", ")))
#'
#' # Use with actual GRID results
#' grid_result <- grid(example_data_5_subject, gap = 15, threshold = 130)
#' grid_starts <- start_finder(grid_result$grid_vector)
#' print(paste("GRID episode starts:", length(grid_starts$start_index)))
#'
#' # Analysis on larger dataset
#' large_grid <- grid(example_data_hall, gap = 15, threshold = 130)
#' large_starts <- start_finder(large_grid$grid_vector)
#' print(paste("GRID episode starts in larger dataset:", length(large_starts$start_index)))
NULL

#' @title Transform Dataframe for Analysis
#' @name transform_df
#' @description
#' Performs data transformations required for GRID analysis, including
#' mapping GRID episode starts to maxima within a 4-hour window and
#' merging grid and maxima information. This function prepares data
#' for downstream analysis by combining these results.
#'
#' @param grid_df A dataframe containing grid analysis results
#' @param maxima_df A dataframe containing maxima detection results
#' @usage transform_df(grid_df, maxima_df)
#' @seealso \link{grid}, \link{find_new_maxima}, \link{detect_between_maxima}
#' @family GRID pipeline
#'
#' @return A tibble with transformed data containing columns (\code{id}, \code{grid_time}, \code{grid_gl}, \code{maxima_time}, \code{maxima_gl})
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Complete pipeline example with smaller dataset
#' threshold <- 130
#' gap <- 60
#' hours <- 2
#' # 1) Find GRID points
#' grid_result <- grid(example_data_5_subject, gap = gap, threshold = threshold)

#' # 2) Find modified GRID points before 2 hours minimum
#' mod_grid <- mod_grid(example_data_5_subject,
#'                      start_finder(grid_result$grid_vector),
#'                      hours = hours,
#'                      gap = gap)
#'
#' # 3) Find maximum point 2 hours after mod_grid point
#' mod_grid_maxima <- find_max_after_hours(example_data_5_subject,
#'                                         start_finder(mod_grid$mod_grid_vector),
#'                                         hours = hours)
#'
#' # 4) Identify local maxima around episodes/windows
#' local_maxima <- find_local_maxima(example_data_5_subject)
#'
#' # 5) Among local maxima, find maximum point after two hours
#' final_maxima <- find_new_maxima(example_data_5_subject,
#'                                 mod_grid_maxima$max_index,
#'                                 local_maxima$local_maxima_vector)
#'
#' # 6) Map GRID points to maximum points (within 4 hours)
#' transform_maxima <- transform_df(grid_result$episode_start, final_maxima)
#'
#' # 7) Redistribute overlapping maxima between GRID points
#' final_between_maxima <- detect_between_maxima(example_data_5_subject, transform_maxima)

#' # Complete pipeline example with larger dataset (example_data_hall)
#' # This demonstrates the same workflow on a more comprehensive dataset
#' hall_threshold <- 130
#' hall_gap <- 60
#' hall_hours <- 2
#'
#' # 1) Find GRID points on larger dataset
#' hall_grid_result <- grid(example_data_hall, gap = hall_gap, threshold = hall_threshold)
#'
#' # 2) Find modified GRID points
#' hall_mod_grid <- mod_grid(example_data_hall,
#'                          start_finder(hall_grid_result$grid_vector),
#'                          hours = hall_hours,
#'                          gap = hall_gap)
#'
#' # 3) Find maximum points after mod_grid
#' hall_mod_grid_maxima <- find_max_after_hours(example_data_hall,
#'                                             start_finder(hall_mod_grid$mod_grid_vector),
#'                                             hours = hall_hours)
#'
#' # 4) Identify local maxima
#' hall_local_maxima <- find_local_maxima(example_data_hall)
#'
#' # 5) Find new maxima
#' hall_final_maxima <- find_new_maxima(example_data_hall,
#'                                     hall_mod_grid_maxima$max_index,
#'                                     hall_local_maxima$local_maxima_vector)
#'
#' # 6) Transform data
#' hall_transform_maxima <- transform_df(hall_grid_result$episode_start, hall_final_maxima)
#'
#' # 7) Detect between maxima
#' hall_final_between_maxima <- detect_between_maxima(example_data_hall, hall_transform_maxima)

NULL

#' @title Interpolate CGM Data
#' @name interpolate_cgm
#' @description
#' Interpolates continuous glucose monitoring (CGM) data onto the same
#' iglu-compatible, midnight-aligned full-day grid used internally by
#' cgmguru's event-detection functions. The interpolation is implemented in C++
#' and is intended for users who want to inspect or reuse the preprocessed grid
#' behind \code{\link{detect_all_events}},
#' \code{\link{detect_hyperglycemic_events}}, and
#' \code{\link{detect_hypoglycemic_events}}.
#'
#' For each subject, \code{interpolate_cgm()} builds an equally spaced grid at
#' \code{reading_minutes} intervals. If \code{reading_minutes} is omitted, it is
#' inferred per subject from the median positive timestamp difference. Glucose
#' values are linearly interpolated only across gaps up to \code{inter_gap};
#' larger gaps are treated as missing and removed from the returned data,
#' preserving segment boundaries used by event calculation.
#'
#' The GRID-family functions \code{\link{grid}}, \code{\link{maxima_grid}}, and
#' \code{\link{excursion}} do not call this helper automatically; they operate
#' on the rows supplied by the user unless the caller explicitly passes an
#' interpolated dataset.
#'
#' @param df A dataframe containing CGM data with columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier
#'     \item \code{time}: POSIXct measurement timestamp
#'     \item \code{gl}: Glucose value in mg/dL
#'   }
#' @param reading_minutes Time interval for the interpolation grid in minutes.
#'   If omitted or \code{NULL}, it is calculated automatically per id as the
#'   median positive time difference in the data.
#' @param sort_time Logical. If \code{TRUE}, sort rows within each id by
#'   \code{time} in C++ before interpolation. Defaults to \code{FALSE}.
#' @param inter_gap Maximum gap in minutes to interpolate across. Defaults to
#'   45; larger gaps split the event-detection grid.
#' @usage interpolate_cgm(df, reading_minutes = NULL, sort_time = FALSE,
#'  inter_gap = 45)
#' @return A tibble with columns \code{id}, interpolated \code{time}, and
#'   interpolated \code{gl}. Rows inside gaps larger than \code{inter_gap} are
#'   omitted.
#' @seealso \link{detect_all_events}, \link{detect_hyperglycemic_events},
#'   \link{detect_hypoglycemic_events}
#' @export
#' @examples
#' df <- data.frame(
#'   id = "A",
#'   time = as.POSIXct(c("2026-01-01 00:15:00", "2026-01-01 00:25:00"),
#'                     tz = "UTC"),
#'   gl = c(100, 120)
#' )
#' interpolate_cgm(df)
NULL

#' @title Calculate Sensor Wear
#' @name sensor_wear
#' @description
#' Calculates the percent of expected CGM readings observed during a fixed
#' retrospective window. This C++ implementation follows the manual-range logic
#' used by \code{iglu::active_percent(range_type = "manual")}: valid readings
#' in \code{[end_date - ndays, end_date]} are divided by the expected number of
#' readings over \code{ndays} days.
#'
#' If \code{end_date = NULL}, each subject's last valid timestamp defines that
#' subject's retrospective window. If \code{end_date} is supplied, the same
#' endpoint is used for all subjects, which is useful for a common study cutoff
#' or report date. Duplicate timestamps within a subject are de-duplicated after
#' sorting, and rows with missing \code{time} or \code{gl} do not count as
#' observed readings.
#'
#' @param df A dataframe containing CGM data with columns:
#'   \itemize{
#'     \item \code{id}: Subject identifier
#'     \item \code{time}: POSIXct measurement timestamp
#'     \item \code{gl}: Glucose value in mg/dL
#'   }
#' @param end_date End timestamp for the calculation window. If \code{NULL},
#'   each subject's last valid timestamp is used. \code{Date} values are
#'   converted with \code{as.POSIXct()}, matching iglu's manual active-percent
#'   behavior.
#' @param ndays Number of days in the retrospective window. Defaults to 14.
#' @param reading_minutes Reading interval in minutes. If \code{NULL}, it is
#'   inferred per id from the median positive difference between valid readings.
#' @usage sensor_wear(df, end_date = NULL, ndays = 14,
#'  reading_minutes = NULL)
#' @return A tibble with columns \code{id}, \code{sensor_wear}, \code{ndays},
#'   \code{start_date}, and \code{end_date}.
#' @seealso \link{detect_all_events}, \code{\link[iglu:active_percent]{iglu::active_percent}}
#' @export
#' @examples
#' library(iglu)
#' data(example_data_5_subject)
#' sensor_wear(example_data_5_subject, ndays = 14, reading_minutes = 5)
NULL

#' @title Fast Ordering Function
#' @name orderfast
#' @description
#' Orders a dataframe by \code{id} and \code{time} columns using a C++
#' \code{std::sort} backend. Optimized for large CGM datasets, it returns the
#' input with rows sorted by subject then timestamp while preserving all columns.
#'
#' @param df A dataframe with 'id' and 'time' columns
#' @return A dataframe ordered by id and time
#'
#' @export
#' @examples
#' # Load sample data
#' library(iglu)
#' data(example_data_5_subject)
#' data(example_data_hall)
#'
#' # Shuffle without replacement, then order and compare to baseline
#' set.seed(123)
#' shuffled <- example_data_5_subject[sample(seq_len(nrow(example_data_5_subject)),
#'                                           replace = FALSE), ]
#' baseline <- orderfast(example_data_5_subject)
#' ordered_shuffled <- orderfast(shuffled)
#'
#' # Compare results
#' print(paste("Identical after ordering:", identical(baseline, ordered_shuffled)))
#' head(baseline[, c("id", "time", "gl")])
#' head(ordered_shuffled[, c("id", "time", "gl")])
#'
#' # Order larger dataset
#' ordered_large <- orderfast(example_data_hall)
#' print(paste("Ordered", nrow(ordered_large), "rows in larger dataset"))
NULL

interpolate_cgm <- function(df, reading_minutes = NULL, sort_time = FALSE,
                            inter_gap = 45) {
  tryCatch({
    validated_df <- validate_cgm_data(df)
  }, error = function(e) {
    stop("Error in interpolate_cgm(): ", e$message, call. = FALSE)
  })

  reading_minutes <- validate_reading_minutes(reading_minutes, nrow(validated_df))
  sort_time <- validate_logical_param(sort_time, "sort_time")
  inter_gap <- validate_numeric_param(inter_gap, "inter_gap", min_val = 0.1)

  tryCatch({
    interpolate_cgm_cpp(validated_df, reading_minutes, sort_time, inter_gap)
  }, error = function(e) {
    stop("Error in interpolate_cgm: ", e$message, call. = FALSE)
  })
}

library(testthat)
library(cgmguru)

make_interp_cgm_at <- function(minutes, gl) {
  data.frame(
    id = "A",
    time = as.POSIXct("2026-01-01 00:15:00", tz = "UTC") + minutes * 60,
    gl = gl
  )
}

iglu_episode_grid <- function(df, dt0 = 5, inter_gap = 45, tz = "UTC") {
  data_ip <- iglu::CGMS2DayByDay(df, dt0 = dt0, inter_gap = inter_gap, tz = tz)
  n_points <- length(as.vector(t(data_ip$gd2d)))
  first_time <- as.POSIXct(data_ip$actual_dates[1], tz = tz) + data_ip$dt0 * 60
  out <- data.frame(
    id = df$id[1],
    time = seq(from = first_time, by = paste(data_ip$dt0, "mins"), length.out = n_points),
    gl = as.vector(t(data_ip$gd2d))
  )
  out[!is.na(out$gl), ]
}

test_that("interpolate_cgm returns the C++ interpolated grid", {
  df <- make_interp_cgm_at(c(0, 10, 15), c(60, 80, 90))

  out <- interpolate_cgm(df, reading_minutes = 5)
  interpolated_idx <- match(df$time[1] + 5 * 60, out$time)

  expect_s3_class(out, "data.frame")
  expect_named(out, c("id", "time", "gl"))
  expect_s3_class(out$time, "POSIXct")
  expect_equal(out$time[1], df$time[1])
  expect_false(is.na(interpolated_idx))
  expect_equal(out$gl[interpolated_idx], 70)
})

test_that("detect_all_events does not return interpolated data", {
	df <- make_interp_cgm_at(c(0, 10, 15, 20), c(60, 80, 82, 84))

	event_result <- detect_all_events(
		df,
		reading_minutes = 5,
		return_interpolated = TRUE
	)

	expect_named(event_result, c("events_long_df", "summary_df"))
	expect_false("interpolated_data" %in% names(event_result))
})

test_that("interpolate_cgm uses iglu midnight-aligned episode grid", {
  skip_if_not_installed("iglu")

  df <- data.frame(
    id = "A",
    time = as.POSIXct("2026-01-01 00:17:12", tz = "UTC") + c(0, 5, 10) * 60,
    gl = c(60, 80, 82)
  )

  standalone <- interpolate_cgm(df, reading_minutes = 5)
  expected <- iglu_episode_grid(df)

  expect_equal(standalone$time, expected$time)
  expect_equal(standalone$gl, expected$gl)
  expect_false(any(is.na(standalone$gl)))
})

test_that("interpolate_cgm respects sort_time and inter_gap gaps", {
  df <- make_interp_cgm_at(c(0, 50, 55), c(60, 100, 110))
  shuffled <- df[c(1, 3, 2), ]

  expect_error(
    interpolate_cgm(shuffled, reading_minutes = 5),
    "time must be nondecreasing"
  )

  out <- interpolate_cgm(
    shuffled,
    reading_minutes = 5,
    sort_time = TRUE,
    inter_gap = 45
  )

  expect_false(any(is.na(out$gl)))
  expect_named(out, c("id", "time", "gl"))
})

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Level 1 Hypoglycemia Event (≥15 consecutive min of <70 mg/dL)
hypo_lv1 <- detect_hypoglycemic_events(
  example_data_5_subject,
  type = "lv1"
)
print(hypo_lv1$events_total)

# Level 2 Hypoglycemia Event (≥15 consecutive min of <54 mg/dL)
hypo_lv2 <- detect_hypoglycemic_events(
  example_data_5_subject,
  type = "lv2"
)

# Extended Hypoglycemia Event (default parameters)
hypo_extended <- detect_hypoglycemic_events(example_data_5_subject, type = "extended")

# Analysis on larger dataset
large_hypo <- detect_hypoglycemic_events(
  example_data_hall,
  type = "lv1"
)
print(paste("Total hypoglycemic episodes:", sum(large_hypo$events_total$total_episodes)))

# Compare Level 1 vs Level 2 hypoglycemia
cat("Level 1 episodes:", sum(hypo_lv1$events_total$total_episodes), "\n")
cat("Level 2 episodes:", sum(hypo_lv2$events_total$total_episodes), "\n")

# Function overrides to provide safe versions of all C++ functions
# This file overrides the original function names with safe, validated versions

# Store original functions for potential internal use
.detect_hyperglycemic_events_original <- detect_hyperglycemic_events
.detect_hypoglycemic_events_original <- detect_hypoglycemic_events
.detect_all_events_original <- detect_all_events
.find_local_maxima_original <- find_local_maxima
.grid_original <- grid
.maxima_grid_original <- maxima_grid
.excursion_original <- excursion
# .start_finder_original <- start_finder  # Commented out since start_finder is overridden
.find_max_after_hours_original <- find_max_after_hours
.find_max_before_hours_original <- find_max_before_hours
.find_min_after_hours_original <- find_min_after_hours
.find_min_before_hours_original <- find_min_before_hours
.detect_between_maxima_original <- detect_between_maxima
.find_new_maxima_original <- find_new_maxima
.mod_grid_original <- mod_grid
.transform_df_original <- transform_df

# Override original functions with safe versions
detect_hyperglycemic_events <- function(df, ..., type = "extended", reading_minutes = NULL,
                                        sort_time = FALSE, inter_gap = 45,
                                        return_interpolated = TRUE) {
  type_provided <- !missing(type)
  reading_minutes_provided <- !missing(reading_minutes)
  old_args <- list(...)
  dur_length <- NULL
  end_length <- NULL
  start_gl <- NULL
  end_gl <- NULL
  lv1_excl <- FALSE

  if (length(old_args) > 0) {
    old_arg_names <- names(old_args)
    if (is.null(old_arg_names)) {
      old_arg_names <- rep("", length(old_args))
    }
    has_names <- nzchar(old_arg_names)

    if (any(has_names) && !all(has_names)) {
      stop("Do not mix named and unnamed arguments after df.", call. = FALSE)
    }
    if (length(old_args) > 5) {
      stop("Too many positional arguments supplied to detect_hyperglycemic_events().", call. = FALSE)
    }

    old_arg_targets <- c("reading_minutes", "dur_length", "end_length", "start_gl", "end_gl")
    if (all(has_names)) {
      invalid_names <- setdiff(old_arg_names, old_arg_targets)
      if (length(invalid_names) > 0) {
        stop("Unknown argument in detect_hyperglycemic_events(): ", invalid_names[1], call. = FALSE)
      }
      if (anyDuplicated(old_arg_names)) {
        stop(old_arg_names[anyDuplicated(old_arg_names)], " was supplied more than once.", call. = FALSE)
      }
      if ("reading_minutes" %in% old_arg_names && reading_minutes_provided) {
        stop("reading_minutes was supplied more than once.", call. = FALSE)
      }
      for (i in seq_along(old_args)) {
        assign(old_arg_names[i], old_args[[i]])
      }
    } else {
      for (i in seq_along(old_args)) {
        target <- old_arg_targets[i]
        if (target == "reading_minutes" && reading_minutes_provided) {
          stop("reading_minutes was supplied more than once.", call. = FALSE)
        }
        assign(target, old_args[[i]])
      }
    }
  }

  custom_criteria <- !is.null(dur_length) || !is.null(end_length) ||
    !is.null(start_gl) || !is.null(end_gl) || length(old_args) > 1

  if (type_provided && custom_criteria) {
    warning(
      "Both type and custom numeric criteria were supplied; using type and ignoring dur_length, end_length, start_gl, and end_gl.",
      call. = FALSE
    )
    custom_criteria <- FALSE
  }

  if (custom_criteria) {
    lv1_excl <- FALSE
    if (is.null(dur_length)) dur_length <- 120
    if (is.null(end_length)) end_length <- 15
    if (is.null(start_gl)) start_gl <- 250
    if (is.null(end_gl)) end_gl <- 180
  } else {
    type <- match.arg(type, c("extended", "lv1", "lv2", "lv1_excl"))
    criteria <- switch(type,
      lv1 = list(dur_length = 15, end_length = 15, start_gl = 180, end_gl = 180),
      lv2 = list(dur_length = 15, end_length = 15, start_gl = 250, end_gl = 250),
      extended = list(dur_length = 120, end_length = 15, start_gl = 250, end_gl = 180),
      lv1_excl = list(dur_length = 15, end_length = 15, start_gl = 180, end_gl = 180)
    )
    lv1_excl <- identical(type, "lv1_excl")
    dur_length <- criteria$dur_length
    end_length <- criteria$end_length
    start_gl <- criteria$start_gl
    end_gl <- criteria$end_gl
  }

  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
  }, error = function(e) {
    stop("Error in detect_hyperglycemic_events(): ", e$message, call. = FALSE)
  })
  
  # Validate parameters
  dur_length <- validate_numeric_param(dur_length, "dur_length", min_val = 0)
  end_length <- validate_numeric_param(end_length, "end_length", min_val = 0)
  start_gl <- validate_numeric_param(start_gl, "start_gl", min_val = 0)
  end_gl <- validate_numeric_param(end_gl, "end_gl", min_val = 0)
  
  # Validate reading_minutes
  reading_minutes <- validate_reading_minutes(reading_minutes, nrow(validated_df))
  sort_time <- validate_logical_param(sort_time, "sort_time")
  inter_gap <- validate_numeric_param(inter_gap, "inter_gap", min_val = 0.1)
  return_interpolated <- validate_logical_param(return_interpolated, "return_interpolated")
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .detect_hyperglycemic_events_original(
      validated_df, reading_minutes, dur_length, end_length, start_gl, end_gl,
      sort_time, inter_gap, return_interpolated, lv1_excl
    )
    return(result)
  }, error = function(e) {
    stop("Error in detect_hyperglycemic_events: ", e$message, call. = FALSE)
  })
}

detect_hypoglycemic_events <- function(df, ..., type = "extended", reading_minutes = NULL,
                                       sort_time = FALSE, inter_gap = 45,
                                       return_interpolated = TRUE) {
  type_provided <- !missing(type)
  reading_minutes_provided <- !missing(reading_minutes)
  old_args <- list(...)
  dur_length <- NULL
  end_length <- NULL
  start_gl <- NULL
  lv1_excl <- FALSE

  if (length(old_args) > 0) {
    old_arg_names <- names(old_args)
    if (is.null(old_arg_names)) {
      old_arg_names <- rep("", length(old_args))
    }
    has_names <- nzchar(old_arg_names)

    if (any(has_names) && !all(has_names)) {
      stop("Do not mix named and unnamed arguments after df.", call. = FALSE)
    }
    if (length(old_args) > 4) {
      stop("Too many positional arguments supplied to detect_hypoglycemic_events().", call. = FALSE)
    }

    old_arg_targets <- c("reading_minutes", "dur_length", "end_length", "start_gl")
    if (all(has_names)) {
      invalid_names <- setdiff(old_arg_names, old_arg_targets)
      if (length(invalid_names) > 0) {
        stop("Unknown argument in detect_hypoglycemic_events(): ", invalid_names[1], call. = FALSE)
      }
      if (anyDuplicated(old_arg_names)) {
        stop(old_arg_names[anyDuplicated(old_arg_names)], " was supplied more than once.", call. = FALSE)
      }
      if ("reading_minutes" %in% old_arg_names && reading_minutes_provided) {
        stop("reading_minutes was supplied more than once.", call. = FALSE)
      }
      for (i in seq_along(old_args)) {
        assign(old_arg_names[i], old_args[[i]])
      }
    } else {
      for (i in seq_along(old_args)) {
        target <- old_arg_targets[i]
        if (target == "reading_minutes" && reading_minutes_provided) {
          stop("reading_minutes was supplied more than once.", call. = FALSE)
        }
        assign(target, old_args[[i]])
      }
    }
  }

  custom_criteria <- !is.null(dur_length) || !is.null(end_length) ||
    !is.null(start_gl) || length(old_args) > 1

  if (type_provided && custom_criteria) {
    warning(
      "Both type and custom numeric criteria were supplied; using type and ignoring dur_length, end_length, and start_gl.",
      call. = FALSE
    )
    custom_criteria <- FALSE
  }

  if (custom_criteria) {
    lv1_excl <- FALSE
    if (is.null(dur_length)) dur_length <- 120
    if (is.null(end_length)) end_length <- 15
    if (is.null(start_gl)) start_gl <- 70
  } else {
    type <- match.arg(type, c("extended", "lv1", "lv2", "lv1_excl"))
    criteria <- switch(type,
      lv1 = list(dur_length = 15, end_length = 15, start_gl = 70),
      lv2 = list(dur_length = 15, end_length = 15, start_gl = 54),
      extended = list(dur_length = 120, end_length = 15, start_gl = 70),
      lv1_excl = list(dur_length = 15, end_length = 15, start_gl = 70)
    )
    lv1_excl <- identical(type, "lv1_excl")
    dur_length <- criteria$dur_length
    end_length <- criteria$end_length
    start_gl <- criteria$start_gl
  }

  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
  }, error = function(e) {
    stop("Error in detect_hypoglycemic_events(): ", e$message, call. = FALSE)
  })
  
  # Validate parameters
  dur_length <- validate_numeric_param(dur_length, "dur_length", min_val = 0)
  end_length <- validate_numeric_param(end_length, "end_length", min_val = 0)
  start_gl <- validate_numeric_param(start_gl, "start_gl", min_val = 0)
  
  # Validate reading_minutes
  reading_minutes <- validate_reading_minutes(reading_minutes, nrow(validated_df))
  sort_time <- validate_logical_param(sort_time, "sort_time")
  inter_gap <- validate_numeric_param(inter_gap, "inter_gap", min_val = 0.1)
  return_interpolated <- validate_logical_param(return_interpolated, "return_interpolated")
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .detect_hypoglycemic_events_original(
      validated_df, reading_minutes, dur_length, end_length, start_gl,
      sort_time, inter_gap, return_interpolated, lv1_excl
    )
    return(result)
  }, error = function(e) {
    stop("Error in detect_hypoglycemic_events: ", e$message, call. = FALSE)
  })
}

detect_all_events <- function(df, reading_minutes = NULL, sort_time = FALSE,
                              inter_gap = 45, return_interpolated = FALSE) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
  }, error = function(e) {
    stop("Error in detect_all_events(): ", e$message, call. = FALSE)
  })
  
  # Validate reading_minutes
  reading_minutes <- validate_reading_minutes(reading_minutes, nrow(validated_df))
  sort_time <- validate_logical_param(sort_time, "sort_time")
  inter_gap <- validate_numeric_param(inter_gap, "inter_gap", min_val = 0.1)
  return_interpolated <- validate_logical_param(return_interpolated, "return_interpolated")
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .detect_all_events_original(
      validated_df, reading_minutes, sort_time, inter_gap, return_interpolated
    )
    return(result)
  }, error = function(e) {
    stop("Error in detect_all_events: ", e$message, call. = FALSE)
  })
}

find_local_maxima <- function(df) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
  }, error = function(e) {
    stop("Error in find_local_maxima(): ", e$message, call. = FALSE)
  })
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .find_local_maxima_original(validated_df)
    return(result)
  }, error = function(e) {
    stop("Error in find_local_maxima: ", e$message, call. = FALSE)
  })
}

grid <- function(df, gap = 15, threshold = 130) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
  }, error = function(e) {
    stop("Error in grid(): ", e$message, call. = FALSE)
  })
  
  # Validate parameters
  gap <- validate_numeric_param(gap, "gap", min_val = 0)
  threshold <- validate_numeric_param(threshold, "threshold", min_val = 0)
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .grid_original(validated_df, gap, threshold)
    return(result)
  }, error = function(e) {
    stop("Error in grid: ", e$message, call. = FALSE)
  })
}

maxima_grid <- function(df, threshold = 130, gap = 60, hours = 2) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
  }, error = function(e) {
    stop("Error in maxima_grid(): ", e$message, call. = FALSE)
  })
  
  # Validate parameters
  threshold <- validate_numeric_param(threshold, "threshold", min_val = 0)
  gap <- validate_numeric_param(gap, "gap", min_val = 0)
  hours <- validate_numeric_param(hours, "hours", min_val = 0)
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .maxima_grid_original(validated_df, threshold, gap, hours)
    return(result)
  }, error = function(e) {
    stop("Error in maxima_grid: ", e$message, call. = FALSE)
  })
}

excursion <- function(df, gap = 15) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
  }, error = function(e) {
    stop("Error in excursion(): ", e$message, call. = FALSE)
  })
  
  # Validate parameters
  gap <- validate_numeric_param(gap, "gap", min_val = 0)
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .excursion_original(validated_df, gap)
    return(result)
  }, error = function(e) {
    stop("Error in excursion: ", e$message, call. = FALSE)
  })
}

start_finder <- function(df) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_intermediary_df(df)
  }, error = function(e) {
    stop("Error in start_finder(): ", e$message, call. = FALSE)
  })
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .Call(`_cgmguru_start_finder`, validated_df)
    return(result)
  }, error = function(e) {
    stop("Error in start_finder: ", e$message, call. = FALSE)
  })
}

find_max_after_hours <- function(df, start_point_df, hours) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
    validated_start_df <- validate_intermediary_df(start_point_df)
  }, error = function(e) {
    stop("Error in find_max_after_hours(): ", e$message, call. = FALSE)
  })
  
  # Validate parameters
  hours <- validate_numeric_param(hours, "hours", min_val = 0)
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .find_max_after_hours_original(validated_df, validated_start_df, hours)
    return(result)
  }, error = function(e) {
    stop("Error in find_max_after_hours: ", e$message, call. = FALSE)
  })
}

find_max_before_hours <- function(df, start_point_df, hours) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
    validated_start_df <- validate_intermediary_df(start_point_df)
  }, error = function(e) {
    stop("Error in find_max_before_hours(): ", e$message, call. = FALSE)
  })
  
  # Validate parameters
  hours <- validate_numeric_param(hours, "hours", min_val = 0)
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .find_max_before_hours_original(validated_df, validated_start_df, hours)
    return(result)
  }, error = function(e) {
    stop("Error in find_max_before_hours: ", e$message, call. = FALSE)
  })
}

find_min_after_hours <- function(df, start_point_df, hours) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
    validated_start_df <- validate_intermediary_df(start_point_df)
  }, error = function(e) {
    stop("Error in find_min_after_hours(): ", e$message, call. = FALSE)
  })
  
  # Validate parameters
  hours <- validate_numeric_param(hours, "hours", min_val = 0)
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .find_min_after_hours_original(validated_df, validated_start_df, hours)
    return(result)
  }, error = function(e) {
    stop("Error in find_min_after_hours: ", e$message, call. = FALSE)
  })
}

find_min_before_hours <- function(df, start_point_df, hours) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
    validated_start_df <- validate_intermediary_df(start_point_df)
  }, error = function(e) {
    stop("Error in find_min_before_hours(): ", e$message, call. = FALSE)
  })
  
  # Validate parameters
  hours <- validate_numeric_param(hours, "hours", min_val = 0)
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .find_min_before_hours_original(validated_df, validated_start_df, hours)
    return(result)
  }, error = function(e) {
    stop("Error in find_min_before_hours: ", e$message, call. = FALSE)
  })
}

detect_between_maxima <- function(df, transform_df) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
    validated_transform_df <- validate_intermediary_df(transform_df)
  }, error = function(e) {
    stop("Error in detect_between_maxima(): ", e$message, call. = FALSE)
  })
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .detect_between_maxima_original(validated_df, validated_transform_df)
    return(result)
  }, error = function(e) {
    stop("Error in detect_between_maxima: ", e$message, call. = FALSE)
  })
}

find_new_maxima <- function(df, mod_grid_max_point_df, local_maxima_df) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
    validated_mod_grid_df <- validate_intermediary_df(mod_grid_max_point_df)
    validated_maxima_df <- validate_intermediary_df(local_maxima_df)
  }, error = function(e) {
    stop("Error in find_new_maxima(): ", e$message, call. = FALSE)
  })
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .find_new_maxima_original(validated_df, validated_mod_grid_df, validated_maxima_df)
    return(result)
  }, error = function(e) {
    stop("Error in find_new_maxima: ", e$message, call. = FALSE)
  })
}

mod_grid <- function(df, grid_point_df, hours = 2, gap = 15) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_df <- validate_cgm_data(df)
    validated_grid_df <- validate_intermediary_df(grid_point_df)
  }, error = function(e) {
    stop("Error in mod_grid(): ", e$message, call. = FALSE)
  })
  
  # Validate parameters
  hours <- validate_numeric_param(hours, "hours", min_val = 0)
  gap <- validate_numeric_param(gap, "gap", min_val = 0)
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .mod_grid_original(validated_df, validated_grid_df, hours, gap)
    return(result)
  }, error = function(e) {
    stop("Error in mod_grid: ", e$message, call. = FALSE)
  })
}

transform_df <- function(grid_df, maxima_df) {
  # Validate input data with context-aware error messages
  tryCatch({
    validated_grid_df <- validate_intermediary_df(grid_df)
    validated_maxima_df <- validate_intermediary_df(maxima_df)
  }, error = function(e) {
    stop("Error in transform_df(): ", e$message, call. = FALSE)
  })
  
  # Call the original C++ function with validated inputs
  tryCatch({
    result <- .transform_df_original(validated_grid_df, validated_maxima_df)
    return(result)
  }, error = function(e) {
    stop("Error in transform_df: ", e$message, call. = FALSE)
  })
}

library(testthat)
library(cgmguru)
library(iglu)

data(example_data_5_subject)

test_that("per-id tzone mapping attaches and propagates", {
  df <- head(example_data_5_subject, 60)
  df$tz <- rep(c("UTC","EST","PST"), length.out = nrow(df))
  gr <- grid(df, gap = 15, threshold = 130)
  maxa <- find_max_after_hours(df, start_finder(gr$grid_vector), hours = 2)
  expect_true(!is.null(attr(maxa$episode_start, "tzone_by_id")))
  expect_true(!is.null(attr(maxa$episode_counts, "tzone_by_id")))
})

test_that("episode counting runs across multiple ids and boundaries", {
  df <- example_data_5_subject
  gr <- grid(df, gap = 15, threshold = 130)
  mg <- mod_grid(df, gr$grid_vector, hours = 2, gap = 15)
  res <- find_max_after_hours(df, start_finder(mg$mod_grid_vector), hours = 1)
  expect_true(is.data.frame(res$episode_counts))
})

library(testthat)
library(cgmguru)
library(iglu)

data(example_data_5_subject, package = "iglu")
data(example_data_hall, package = "iglu")

# iglu::episode_calculation() does not expose an extended hyperglycemia label,
# so these are the seven shared event definitions.
iglu_event_specs <- data.frame(
  type = c("hypo", "hypo", "hypo", "hypo", "hyper", "hyper", "hyper"),
  level = c("lv1", "lv2", "extended", "lv1_excl", "lv1", "lv2", "lv1_excl"),
  label = c(
    "lv1_hypo", "lv2_hypo", "ext_hypo", "lv1_hypo_excl",
    "lv1_hyper", "lv2_hyper", "lv1_hyper_excl"
  ),
  threshold = c(70, 54, 70, 70, 180, 250, 180),
  stringsAsFactors = FALSE
)

empty_event_spans <- function() {
  data.frame(
    id = character(),
    start_time = character(),
    end_time = character(),
    stringsAsFactors = FALSE
  )
}

event_time_key <- function(x) {
  # iglu reconstructs these example-data timestamps with UTC wall-clock labels
  # while cgmguru preserves the source EST tzone. Compare the displayed event
  # boundary labels rather than POSIXct offsets.
  format(x, "%Y-%m-%d %H:%M:%S")
}

sort_event_spans <- function(x) {
  if (nrow(x) == 0) {
    return(empty_event_spans())
  }

  x <- as.data.frame(x[, c("id", "start_time", "end_time")])
  x$id <- as.character(x$id)
  x$start_time <- as.character(x$start_time)
  x$end_time <- as.character(x$end_time)
  rownames(x) <- NULL
  x <- x[order(x$id, x$start_time, x$end_time), , drop = FALSE]
  rownames(x) <- NULL
  x
}

iglu_excursion_spans <- function(episode_data, spec) {
  event_label <- episode_data[[spec$label]]
  event_rows <- which(event_label > 0)

  if (length(event_rows) == 0) {
    return(empty_event_spans())
  }

  event_groups <- split(
    event_rows,
    paste(
      as.character(episode_data$id[event_rows]),
      episode_data$segment[event_rows],
      event_label[event_rows],
      sep = "\r"
    )
  )

  spans <- do.call(rbind, lapply(event_groups, function(rows) {
    in_excursion <- if (spec$type == "hypo") {
      episode_data$gl[rows] < spec$threshold
    } else {
      episode_data$gl[rows] > spec$threshold
    }
    excursion_rows <- rows[in_excursion]

    data.frame(
      id = as.character(episode_data$id[excursion_rows[1]]),
      start_time = event_time_key(episode_data$time[excursion_rows[1]]),
      end_time = event_time_key(episode_data$time[excursion_rows[length(excursion_rows)]]),
      stringsAsFactors = FALSE
    )
  }))

  sort_event_spans(spans)
}

cgmguru_detailed_spans <- function(data, spec) {
  events <- if (spec$type == "hypo") {
    detect_hypoglycemic_events(data, type = spec$level, reading_minutes = 5)
  } else {
    detect_hyperglycemic_events(data, type = spec$level, reading_minutes = 5)
  }

  details <- events$events_detailed
  if (nrow(details) == 0) {
    return(empty_event_spans())
  }

  sort_event_spans(data.frame(
    id = as.character(details$id),
    start_time = event_time_key(details$start_time),
    end_time = event_time_key(details$end_time),
    stringsAsFactors = FALSE
  ))
}

normalise_counts <- function(x, count_col) {
  out <- data.frame(
    id = as.character(x$id),
    type = as.character(x$type),
    level = as.character(x$level),
    event_count = as.integer(x[[count_col]]),
    stringsAsFactors = FALSE
  )
  rownames(out) <- NULL
  out <- out[order(out$id, out$type, out$level), , drop = FALSE]
  rownames(out) <- NULL
  out
}

expect_iglu_count_parity <- function(data, dataset_name) {
  cgmguru_counts <- detect_all_events(data, reading_minutes = 5)$events_long_df
  iglu_counts <- iglu::episode_calculation(data, dt0 = 5, tz = "UTC")

  for (i in seq_len(nrow(iglu_event_specs))) {
    spec <- iglu_event_specs[i, ]
    info <- paste(dataset_name, spec$type, spec$level)

    cgmguru_subset <- cgmguru_counts[
      cgmguru_counts$type == spec$type & cgmguru_counts$level == spec$level,
    ]
    iglu_subset <- iglu_counts[
      iglu_counts$type == spec$type & iglu_counts$level == spec$level,
    ]

    expect_equal(
      normalise_counts(cgmguru_subset, "total_episodes"),
      normalise_counts(iglu_subset, "total_episodes"),
      info = info
    )
  }
}

expect_iglu_boundary_parity <- function(data, dataset_name) {
  iglu_episode_data <- iglu::episode_calculation(
    data,
    return_data = TRUE,
    dt0 = 5,
    tz = "UTC"
  )$data

  for (i in seq_len(nrow(iglu_event_specs))) {
    spec <- iglu_event_specs[i, ]
    info <- paste(dataset_name, spec$type, spec$level)

    expect_equal(
      cgmguru_detailed_spans(data, spec),
      iglu_excursion_spans(iglu_episode_data, spec),
      info = info
    )
  }
}

test_that("detect_all_events event counts match iglu episode_calculation examples", {
  expect_iglu_count_parity(example_data_5_subject, "example_data_5_subject")
  expect_iglu_count_parity(example_data_hall, "example_data_hall")
})

test_that("detailed cgmguru event starts and ends match iglu episode labels", {
  expect_iglu_boundary_parity(example_data_5_subject, "example_data_5_subject")
  expect_iglu_boundary_parity(example_data_hall, "example_data_hall")
})

library(testthat)
library(cgmguru)

make_cgm_at <- function(minutes, gl) {
  data.frame(
    id = "A",
    time = as.POSIXct("2026-01-01 00:15:00", tz = "UTC") + minutes * 60,
    gl = gl
  )
}

test_that("reading_minutes is inferred per id when omitted", {
  df <- make_cgm_at(c(0, 15, 30, 45), c(60, 80, 82, 84))

  res <- detect_hypoglycemic_events(
    df,
    start_gl = 70,
    dur_length = 15,
    end_length = 15,
    return_interpolated = TRUE
  )

  expect_equal(sum(res$events_total$total_episodes), 1)
  expect_named(res$interpolated_data, c("id", "time", "gl"))
  expect_equal(res$events_detailed$start_index[1], 1)
  expect_equal(res$events_detailed$end_index[1], 1)
})

test_that("sort_time is optional and uses C++ sorting when enabled", {
  df <- make_cgm_at(c(0, 5, 10, 15, 20, 25), c(60, 62, 65, 80, 82, 84))
  shuffled <- df[c(1, 3, 2, 4, 5, 6), ]

  expect_error(
    detect_hypoglycemic_events(
      shuffled,
      start_gl = 70,
      dur_length = 15,
      end_length = 15
    ),
    "time must be nondecreasing"
  )

  sorted_res <- detect_hypoglycemic_events(
    shuffled,
    start_gl = 70,
    dur_length = 15,
    end_length = 15,
    sort_time = TRUE
  )
  baseline <- detect_hypoglycemic_events(
    df,
    start_gl = 70,
    dur_length = 15,
    end_length = 15
  )

  expect_equal(sorted_res$events_total$total_episodes, baseline$events_total$total_episodes)
  expect_equal(sorted_res$events_detailed$start_time, baseline$events_detailed$start_time)
})

test_that("gaps within inter_gap are linearly interpolated before event detection", {
  df <- make_cgm_at(c(0, 10, 15, 20, 25), c(60, 60, 80, 82, 84))

  res <- detect_hypoglycemic_events(
    df,
    start_gl = 70,
    dur_length = 15,
    end_length = 15,
    return_interpolated = TRUE
  )

  expect_equal(sum(res$events_total$total_episodes), 1)
  expect_equal(res$events_detailed$end_time[1], df$time[2])
  interpolated_idx <- match(df$time[1] + 5 * 60, res$interpolated_data$time)
  expect_false(is.na(interpolated_idx))
  expect_equal(res$interpolated_data$gl[interpolated_idx], 60)
})

test_that("gaps above inter_gap split segments and do not finalize events", {
  df <- make_cgm_at(c(0, 50, 55, 60, 65), c(60, 60, 80, 82, 84))

  res <- detect_hypoglycemic_events(
    df,
    start_gl = 70,
    dur_length = 15,
    end_length = 15,
    return_interpolated = TRUE
  )

  expect_equal(sum(res$events_total$total_episodes), 0)
  expect_equal(nrow(res$events_detailed), 0)
  expect_false(any(is.na(res$interpolated_data$gl)))
  expect_named(res$interpolated_data, c("id", "time", "gl"))
})

test_that("events_detailed indices point into returned interpolated_data", {
  start_time <- as.POSIXct("2026-01-01 00:15:00", tz = "UTC")
  no_event <- data.frame(
    id = "A",
    time = start_time + 0:3 * 5 * 60,
    gl = c(100, 101, 102, 103)
  )
  hypo_event <- data.frame(
    id = "B",
    time = start_time + 0:5 * 5 * 60,
    gl = c(50, 51, 52, 80, 82, 84)
  )
  hyper_event <- data.frame(
    id = "B",
    time = start_time + 0:5 * 5 * 60,
    gl = c(190, 195, 200, 170, 165, 160)
  )

  hypo <- detect_hypoglycemic_events(
    rbind(hypo_event, no_event),
    reading_minutes = 5,
    start_gl = 70,
    dur_length = 15,
    end_length = 15,
    return_interpolated = TRUE
  )
  hypo_rows <- hypo$interpolated_data[
    hypo$events_detailed$start_index[1]:hypo$events_detailed$end_index[1],
  ]

  expect_gt(hypo$events_detailed$start_index[1], 1)
  expect_true(all(hypo_rows$id == hypo$events_detailed$id[1]))
  expect_equal(hypo_rows$time[1], hypo$events_detailed$start_time[1])
  expect_equal(hypo_rows$time[nrow(hypo_rows)], hypo$events_detailed$end_time[1])
  expect_equal(hypo_rows$gl[1], hypo$events_detailed$start_glucose[1])
  expect_equal(hypo_rows$gl[nrow(hypo_rows)], hypo$events_detailed$end_glucose[1])

  hyper <- detect_hyperglycemic_events(
    rbind(hyper_event, no_event),
    reading_minutes = 5,
    start_gl = 180,
    dur_length = 15,
    end_length = 15,
    end_gl = 180,
    return_interpolated = TRUE
  )
  hyper_rows <- hyper$interpolated_data[
    hyper$events_detailed$start_index[1]:hyper$events_detailed$end_index[1],
  ]

  expect_gt(hyper$events_detailed$start_index[1], 1)
  expect_true(all(hyper_rows$id == hyper$events_detailed$id[1]))
  expect_equal(hyper_rows$time[1], hyper$events_detailed$start_time[1])
  expect_equal(hyper_rows$time[nrow(hyper_rows)], hyper$events_detailed$end_time[1])
  expect_equal(hyper_rows$gl[1], hyper$events_detailed$start_glucose[1])
  expect_equal(hyper_rows$gl[nrow(hyper_rows)], hyper$events_detailed$end_glucose[1])
})

test_that("detect_all_events returns only event and summary tables", {
  df <- make_cgm_at(c(0, 15, 30, 45), c(50, 80, 82, 84))

  res <- detect_all_events(df, return_interpolated = TRUE)

  expect_true(is.list(res))
  expect_named(res, c("events_long_df", "summary_df"))
  expect_true(is.data.frame(res$events_long_df))
  expect_true(is.data.frame(res$summary_df))
  expect_false("interpolated_data" %in% names(res))
})

test_that("standalone lv1_excl excludes lv1 episodes overlapping lv2", {
  hyper_df <- make_cgm_at(
    c(0, 5, 10, 15, 20, 25, 30, 35, 40, 45),
    c(190, 195, 200, 170, 165, 160, 260, 265, 270, 170)
  )
  hyper <- detect_hyperglycemic_events(
    hyper_df,
    type = "lv1_excl",
    reading_minutes = 5
  )
  expect_equal(sum(hyper$events_total$total_episodes), 1)
  expect_equal(nrow(hyper$events_detailed), 1)
  expect_equal(hyper$events_detailed$start_time[1], hyper_df$time[1])

  hypo_df <- make_cgm_at(
    c(0, 5, 10, 15, 20, 25, 30, 35, 40, 45),
    c(60, 62, 65, 80, 82, 84, 50, 51, 52, 80)
  )
  hypo <- detect_hypoglycemic_events(
    hypo_df,
    type = "lv1_excl",
    reading_minutes = 5
  )
  expect_equal(sum(hypo$events_total$total_episodes), 1)
  expect_equal(nrow(hypo$events_detailed), 1)
  expect_equal(hypo$events_detailed$start_time[1], hypo_df$time[1])
})

library(testthat)
library(cgmguru)
library(iglu)

data(example_data_5_subject)

test_that("hyper boundary equality is included (>=)", {
  res <- detect_hyperglycemic_events(example_data_5_subject,
                                     start_gl = 180, end_gl = 180,
                                     dur_length = 15, end_length = 15)
  expect_true(is.list(res))
})

test_that("hypo boundary equality is included (<=)", {
  res <- detect_hypoglycemic_events(example_data_5_subject,
                                    start_gl = 70, dur_length = 15, end_length = 15)
  expect_true(is.list(res))
})

test_that("reading_minutes vector path is exercised", {
  rm <- rep(5, nrow(example_data_5_subject))
  res_hyper <- detect_hyperglycemic_events(example_data_5_subject, reading_minutes = rm)
  res_hypo  <- detect_hypoglycemic_events(example_data_5_subject, reading_minutes = rm)
  expect_true(is.list(res_hyper) && is.list(res_hypo))
})

test_that("no-event and NA-paths don’t crash", {
  df <- example_data_5_subject
  df$gl[] <- 100
  with_na <- df; with_na$gl[1:3] <- NA
  r1 <- detect_hyperglycemic_events(df)
  r2 <- detect_hypoglycemic_events(with_na)
  expect_true(is.list(r1) && is.list(r2))
})

library(testthat)
library(cgmguru)
library(iglu)

data(example_data_5_subject)

make_pipeline <- function(df) {
	gap <- 15; threshold <- 130; hours <- 2
	grid_result <- grid(df, gap = gap, threshold = threshold)
	mod_grid_result <- mod_grid(df, start_finder(grid_result$grid_vector), hours = hours, gap = gap)
	max_after <- find_max_after_hours(df, start_finder(mod_grid_result$mod_grid_vector), hours = hours)
	local_maxima <- find_local_maxima(df)
	final_maxima <- find_new_maxima(df, max_after$max_index, local_maxima$local_maxima_vector)
	list(grid_result = grid_result, final_maxima = final_maxima)
}

test_that("transform_df returns expected columns and non-empty with typical data", {
	pl <- make_pipeline(example_data_5_subject)
	trans <- transform_df(pl$grid_result$episode_start, pl$final_maxima)
	expect_true(is.data.frame(trans))
	expect_true(all(c("id","grid_time","grid_gl","maxima_time","maxima_gl") %in% names(trans)))
	# grid_time <= maxima_time (0 to 4 hours window) if present
	if (nrow(trans) > 0) {
		dh <- as.numeric(difftime(trans$maxima_time, trans$grid_time, units = "hours"))
		expect_true(all(dh >= 0 | is.na(dh)))
		expect_true(all(dh <= 4 + 1e-9 | is.na(dh)))
	}
})

test_that("detect_between_maxima consumes transform_df and returns expected schema", {
	pl <- make_pipeline(example_data_5_subject)
	trans <- transform_df(pl$grid_result$episode_start, pl$final_maxima)
	res <- detect_between_maxima(example_data_5_subject, trans)
	expect_true(is.list(res) || is.data.frame(res))
	expect_true(all(c("results","episode_counts") %in% names(res)))
	expect_true(is.data.frame(res$results))
	# results columns per implementation
	expect_true(all(c("id","grid_time","grid_gl","maxima_time","maxima_glucose","time_to_peak") %in% names(res$results)))
	# time_to_peak should be >= 0 when not NA
	if (nrow(res$results) > 0) {
		ttp <- res$results$time_to_peak
		expect_true(all(is.na(ttp) | ttp >= 0))
	}
})

library(testthat)
library(cgmguru)

test_that("start_finder works with grid_vector intermediary and handles empty start flags", {
  # Build intermediary via grid
  data(example_data_5_subject, package = "iglu")
  gr <- grid(example_data_5_subject, gap = 15, threshold = 130)
  starts <- start_finder(gr$grid_vector)
  expect_true(is.data.frame(starts))
  expect_true("start_index" %in% names(starts))

  # Empty start flags -> no index
  empty_intermediate <- data.frame(flag = integer(0))
  res_empty <- start_finder(empty_intermediate)
  expect_true(is.data.frame(res_empty))
  expect_equal(nrow(res_empty), 0)
})

library(testthat)
library(cgmguru)
library(iglu)

data(example_data_5_subject)

test_that("detect_all_events returns named tables and validates reading_minutes", {
	res <- detect_all_events(example_data_5_subject)
	expect_true(is.list(res))
	expect_named(res, c("events_long_df", "summary_df"))
	expect_true(is.data.frame(res$events_long_df))
	expect_true(is.data.frame(res$summary_df))
	expect_true(all(c(
		"id", "TIR", "TITR", "TBR70", "TBR54", "TAR180", "TAR250",
		"CV", "SD", "mean_glucose", "GMI", "uGMI", "GRI", "sensor_wear",
		"hypo_lv1_event_count", "hypo_lv2_event_count",
		"hypo_extended_event_count", "hypo_lv1_excl_event_count",
		"hyper_lv1_event_count", "hyper_lv2_event_count",
		"hyper_extended_event_count", "hyper_lv1_excl_event_count"
	) %in% names(res$summary_df)))
	expect_named(res$events_long_df, c(
		"id", "type", "level", "total_episodes",
		"avg_ep_per_day", "avg_episode_duration_below_54"
	))

	res5 <- detect_all_events(example_data_5_subject, reading_minutes = 5)
	expect_true(is.list(res5))

	expect_error(detect_all_events(example_data_5_subject, reading_minutes = c(5, 5)),
		"reading_minutes vector length must match data length or be a single value")

	# Empty input returns empty data.frame
	empty_cgm <- data.frame(
		id = character(0),
		time = as.POSIXct(character(0), tz = "UTC"),
		gl = numeric(0),
		stringsAsFactors = FALSE
	)
	res_empty <- detect_all_events(empty_cgm)
	expect_true(is.list(res_empty))
	expect_named(res_empty, c("events_long_df", "summary_df"))
	expect_equal(nrow(res_empty$events_long_df), 0)
	expect_equal(nrow(res_empty$summary_df), 0)
})

test_that("detect_all_events CGM summary metrics use interpolated data", {
	df <- data.frame(
		id = "A",
		time = as.POSIXct("2026-01-01 00:05:00", tz = "UTC") + c(0, 10) * 60,
		gl = c(100, 200)
	)

	res <- detect_all_events(df, reading_minutes = 5)$summary_df

	expect_equal(nrow(res), 1)
	expect_equal(res$TIR, 100 * 2 / 3, tolerance = 1e-8)
	expect_equal(res$TITR, 100 / 3, tolerance = 1e-8)
	expect_equal(res$TAR180, 100 / 3, tolerance = 1e-8)
	expect_equal(res$mean_glucose, 150, tolerance = 1e-8)
	expect_equal(res$SD, stats::sd(c(100, 150, 200)), tolerance = 1e-8)
	expect_equal(res$CV, stats::sd(c(100, 150, 200)) / 150, tolerance = 1e-8)
	expect_equal(res$GMI, 3.31 + 0.02392 * 150, tolerance = 1e-8)
	expect_equal(res$uGMI, 1 / (15.36 / 150 + 0.0425), tolerance = 1e-8)
	expect_equal(res$GRI, 0.8 * (100 / 3), tolerance = 1e-8)
	expect_equal(res$sensor_wear, 100 * 2 / 3, tolerance = 1e-8)
})

test_that("detect_all_events calculates Glycemia Risk Index components", {
	df <- data.frame(
		id = "A",
		time = as.POSIXct("2026-01-01 00:05:00", tz = "UTC") + 0:4 * 5 * 60,
		gl = c(50, 60, 100, 200, 260)
	)

	res <- detect_all_events(df, reading_minutes = 5)$summary_df

	expect_equal(res$TBR54, 20, tolerance = 1e-8)
	expect_equal(res$TBR70, 40, tolerance = 1e-8)
	expect_equal(res$TAR180, 40, tolerance = 1e-8)
	expect_equal(res$TAR250, 20, tolerance = 1e-8)
	expect_equal(res$GRI, 3.0 * 20 + 2.4 * 20 + 1.6 * 20 + 0.8 * 20,
							 tolerance = 1e-8)
	expect_equal(res$sensor_wear, 100, tolerance = 1e-8)
})

test_that("detect_hypoglycemic_events structure and parameter validation", {
	res <- detect_hypoglycemic_events(example_data_5_subject, start_gl = 70, dur_length = 15, end_length = 15)
	res_lv1_type <- detect_hypoglycemic_events(example_data_5_subject, type = "lv1")
	expect_true(is.list(res))
	expect_true(all(c("events_detailed", "events_total") %in% names(res)))
	expect_true(is.data.frame(res$events_detailed))
	expect_true(is.data.frame(res$events_total))
	expect_equal(res_lv1_type, res)

	res_lv2 <- detect_hypoglycemic_events(example_data_5_subject, start_gl = 54, dur_length = 15, end_length = 15)
	res_lv2_type <- detect_hypoglycemic_events(example_data_5_subject, type = "lv2")
	expect_equal(res_lv2_type, res_lv2)

	res_extended <- detect_hypoglycemic_events(example_data_5_subject)
	res_extended_type <- detect_hypoglycemic_events(example_data_5_subject, type = "extended")
	expect_equal(res_extended_type, res_extended)

	expect_error(detect_hypoglycemic_events(example_data_5_subject, dur_length = -1), "dur_length must be between 0 and Inf")
	expect_error(detect_hypoglycemic_events(example_data_5_subject, end_length = -1), "end_length must be between 0 and Inf")
	expect_error(detect_hypoglycemic_events(example_data_5_subject, start_gl = -1), "start_gl must be between 0 and Inf")
	expect_error(detect_hypoglycemic_events(example_data_5_subject, type = "bad"), "should be one of")
	expect_warning(
		res_type_wins <- detect_hypoglycemic_events(example_data_5_subject, type = "lv1", start_gl = 54),
		"using type and ignoring"
	)
	expect_equal(res_type_wins, res_lv1_type)
})

test_that("detect_hyperglycemic_events structure, params and boundary thresholds", {
	# Level 1 boundary at 180
	res_lv1 <- detect_hyperglycemic_events(example_data_5_subject, start_gl = 180, dur_length = 15, end_length = 15, end_gl = 180)
	res_lv1_type <- detect_hyperglycemic_events(example_data_5_subject, type = "lv1")
	expect_true(is.list(res_lv1))
	expect_true(is.data.frame(res_lv1$events_detailed))
	expect_true(is.data.frame(res_lv1$events_total))
	expect_equal(res_lv1_type, res_lv1)

	# Level 2 boundary at 250
	res_lv2 <- detect_hyperglycemic_events(example_data_5_subject, start_gl = 250, dur_length = 15, end_length = 15, end_gl = 250)
	res_lv2_type <- detect_hyperglycemic_events(example_data_5_subject, type = "lv2")
	expect_true(is.list(res_lv2))
	expect_true(is.data.frame(res_lv2$events_detailed))
	expect_true(is.data.frame(res_lv2$events_total))
	expect_equal(res_lv2_type, res_lv2)

	# Default and explicit extended type preserve existing behavior
	res_extended <- detect_hyperglycemic_events(example_data_5_subject)
	res_extended_type <- detect_hyperglycemic_events(example_data_5_subject, type = "extended")
	expect_equal(res_extended_type, res_extended)

	# Parameter validation
	expect_error(detect_hyperglycemic_events(example_data_5_subject, dur_length = -1), "dur_length must be between 0 and Inf")
	expect_error(detect_hyperglycemic_events(example_data_5_subject, start_gl = -1), "start_gl must be between 0 and Inf")
	expect_error(detect_hyperglycemic_events(example_data_5_subject, type = "bad"), "should be one of")
	expect_warning(
		res_type_wins <- detect_hyperglycemic_events(example_data_5_subject, type = "lv1", start_gl = 250),
		"using type and ignoring"
	)
	expect_equal(res_type_wins, res_lv1_type)

	# Sanity: level 1 should detect at least as many events as stricter level 2
	total_lv1 <- if (nrow(res_lv1$events_total)) sum(res_lv1$events_total$total_episodes) else 0
	total_lv2 <- if (nrow(res_lv2$events_total)) sum(res_lv2$events_total$total_episodes) else 0
	expect_true(total_lv1 >= total_lv2)
})

library(testthat)
library(cgmguru)

make_cgm <- function(gl, start = "2026-01-01 00:15:00", step_mins = 5) {
  data.frame(
    id = "A",
    time = as.POSIXct(start, tz = "UTC") + seq_along(gl) * 0 + (seq_along(gl) - 1) * step_mins * 60,
    gl = gl
  )
}

test_that("hypoglycemic detailed end is last low reading before confirmed recovery", {
  df <- make_cgm(c(60, 62, 65, 80, 82, 84, 86, 88))

  res <- detect_hypoglycemic_events(df, start_gl = 70, dur_length = 15, end_length = 15)


  expect_equal(sum(res$events_total$total_episodes), 1)
  expect_equal(nrow(res$events_detailed), 1)
  expect_equal(res$events_detailed$start_time[1], df$time[1])
  expect_equal(res$events_detailed$end_time[1], df$time[3])
  expect_equal(res$events_detailed$end_index[1], 3)
  expect_true(is.data.frame(res$interpolated_data))
  expect_named(res$interpolated_data, c("id", "time", "gl"))
})

test_that("hyperglycemic detailed end is last high reading before confirmed recovery", {
  df <- make_cgm(c(190, 195, 200, 170, 165, 160, 155))

  res <- detect_hyperglycemic_events(df, start_gl = 180, dur_length = 15, end_length = 15, end_gl = 180)

  expect_equal(sum(res$events_total$total_episodes), 1)
  expect_equal(nrow(res$events_detailed), 1)
  expect_equal(res$events_detailed$start_time[1], df$time[1])
  expect_equal(res$events_detailed$end_time[1], df$time[3])
  expect_equal(res$events_detailed$end_index[1], 3)
  expect_true(is.data.frame(res$interpolated_data))
  expect_named(res$interpolated_data, c("id", "time", "gl"))
})

test_that("extended hyperglycemic detailed end is last high reading before confirmed recovery", {
  cases <- list(
    list(step_mins = 5, gl = c(rep(260, 18), 240, rep(170, 4)), end_index = 19),
    list(step_mins = 15, gl = c(rep(260, 6), 240, rep(170, 3)), end_index = 7)
  )

  for (case in cases) {
    df <- make_cgm(case$gl, step_mins = case$step_mins)

    res <- detect_hyperglycemic_events(df, reading_minutes = case$step_mins)

    expect_equal(sum(res$events_total$total_episodes), 1)
    expect_equal(nrow(res$events_detailed), 1)
    expect_equal(res$events_detailed$start_time[1], df$time[1])
    expect_equal(res$events_detailed$end_time[1], df$time[case$end_index])
    expect_equal(res$events_detailed$end_glucose[1], df$gl[case$end_index])
    expect_equal(res$events_detailed$start_index[1], 1)
    expect_equal(res$events_detailed$end_index[1], case$end_index)
  }
})

test_that("broken recovery reports the end before final successful recovery", {
  hypo_df <- make_cgm(c(50, 51, 52, 80, 82, 50, 51, 52, 80, 82, 84, 86, 88))
  hypo <- detect_hypoglycemic_events(hypo_df, start_gl = 70, dur_length = 15, end_length = 15)

  expect_equal(sum(hypo$events_total$total_episodes), 1)
  expect_equal(nrow(hypo$events_detailed), 1)
  expect_equal(hypo$events_detailed$end_time[1], hypo_df$time[8])
  expect_equal(hypo$events_detailed$duration_below_54_minutes[1], 30)

  hyper_df <- make_cgm(c(190, 195, 200, 170, 165, 190, 195, 200, 170, 165, 160, 155))
  hyper <- detect_hyperglycemic_events(hyper_df, start_gl = 180, dur_length = 15, end_length = 15, end_gl = 180)

  expect_equal(sum(hyper$events_total$total_episodes), 1)
  expect_equal(nrow(hyper$events_detailed), 1)
  expect_equal(hyper$events_detailed$end_time[1], hyper_df$time[8])
})

test_that("events without confirmed recovery are counted at segment end like iglu", {
  hypo_df <- make_cgm(c(60, 62, 65, 80, 82, 60))
  hyper_df <- make_cgm(c(190, 195, 200, 170, 165, 190))

  hypo <- detect_hypoglycemic_events(hypo_df, start_gl = 70, dur_length = 15, end_length = 15)
  hyper <- detect_hyperglycemic_events(hyper_df, start_gl = 180, dur_length = 15, end_length = 15, end_gl = 180)

  expect_equal(sum(hypo$events_total$total_episodes), 1)
  expect_equal(nrow(hypo$events_detailed), 1)
  expect_equal(sum(hyper$events_total$total_episodes), 1)
  expect_equal(nrow(hyper$events_detailed), 1)
})

test_that("detect_all_events keeps counts aligned with standalone confirmed events", {
  hypo_df <- make_cgm(c(50, 51, 52, 80, 82, 84, 86, 88))
  hypo <- detect_hypoglycemic_events(hypo_df, start_gl = 54, dur_length = 15, end_length = 15)
  all_hypo <- detect_all_events(hypo_df, reading_minutes = 5)
  all_hypo_lv2 <- subset(all_hypo$events_long_df, type == "hypo" & level == "lv2")

  expect_equal(all_hypo$summary_df$hypo_lv2_event_count,
               sum(hypo$events_total$total_episodes))
  expect_equal(all_hypo_lv2$avg_episode_duration_below_54, 15)

  hyper_df <- make_cgm(c(190, 195, 200, 170, 165, 160, 155, 150))
  hyper <- detect_hyperglycemic_events(hyper_df, start_gl = 180, dur_length = 15, end_length = 15, end_gl = 180)
  all_hyper <- detect_all_events(hyper_df, reading_minutes = 5)

  expect_equal(all_hyper$summary_df$hyper_lv1_event_count,
               sum(hyper$events_total$total_episodes))
})

test_that("15-minute sampling detects one-point hypoglycemic event and reports pre-recovery end", {
  df <- make_cgm(c(60, 80, 82, 84), step_mins = 15)

  res <- detect_hypoglycemic_events(df, reading_minutes = 15,
                                    start_gl = 70, dur_length = 15, end_length = 15)

  expect_equal(sum(res$events_total$total_episodes), 1)
  expect_equal(nrow(res$events_detailed), 1)
  expect_equal(res$events_detailed$start_time[1], df$time[1])
  expect_equal(res$events_detailed$end_time[1], df$time[1])
  expect_equal(res$events_detailed$start_index[1], 1)
  expect_equal(res$events_detailed$end_index[1], 1)
})

test_that("15-minute sampling detects one-point hyperglycemic event and reports pre-recovery end", {
  df <- make_cgm(c(190, 170, 165, 160), step_mins = 15)

  res <- detect_hyperglycemic_events(df, reading_minutes = 15,
                                     start_gl = 180, dur_length = 15,
                                     end_length = 15, end_gl = 180)

  expect_equal(sum(res$events_total$total_episodes), 1)
  expect_equal(nrow(res$events_detailed), 1)
  expect_equal(res$events_detailed$start_time[1], df$time[1])
  expect_equal(res$events_detailed$end_time[1], df$time[1])
  expect_equal(res$events_detailed$start_index[1], 1)
  expect_equal(res$events_detailed$end_index[1], 1)
})

test_that("15-minute sampling detect_all_events counts one-point events after recovery confirmation", {
  hypo_df <- make_cgm(c(50, 80, 82, 84), step_mins = 15)
  hypo <- detect_hypoglycemic_events(hypo_df, reading_minutes = 15,
                                    start_gl = 54, dur_length = 15, end_length = 15)
  all_hypo <- detect_all_events(hypo_df, reading_minutes = 15)
  all_hypo_lv2 <- subset(all_hypo$events_long_df, type == "hypo" & level == "lv2")
  expect_equal(all_hypo$summary_df$hypo_lv2_event_count,
               sum(hypo$events_total$total_episodes))
  expect_equal(all_hypo_lv2$avg_episode_duration_below_54, 15)

  hyper_df <- make_cgm(c(190, 170, 165, 160, 155), step_mins = 15)
  hyper <- detect_hyperglycemic_events(hyper_df, reading_minutes = 15,
                                       start_gl = 180, dur_length = 15,
                                       end_length = 15, end_gl = 180)
  all_hyper <- detect_all_events(hyper_df, reading_minutes = 15)
  expect_equal(all_hyper$summary_df$hyper_lv1_event_count,
               sum(hyper$events_total$total_episodes))
})

test_that("120-minute hypoglycemia duration uses whole interpolated grid counts", {
  short_df <- make_cgm(c(rep(60, 23), rep(80, 3)))
  full_df <- make_cgm(c(rep(60, 24), rep(80, 3)))

  short <- detect_hypoglycemic_events(short_df, reading_minutes = 5,
                                      start_gl = 70, dur_length = 120,
                                      end_length = 15)
  full <- detect_hypoglycemic_events(full_df, reading_minutes = 5,
                                     start_gl = 70, dur_length = 120,
                                     end_length = 15)

  expect_equal(sum(short$events_total$total_episodes), 0)
  expect_equal(sum(full$events_total$total_episodes), 1)

  all_short <- detect_all_events(short_df, reading_minutes = 5)
  all_full <- detect_all_events(full_df, reading_minutes = 5)

  expect_equal(all_short$summary_df$hypo_extended_event_count, 0)
  expect_equal(all_full$summary_df$hypo_extended_event_count, 1)
})

test_that("default extended hyperglycemia uses 120-minute window and 90-minute requirement", {
  short_df <- make_cgm(c(rep(260, 17), rep(170, 3)))
  full_df <- make_cgm(c(rep(260, 18), rep(170, 3)))

  short <- detect_hyperglycemic_events(short_df, reading_minutes = 5,
                                       type = "extended")
  full <- detect_hyperglycemic_events(full_df, reading_minutes = 5,
                                      type = "extended")

  expect_equal(sum(short$events_total$total_episodes), 0)
  expect_equal(sum(full$events_total$total_episodes), 1)

  all_short <- detect_all_events(short_df, reading_minutes = 5)
  all_full <- detect_all_events(full_df, reading_minutes = 5)

  expect_equal(all_short$summary_df$hyper_extended_event_count, 0)
  expect_equal(all_full$summary_df$hyper_extended_event_count, 1)
})

library(testthat)
library(cgmguru)
library(iglu)

# Use example CGM data from iglu
data(example_data_5_subject)

empty_cgm <- data.frame(
  id = character(0),
  time = as.POSIXct(character(0), tz = "UTC"),
  gl = numeric(0),
  stringsAsFactors = FALSE
)

test_that("detect_hypoglycemic_events wrapper returns expected structure and validates", {
  res <- detect_hypoglycemic_events(example_data_5_subject, start_gl = 70, dur_length = 15, end_length = 15)
  expect_true(is.list(res))
  expect_true(all(c("events_detailed", "events_total") %in% names(res)))
  expect_true(is.data.frame(res$events_detailed))
  expect_true(is.data.frame(res$events_total))

  # Empty input should not error and should return empty data.frames
  res_empty <- detect_hypoglycemic_events(empty_cgm)
  expect_true(is.list(res_empty))
  expect_true(nrow(res_empty$events_detailed) == 0)
  expect_true(nrow(res_empty$events_total) == 0)

  # Parameter validation - these should work now with overrides active
  expect_error(detect_hypoglycemic_events(example_data_5_subject, dur_length = -1),
               "dur_length must be between 0 and Inf")
  expect_error(detect_hypoglycemic_events(example_data_5_subject, end_length = -1),
               "end_length must be between 0 and Inf")
  expect_error(detect_hypoglycemic_events(example_data_5_subject, start_gl = -1),
               "start_gl must be between 0 and Inf")
})

test_that("detect_all_events wrapper returns named tables and validates reading_minutes", {
  res <- detect_all_events(example_data_5_subject)
  expect_true(is.list(res))
  expect_named(res, c("events_long_df", "summary_df"))
  expect_true(is.data.frame(res$events_long_df))
  expect_true(is.data.frame(res$summary_df))

  # Single numeric reading_minutes is accepted
  res5 <- detect_all_events(example_data_5_subject, reading_minutes = 5)
  expect_true(is.list(res5))

  # reading_minutes vector of wrong length should error
  expect_error(detect_all_events(example_data_5_subject, reading_minutes = c(5, 5)),
               "reading_minutes vector length must match data length or be a single value")

  # Empty input returns empty data.frame
  res_empty <- detect_all_events(empty_cgm)
  expect_true(is.list(res_empty))
  expect_true(nrow(res_empty$events_long_df) == 0)
  expect_true(nrow(res_empty$summary_df) == 0)
})

test_that("find_local_maxima wrapper returns expected components", {
  res <- find_local_maxima(example_data_5_subject)
  expect_true(is.list(res))
  expect_true(all(c("local_maxima_vector", "merged_results") %in% names(res)))
  expect_true(is.data.frame(res$local_maxima_vector))
  expect_true(is.data.frame(res$merged_results))
})

test_that("grid wrapper returns expected components and validates", {
  res <- grid(example_data_5_subject, gap = 15, threshold = 130)
  expect_true(is.list(res))
  expect_true(all(c("grid_vector", "episode_counts", "episode_start") %in% names(res)))
  expect_true(is.data.frame(res$grid_vector))

  expect_error(grid(example_data_5_subject, gap = -1),
               "gap must be between 0 and Inf")
  expect_error(grid(example_data_5_subject, threshold = -1),
               "threshold must be between 0 and Inf")
})

test_that("maxima_grid wrapper returns list with expected components and validates", {
  res <- maxima_grid(example_data_5_subject, threshold = 130, gap = 60, hours = 2)
  expect_true(is.list(res))
  expect_true(all(c("results", "episode_counts") %in% names(res)))
  expect_true(is.data.frame(res$results))

  expect_error(maxima_grid(example_data_5_subject, threshold = -1),
               "threshold must be between 0 and Inf")
  expect_error(maxima_grid(example_data_5_subject, gap = -1),
               "gap must be between 0 and Inf")
  expect_error(maxima_grid(example_data_5_subject, hours = -1),
               "hours must be between 0 and Inf")
})

test_that("excursion wrapper returns list and validates", {
  res <- excursion(example_data_5_subject, gap = 15)
  expect_true(is.list(res))
  expect_true(all(c("excursion_vector", "episode_counts", "episode_start") %in% names(res)))

  expect_error(excursion(example_data_5_subject, gap = -1),
               "gap must be between 0 and Inf")
})

test_that("start_finder wrapper returns expected type", {
  res <- start_finder(example_data_5_subject)
  expect_true(is.list(res) || is.data.frame(res))
})

test_that("find_max/min before/after hours wrappers validate inputs and return data", {
  # Use small subset to form a valid start point df
  start_points <- head(example_data_5_subject, 10)

  r1 <- find_max_after_hours(example_data_5_subject, start_points, hours = 1)
  expect_true(is.list(r1) || is.data.frame(r1))
  expect_error(find_max_after_hours(example_data_5_subject, start_points, hours = -1),
               "hours must be between 0 and Inf")

  r2 <- find_max_before_hours(example_data_5_subject, start_points, hours = 1)
  expect_true(is.list(r2) || is.data.frame(r2))
  expect_error(find_max_before_hours(example_data_5_subject, start_points, hours = -1),
               "hours must be between 0 and Inf")

  r3 <- find_min_after_hours(example_data_5_subject, start_points, hours = 1)
  expect_true(is.list(r3) || is.data.frame(r3))
  expect_error(find_min_after_hours(example_data_5_subject, start_points, hours = -1),
               "hours must be between 0 and Inf")

  r4 <- find_min_before_hours(example_data_5_subject, start_points, hours = 1)
  expect_true(is.list(r4) || is.data.frame(r4))
  expect_error(find_min_before_hours(example_data_5_subject, start_points, hours = -1),
               "hours must be between 0 and Inf")
})

test_that("mod_grid works with grid results as grid_point_df", {
  gr <- grid(example_data_5_subject, gap = 15, threshold = 130)
  mg <- mod_grid(example_data_5_subject, gr$grid_vector, hours = 2, gap = 15)
  expect_true(is.list(mg))
  expect_true(is.data.frame(mg$mod_grid_vector))
})

test_that("end-to-end pipeline produces compatible frames for transform_df and detect_between_maxima", {
  gap <- 15
  threshold <- 130
  hours <- 2

  # 1) GRID points
  grid_result <- grid(example_data_5_subject, gap = gap, threshold = threshold)
  expect_true(is.list(grid_result))
  expect_true(is.data.frame(grid_result$episode_start))

  # 2) Modified GRID points before 2 hours minimum
  mod_grid_result <- mod_grid(example_data_5_subject,
                              start_finder(grid_result$grid_vector),
                              hours = hours,
                              gap = gap)
  expect_true(is.list(mod_grid_result) || is.data.frame(mod_grid_result))

  # 3) Max point 2 hours after mod_grid point
  max_after <- find_max_after_hours(example_data_5_subject,
                                    start_finder(mod_grid_result$mod_grid_vector),
                                    hours = hours)
  expect_true(is.list(max_after) || is.data.frame(max_after))

  # 4) Local maxima
  local_maxima <- find_local_maxima(example_data_5_subject)
  expect_true(is.list(local_maxima))

  # 5) Among local maxima, find maximum point after two hours
  final_maxima <- find_new_maxima(example_data_5_subject,
                                  max_after$max_index,
                                  local_maxima$local_maxima_vector)
  expect_true(is.data.frame(final_maxima))

  # 6) Map GRID points to maximum points (within 4 hours)
  transformed <- transform_df(grid_result$episode_start, final_maxima)
  expect_true(is.data.frame(transformed))

  # 7) Redistribute overlapping maxima between GRID points
  between <- detect_between_maxima(example_data_5_subject, transformed)
  expect_true(is.list(between) || is.data.frame(between))
})


test_that("detect_hyperglycemic_events works correctly", {
  # Test Level 1 Hyperglycemia Event (≥15 consecutive min of >180 mg/dL)
  result_lv1 <- detect_hyperglycemic_events(
    example_data_5_subject, 
    start_gl = 180, 
    dur_length = 15, 
    end_length = 15, 
    end_gl = 180
  )
  
  # Test that the function returns the expected structure
  expect_true(is.list(result_lv1))
  expect_true("events_detailed" %in% names(result_lv1))
  expect_true("events_total" %in% names(result_lv1))

  
  
  # Test Level 2 Hyperglycemia Event (≥15 consecutive min of >250 mg/dL)
  result_lv2 <- detect_hyperglycemic_events(
    example_data_5_subject, 
    start_gl = 250, 
    dur_length = 15, 
    end_length = 15, 
    end_gl = 250
  )
  
  expect_true(is.list(result_lv2))
  expect_true(is.data.frame(result_lv2$events_detailed))
  expect_true(is.data.frame(result_lv2$events_total))

  # Test Extended Hyperglycemia Event (default parameters)
  result_extended <- detect_hyperglycemic_events(example_data_5_subject)
  
  expect_true(is.list(result_extended))
  expect_true(is.data.frame(result_extended$events_detailed))
  expect_true(is.data.frame(result_extended$events_total))
})

test_that("detect_hyperglycemic_events handles edge cases", {
  # Test with empty data - create proper data types
  empty_data <- data.frame(
    id = character(0), 
    time = as.POSIXct(character(0), tz = "UTC"), 
    gl = numeric(0),
    stringsAsFactors = FALSE
  )
  
  result_empty <- detect_hyperglycemic_events(empty_data)
  
  expect_true(is.list(result_empty))
  expect_equal(nrow(result_empty$events_detailed), 0)
  expect_true(is.data.frame(result_empty$events_total))
  expect_equal(nrow(result_empty$events_total), 0)
})

test_that("detect_hyperglycemic_events handles empty data", {
  # Test with empty data - should return empty results without crashing
  empty_data <- data.frame(
    id = character(0), 
    time = character(0),  
    gl = numeric(0)
  )
  
  # This should NOT crash - it should return empty results
  result <- detect_hyperglycemic_events(empty_data)
  expect_true(is.list(result))
  expect_equal(nrow(result$events_detailed), 0)
  expect_equal(nrow(result$events_total), 0)
})

test_that("detect_hyperglycemic_events validates parameters", {
  # Test with invalid parameters
  expect_error(detect_hyperglycemic_events(example_data_5_subject, dur_length = -1), "dur_length must be between 0 and Inf")
  expect_error(detect_hyperglycemic_events(example_data_5_subject, start_gl = -1), "start_gl must be between 0 and Inf")
})

# Note: Parameter validation tests removed as the function doesn't validate input parameters

test_that("detect_hyperglycemic_events returns expected column names", {
  result <- detect_hyperglycemic_events(example_data_5_subject, start_gl = 180, dur_length = 15, end_length = 15, end_gl = 180)
  
  # Check events_detailed columns
  expected_cols <- c("id", "start_time", "start_glucose", "end_time", "end_glucose", 
                     "start_index", "end_index")
  expect_true(all(expected_cols %in% names(result$events_detailed)))
  
  # Check events_total columns
  expected_total_cols <- c("id", "total_episodes", "avg_ep_per_day")
  expect_true(all(expected_total_cols %in% names(result$events_total)))
})

test_that("detect_hyperglycemic_events with different parameters produces different results", {
  # Level 1 vs Level 2 should produce different results
  result_lv1 <- detect_hyperglycemic_events(example_data_5_subject, start_gl = 180, dur_length = 15, end_length = 15, end_gl = 180)
  result_lv2 <- detect_hyperglycemic_events(example_data_5_subject, start_gl = 250, dur_length = 15, end_length = 15, end_gl = 250)
  
  # They should have different number of events (Level 1 should detect more events than Level 2)
  total_lv1 <- sum(result_lv1$events_total$total_episodes)
  total_lv2 <- sum(result_lv2$events_total$total_episodes)
  
  expect_true(total_lv1 >= total_lv2, info = "Level 1 hyperglycemia should detect more or equal events than Level 2")
})

library(testthat)
library(cgmguru)

test_that("find_local_maxima finds expected peaks on synthetic series and respects ids", {
  # Synthetic two-id series with clear local maxima
  times <- as.POSIXct(
    c(
      "2024-01-01 00:00:00","2024-01-01 00:05:00","2024-01-01 00:10:00",
      "2024-01-01 00:15:00","2024-01-01 00:20:00","2024-01-01 00:25:00",
      "2024-01-01 00:00:00","2024-01-01 00:05:00","2024-01-01 00:10:00",
      "2024-01-01 00:15:00","2024-01-01 00:20:00"
    ), tz = "UTC"
  )
  df <- data.frame(
    id = c(rep("A", 6), rep("B", 5)),
    time = times,
    gl = c(
      # id A: rise to 120, drop, rise to 110
      80, 100, 120, 90, 110, 100,
      # id B: single peak at 130
      90, 130, 100, 95, 92
    ),
    stringsAsFactors = FALSE
  )

  res <- find_local_maxima(df)
  expect_true(is.list(res))
  expect_true(all(c("local_maxima_vector", "merged_results") %in% names(res)))

  # The local_maxima_vector contains row index of df (1-based)
  peaks_idx_df <- res$local_maxima_vector
  expect_true(is.data.frame(peaks_idx_df))
  expect_true("local_maxima" %in% names(peaks_idx_df))

  peaks_idx <- peaks_idx_df$local_maxima
  # Map index back to original rows
  if (length(peaks_idx) > 0) {
    peak_rows <- df[peaks_idx, ]
    # Each detected index should be a local maximum within its id neighborhood
    for (idx in peaks_idx) {
      this_id <- df$id[idx]
      id_indices <- which(df$id == this_id)
      pos_in_id <- which(id_indices == which(seq_len(nrow(df)) == idx))
      # compare with neighbors within the same id if they exist
      if (pos_in_id > 1) {
        left_idx <- id_indices[pos_in_id - 1]
        expect_gte(df$gl[idx], df$gl[left_idx])
      }
      if (pos_in_id < length(id_indices)) {
        right_idx <- id_indices[pos_in_id + 1]
        expect_gte(df$gl[idx], df$gl[right_idx])
      }
    }
  } else {
    # If no peaks are detected, that's acceptable for this synthetic series
    succeed()
  }
})

library(testthat)
library(cgmguru)
library(iglu)

data(example_data_5_subject)

make_starts <- function(df) {
	gr <- grid(df, gap = 15, threshold = 130)
	start_finder(gr$grid_vector)
}

within_window_after <- function(df, start_idx, target_idx, hours) {
	t0 <- df$time[start_idx]
	t1 <- df$time[target_idx]
	!is.na(t0) && !is.na(t1) && t1 >= t0 && as.numeric(difftime(t1, t0, units = "hours")) <= hours + 1e-9
}

within_window_before <- function(df, start_idx, target_idx, hours) {
	t0 <- df$time[start_idx]
	t1 <- df$time[target_idx]
	!is.na(t0) && !is.na(t1) && t1 <= t0 && as.numeric(difftime(t0, t1, units = "hours")) <= hours + 1e-9
}

test_that("find_max_after_hours returns index within window", {
	starts <- make_starts(example_data_5_subject)
	res <- find_max_after_hours(example_data_5_subject, starts, hours = 2)
	expect_true(is.list(res) || is.data.frame(res))
	idx <- res$max_index$max_index
	if (length(idx) > 0) {
		# Check window for a sample of pairs (align by length)
		k <- min(length(idx), nrow(starts))
		for (i in seq_len(k)) {
			expect_true(within_window_after(example_data_5_subject, starts$start_index[i], idx[i], 2))
		}
	} else {
		succeed()
	}
})

test_that("find_max_before_hours returns index within window", {
	starts <- make_starts(example_data_5_subject)
	res <- find_max_before_hours(example_data_5_subject, starts, hours = 2)
	expect_true(is.list(res) || is.data.frame(res))
	idx <- res$max_index$max_index
	if (length(idx) > 0) {
		k <- min(length(idx), nrow(starts))
		for (i in seq_len(k)) {
			expect_true(within_window_before(example_data_5_subject, starts$start_index[i], idx[i], 2))
		}
	} else {
		succeed()
	}
})

test_that("find_min_after_hours returns index within window", {
	starts <- make_starts(example_data_5_subject)
	res <- find_min_after_hours(example_data_5_subject, starts, hours = 2)
	expect_true(is.list(res) || is.data.frame(res))
	idx <- res$min_index$min_index
	if (length(idx) > 0) {
		k <- min(length(idx), nrow(starts))
		for (i in seq_len(k)) {
			expect_true(within_window_after(example_data_5_subject, starts$start_index[i], idx[i], 2))
		}
	} else {
		succeed()
	}
})

test_that("find_min_before_hours returns index within window", {
	starts <- make_starts(example_data_5_subject)
	res <- find_min_before_hours(example_data_5_subject, starts, hours = 2)
	expect_true(is.list(res) || is.data.frame(res))
	idx <- res$min_index$min_index
	if (length(idx) > 0) {
		k <- min(length(idx), nrow(starts))
		for (i in seq_len(k)) {
			expect_true(within_window_before(example_data_5_subject, starts$start_index[i], idx[i], 2))
		}
	} else {
		succeed()
	}
})

library(testthat)
library(cgmguru)
library(iglu)

data(example_data_5_subject)

test_that("find_new_maxima handles no-maxima gracefully", {
  df <- example_data_5_subject
  df <- df[order(df$id, df$time), ]
  df$gl <- ave(df$gl, df$id, FUN = function(x) seq_along(x))
  loc <- find_local_maxima(df)$local_maxima_vector
  expect_equal(nrow(loc), 0)

  gr <- grid(df, gap = 15, threshold = 500)
  mg <- mod_grid(df, gr$grid_vector, hours = 1, gap = 15)
  max_after <- find_max_after_hours(df, start_finder(mg$mod_grid_vector), hours = 1)
  fnm <- find_new_maxima(df, max_after$max_index, loc)
  expect_true(is.data.frame(fnm))
})

test_that("find_new_maxima accepts shuffled maxima input without cross-id issues", {
  df <- example_data_5_subject
  loc <- find_local_maxima(df)$local_maxima_vector
  gr <- grid(df, gap = 15, threshold = 130)
  mg <- mod_grid(df, gr$grid_vector, hours = 2, gap = 15)
  max_after <- find_max_after_hours(df, start_finder(mg$mod_grid_vector), hours = 2)
  loc_shuffled <- loc[sample(seq_len(nrow(loc))), , drop = FALSE]
  out <- find_new_maxima(df, max_after$max_index, loc_shuffled)
  expect_true(is.data.frame(out))
})

library(testthat)
library(cgmguru)
library(iglu)

data(example_data_5_subject)

test_that("overrides forward clear messages for bad df", {
  expect_error(detect_all_events(1), "Error in detect_all_events(): Input must be a data frame", fixed = TRUE)
  expect_error(grid(data.frame(a=1), 15, 130), "Error in grid(): Missing required columns", fixed = TRUE)
})

test_that("numeric param validators triggered in overrides", {
  df <- example_data_5_subject
  expect_error(grid(df, gap = -1), "gap must be between 0 and Inf")
  expect_error(maxima_grid(df, hours = -1), "hours must be between 0 and Inf")
  expect_error(detect_hyperglycemic_events(df, start_gl = -1), "start_gl must be between 0 and Inf")
  expect_error(detect_all_events(df, reading_minutes = c(1,2,3)), "vector length must match")
})

library(testthat)
library(cgmguru)
library(iglu)

data(example_data_5_subject)

make_sparse_grid_scope_data <- function() {
  data.frame(
    id = "A",
    time = as.POSIXct("2026-01-01 00:05:00", tz = "UTC") + c(0, 10, 15, 20, 25) * 60,
    gl = c(100, 120, 140, 160, 150)
  )
}

test_that("grid returns expected components and validates params", {
  res <- grid(example_data_5_subject, gap = 15, threshold = 130)
  expect_true(is.list(res))
  expect_true(all(c("grid_vector", "episode_counts", "episode_start") %in% names(res)))
  expect_true(is.data.frame(res$grid_vector))

  expect_error(grid(example_data_5_subject, gap = -1), "gap must be between 0 and Inf")
  expect_error(grid(example_data_5_subject, threshold = -1), "threshold must be between 0 and Inf")
})

test_that("maxima_grid returns expected components and validates", {
  res <- maxima_grid(example_data_5_subject, threshold = 130, gap = 60, hours = 2)
  expect_true(is.list(res))
  expect_true(all(c("results", "episode_counts") %in% names(res)))
  expect_true(is.data.frame(res$results))

  expect_error(maxima_grid(example_data_5_subject, threshold = -1), "threshold must be between 0 and Inf")
  expect_error(maxima_grid(example_data_5_subject, gap = -1), "gap must be between 0 and Inf")
  expect_error(maxima_grid(example_data_5_subject, hours = -1), "hours must be between 0 and Inf")
})

test_that("excursion returns expected components and validates gap", {
  res <- excursion(example_data_5_subject, gap = 15)
  expect_true(is.list(res))
  expect_true(all(c("excursion_vector", "episode_counts", "episode_start") %in% names(res)))

  expect_error(excursion(example_data_5_subject, gap = -1), "gap must be between 0 and Inf")
})

test_that("GRID and excursion operate on supplied rows, not the event grid", {
  raw <- make_sparse_grid_scope_data()
  interpolated <- interpolate_cgm(raw, reading_minutes = 5)

  expect_gt(nrow(interpolated), nrow(raw))
  expect_equal(nrow(grid(raw, gap = 0, threshold = 105)$grid_vector), nrow(raw))
  expect_equal(nrow(excursion(raw, gap = 0)$excursion_vector), nrow(raw))
  expect_equal(nrow(grid(interpolated, gap = 0, threshold = 105)$grid_vector), nrow(interpolated))
  expect_equal(nrow(excursion(interpolated, gap = 0)$excursion_vector), nrow(interpolated))
})

test_that("maxima_grid uses supplied timestamps unless interpolated data is passed", {
  raw <- make_sparse_grid_scope_data()
  interpolated <- interpolate_cgm(raw, reading_minutes = 5)

  raw_result <- maxima_grid(raw, threshold = 105, gap = 0, hours = 2)
  interpolated_result <- maxima_grid(interpolated, threshold = 105, gap = 0, hours = 2)

  expect_gt(nrow(raw_result$results), 0)
  expect_gt(nrow(interpolated_result$results), 0)
  expect_true(all(raw_result$results$grid_time %in% raw$time))
  expect_true(all(raw_result$results$maxima_time %in% raw$time))
  expect_true(all(interpolated_result$results$grid_time %in% interpolated$time))
  expect_true(all(interpolated_result$results$maxima_time %in% interpolated$time))
  expect_false(identical(
    as.numeric(raw_result$results$grid_time),
    as.numeric(interpolated_result$results$grid_time)
  ))
})

# Input validation functions for cgmguru package
# These functions provide validation helpers for the safe wrapper functions

#' @noRd
canonicalize_fixed_tzone <- function(tz) {
  if (is.null(tz) || length(tz) == 0) {
    return(tz)
  }

  out <- as.character(tz)
  valid <- !is.na(out) & nzchar(out)
  out[valid] <- vapply(out[valid], function(one_tz) {
    switch(one_tz,
      EST = "Etc/GMT+5",
      MST = "Etc/GMT+7",
      HST = "Etc/GMT+10",
      one_tz
    )
  }, character(1))
  out
}

#' @noRd
canonicalize_posixct_tzone <- function(x) {
  tz <- attr(x, "tzone")
  canonical_tz <- canonicalize_fixed_tzone(tz)
  if (!is.null(tz) && length(tz) > 0 && !identical(canonical_tz, tz[1])) {
    attr(x, "tzone") <- canonical_tz
  }
  x
}

#' Validate CGM data input
#' @param df Data frame to validate
#' @param required_cols Required column names
#' @param optional_cols Optional column names
#' @return Validated and cleaned data frame
#' @noRd
validate_cgm_data <- function(df, required_cols = c("id", "time", "gl"), optional_cols = c("tz")) {
  # Check if input is a data frame
  if (!is.data.frame(df)) {
    stop("Input must be a data frame")
  }
  
  # Check if required columns exist
  missing_cols <- setdiff(required_cols, names(df))
  if (length(missing_cols) > 0) {
    stop("Missing required columns: ", paste(missing_cols, collapse = ", "))
  }
  
  # Check if data frame is empty
  if (nrow(df) == 0) {
    # Return empty data frame with correct types
    empty_df <- data.frame(
      id = character(0),
      time = as.POSIXct(character(0), tz = "UTC"),
      gl = numeric(0),
      stringsAsFactors = FALSE
    )
    # Add optional columns if they exist in original
    for (col in optional_cols) {
      if (col %in% names(df)) {
        if (col == "tz") {
          empty_df[[col]] <- character(0)
        } else {
          empty_df[[col]] <- numeric(0)
        }
      }
    }
    return(empty_df)
  }
  
  # Validate and convert column types
  validated_df <- df
  
  # Validate id column
  if (!is.character(df$id)) {
    validated_df$id <- as.character(df$id)
  }
  
  # Validate time column
  if (!inherits(df$time, "POSIXct")) {
    if (is.character(df$time)) {
      validated_df$time <- as.POSIXct(df$time, tz = "UTC")
    } else if (is.numeric(df$time)) {
      # Assume numeric time is already in POSIXct format
      validated_df$time <- as.POSIXct(df$time, origin = "1970-01-01", tz = "UTC")
    } else {
      stop("Time column must be POSIXct, character, or numeric")
    }
  }
  validated_df$time <- canonicalize_posixct_tzone(validated_df$time)
  
  # Validate glucose column
  if (!is.numeric(df$gl)) {
    validated_df$gl <- as.numeric(df$gl)
  }
  
  # Validate optional timezone column
  if ("tz" %in% names(df)) {
    if (!is.character(df$tz)) {
      validated_df$tz <- as.character(df$tz)
    }
    validated_df$tz <- canonicalize_fixed_tzone(validated_df$tz)
  }
  
  # Check for NA values in critical columns
  if (any(is.na(validated_df$id))) {
    warning("Found NA values in id column, removing those rows")
    validated_df <- validated_df[!is.na(validated_df$id), ]
  }
  
  if (any(is.na(validated_df$time))) {
    warning("Found NA values in time column, removing those rows")
    validated_df <- validated_df[!is.na(validated_df$time), ]
  }
  
  return(validated_df)
}

#' Validate numeric parameters
#' @param param Parameter value to validate
#' @param param_name Name of parameter for error messages
#' @param min_val Minimum allowed value
#' @param max_val Maximum allowed value
#' @return Validated parameter
#' @noRd
validate_numeric_param <- function(param, param_name, min_val = -Inf, max_val = Inf) {
  if (!is.numeric(param) || length(param) != 1) {
    stop(param_name, " must be a single numeric value")
  }
  
  if (is.na(param)) {
    stop(param_name, " cannot be NA")
  }
  
  if (param < min_val || param > max_val) {
    stop(param_name, " must be between ", min_val, " and ", max_val)
  }
  
  return(param)
}

#' Validate reading_minutes parameter
#' @param reading_minutes Parameter to validate
#' @param data_length Length of data for validation
#' @return Validated parameter
#' @noRd
validate_reading_minutes <- function(reading_minutes, data_length) {
  if (!is.null(reading_minutes)) {
    if (is.numeric(reading_minutes)) {
      if (length(reading_minutes) == 1) {
        reading_minutes <- validate_numeric_param(reading_minutes, "reading_minutes", min_val = 0.1)
      } else if (length(reading_minutes) != data_length) {
        stop("reading_minutes vector length must match data length or be a single value")
      }
    } else {
      stop("reading_minutes must be numeric")
    }
  }
  return(reading_minutes)
}

#' Validate logical scalar parameters
#' @param param Parameter value to validate
#' @param param_name Name of parameter for error messages
#' @return Validated logical scalar
#' @noRd
validate_logical_param <- function(param, param_name) {
  if (!is.logical(param) || length(param) != 1 || is.na(param)) {
    stop(param_name, " must be TRUE or FALSE")
  }
  return(param)
}

# =============================================================================
# VALIDATION HELPER FUNCTIONS
# =============================================================================
# Note: Safe wrapper functions are now implemented in function_overrides.R
# which overrides the original function names directly

#' Validate intermediary data frames (not raw CGM data)
#' @param df Data frame to validate
#' @return Pass-through of input data frame
#' @noRd
validate_intermediary_df <- function(df) {
  # For intermediary frames, just ensure it's a data frame
  if (!is.data.frame(df)) {
    stop("Input must be a data frame")
  }
  return(df)
}

library(testthat)
library(cgmguru)

test_that("validate_cgm_data enforces schema and converts types", {
  # Not a data.frame
  expect_error(cgmguru:::validate_cgm_data(1), "Input must be a data frame")

  # Missing required cols
  bad_df <- data.frame(a = 1:3)
  expect_error(cgmguru:::validate_cgm_data(bad_df), "Missing required columns:")

  # Proper conversion of types
  df <- data.frame(
    id = factor(c("x","y")),
    time = c("2024-01-01 00:00:00", "2024-01-01 00:05:00"),
    gl = as.character(c(100, 110)),
    stringsAsFactors = FALSE
  )
  res <- cgmguru:::validate_cgm_data(df)
  expect_true(is.character(res$id))
  expect_true(inherits(res$time, "POSIXct"))
  expect_true(is.numeric(res$gl))

  # Numeric time converts from epoch seconds
  df2 <- data.frame(
    id = c("a","b"),
    time = as.numeric(c(0, 60)),
    gl = c(1, 2),
    stringsAsFactors = FALSE
  )
  res2 <- cgmguru:::validate_cgm_data(df2)
  expect_true(inherits(res2$time, "POSIXct"))

  # Empty df returns typed empty df
  empty_in <- data.frame(id = character(0), time = as.POSIXct(character(0), tz = "UTC"), gl = numeric(0))
  empty_out <- cgmguru:::validate_cgm_data(empty_in)
  expect_equal(nrow(empty_out), 0)
  expect_true(is.character(empty_out$id))
  expect_true(inherits(empty_out$time, "POSIXct"))
  expect_true(is.numeric(empty_out$gl))

  # NA in id/time are removed with warning
  df_na <- data.frame(
    id = c("a", NA),
    time = as.POSIXct(c("2024-01-01 00:00:00", NA), tz = "UTC"),
    gl = c(100, 110),
    stringsAsFactors = FALSE
  )
  expect_warning(res_na <- cgmguru:::validate_cgm_data(df_na), "NA values in id column|NA values in time column")
  expect_equal(nrow(res_na), 1)
})

test_that("tz column coerces to character", {
  df <- data.frame(
    id = c("a","b"),
    time = as.POSIXct(c("2024-01-01 00:00:00","2024-01-01 00:05:00"), tz = "UTC"),
    gl = 1:2,
    tz = factor(c("UTC","EST")),
    stringsAsFactors = FALSE
  )
  out <- cgmguru:::validate_cgm_data(df)
  expect_true(is.character(out$tz))
  expect_equal(out$tz, c("UTC", "Etc/GMT+5"))
})

test_that("fixed-offset time zones are canonicalized portably", {
  # Avoid platform-specific parsing of the legacy "EST" timezone name.
  est_time <- as.POSIXct("2024-01-01 00:00:00", tz = "Etc/GMT+5")
  attr(est_time, "tzone") <- "EST"

  df <- data.frame(
    id = "a",
    time = est_time,
    gl = 100,
    stringsAsFactors = FALSE
  )
  original_time <- df$time

  out <- cgmguru:::validate_cgm_data(df)

  expect_identical(attr(out$time, "tzone"), "Etc/GMT+5")
  expect_equal(as.numeric(out$time), as.numeric(original_time))
  expect_equal(format(out$time, "%Y-%m-%d %H:%M:%S"), "2024-01-01 00:00:00")
})

test_that("validate_numeric_param enforces numeric scalar and bounds", {
  expect_error(cgmguru:::validate_numeric_param("a", "p"), "must be a single numeric value")
  expect_error(cgmguru:::validate_numeric_param(c(1,2), "p"), "must be a single numeric value")
  expect_error(cgmguru:::validate_numeric_param(NA_real_, "p"), "cannot be NA")
  expect_error(cgmguru:::validate_numeric_param(-1, "p", min_val = 0), "between 0 and Inf")
  # Accept bounds
  expect_equal(cgmguru:::validate_numeric_param(0, "p", min_val = 0), 0)
  expect_equal(cgmguru:::validate_numeric_param(1, "p", min_val = 0, max_val = 1), 1)
})

test_that("validate_reading_minutes handles NULL, scalar, vector, and errors", {
  # NULL passthrough
  expect_null(cgmguru:::validate_reading_minutes(NULL, 10))
  # Scalar >= 0.1 ok
  expect_equal(cgmguru:::validate_reading_minutes(5, 10), 5)
  # Too small scalar
  expect_error(cgmguru:::validate_reading_minutes(0.01, 10), "reading_minutes")
  # Length mismatch
  expect_error(cgmguru:::validate_reading_minutes(c(1,2), 3), "vector length must match")
  # Non-numeric
  expect_error(cgmguru:::validate_reading_minutes("a", 3), "must be numeric")
})

test_that("validate_intermediary_df accepts data.frame and rejects others", {
  expect_error(cgmguru:::validate_intermediary_df(1), "Input must be a data frame")
  df <- data.frame(x = 1:3)
  expect_identical(cgmguru:::validate_intermediary_df(df), df)
})

#' Fast Ordering Function
#'
#' Orders a dataframe by id and time columns using the C++ backend
#'
#' @param df A dataframe with 'id' and 'time' columns
#' @return A dataframe ordered by id and time
#' @export
#' @examples
#' df <- data.frame(id = c("b", "a", "a"), time = as.POSIXct(
#'   c("2024-01-01 01:00:00", "2024-01-01 00:00:00", "2024-01-01 01:00:00"), tz = "UTC"
#' ))
#' orderfast(df)
orderfast <- function(df) {
  orderfast_cpp(df)
}

library(testthat)
library(cgmguru)
library(iglu)

# Use example CGM data from iglu/cgmguru
data(example_data_5_subject)

test_that("orderfast orders by id then time", {
  set.seed(123)
  shuffled <- example_data_5_subject[sample(seq_len(nrow(example_data_5_subject))), ]
  ordered <- orderfast(shuffled)
  expected <- shuffled[order(shuffled$id, shuffled$time), ]

  # Same dimensions and columns preserved
  expect_equal(nrow(ordered), nrow(shuffled))
  expect_equal(ncol(ordered), ncol(shuffled))
  expect_equal(names(ordered), names(shuffled))
  expect_identical(ordered, expected)

  # Check that data is globally ordered by id then time
  expect_true(all(order(ordered$id, ordered$time) == seq_len(nrow(ordered))))

  # Within each id, time should be non-decreasing
  by_id_ok <- by(ordered$time, ordered$id, function(tt) all(diff(as.numeric(tt)) >= 0))
  expect_true(all(unlist(by_id_ok)))
})

test_that("orderfast respects factor level ordering like base order", {
  df <- data.frame(
    id = factor(c("a", "b", "a", "b"), levels = c("b", "a")),
    time = as.POSIXct(c(
      "2024-01-01 00:30:00",
      "2024-01-01 00:15:00",
      "2024-01-01 00:15:00",
      "2024-01-01 00:30:00"
    ), tz = "UTC"),
    gl = 1:4
  )

  expect_identical(orderfast(df), df[order(df$id, df$time), ])
})

test_that("orderfast is idempotent on already ordered data", {
  already <- orderfast(example_data_5_subject)
  again <- orderfast(already)
  expect_identical(already, again)
})

test_that("orderfast handles empty data frame with required columns", {
  empty_df <- data.frame(
    id = character(0),
    time = as.POSIXct(character(0), tz = "UTC"),
    gl = numeric(0),
    stringsAsFactors = FALSE
  )
  res <- orderfast(empty_df)
  expect_true(is.data.frame(res))
  expect_equal(nrow(res), 0)
  expect_equal(names(res), names(empty_df))
})

test_that("orderfast handles ties in id and time deterministically", {
  # Create small data with deliberate ties
  df <- data.frame(
    id = c("a", "a", "b", "a", "b"),
    time = as.POSIXct(c(
      "2024-01-01 00:00:00",
      "2024-01-01 00:00:00", # tie with first row
      "2024-01-01 01:00:00",
      "2024-01-01 00:30:00",
      "2024-01-01 00:00:00"
    ), tz = "UTC"),
    gl = c(100, 110, 120, 90, 95),
    stringsAsFactors = FALSE
  )

  set.seed(1)
  df_shuffled <- df[sample(seq_len(nrow(df))), ]
  ord <- orderfast(df_shuffled)

  # Check primary ordering by id then time
  expect_true(all(order(ord$id, ord$time) == seq_len(nrow(ord))))

  # Ties kept but relative order among perfect ties may change; ensure the tied block is contiguous
  tie_block <- ord[ord$id == "a" & ord$time == as.POSIXct("2024-01-01 00:00:00", tz = "UTC"), ]
  expect_equal(nrow(tie_block), 2)
})

test_that("orderfast places NA times last within each id and preserves NAs in id", {
  df <- data.frame(
    id = c("a", NA, "a", "b", "b"),
    time = as.POSIXct(c(
      "2024-01-01 01:00:00",
      "2024-01-01 00:00:00",
      NA,
      "2024-01-01 00:00:00",
      NA
    ), tz = "UTC"),
    gl = 1:5,
    stringsAsFactors = FALSE
  )

  ord <- orderfast(df)

  # Non-NA ids should come before NA id when ordering
  expect_true(any(is.na(ord$id)))

  # Within id 'a', NA time should be placed last
  a_rows <- ord[ord$id == "a", ]
  expect_true(is.na(tail(a_rows$time, 1)))

  # Within id 'b', NA time should be placed last
  b_rows <- ord[ord$id == "b", ]
  expect_true(is.na(tail(b_rows$time, 1)))
})

library(cgmguru)
library(iglu)

data(example_data_5_subject)
data(example_data_hall)

# Detect all glycemic events
# reading_minutes is calculated automatically from timestamp spacing when omitted
all_events <- detect_all_events(example_data_5_subject)
print(all_events$summary_df)
print(all_events$events_long_df)

# Detect all events on larger dataset
large_all_events <- detect_all_events(example_data_hall)
print(paste("Total subjects analyzed:", nrow(large_all_events$summary_df)))

# Filter for specific event types
hyperglycemia_events <- all_events$events_long_df[all_events$events_long_df$type == "hyper", ]
hypoglycemia_events <- all_events$events_long_df[all_events$events_long_df$type == "hypo", ]

print("Hyperglycemia events:")
print(hyperglycemia_events)
print("Hypoglycemia events:")
print(hypoglycemia_events)

sensor_wear <- function(df, end_date = NULL, ndays = 14,
                        reading_minutes = NULL) {
  tryCatch({
    validated_df <- validate_cgm_data(df)
  }, error = function(e) {
    stop("Error in sensor_wear(): ", e$message, call. = FALSE)
  })

  reading_minutes <- validate_reading_minutes(reading_minutes, nrow(validated_df))
  ndays <- validate_numeric_param(ndays, "ndays", min_val = 0.1)

  if (!is.null(end_date)) {
    if (length(end_date) != 1) {
      stop("end_date must be NULL or a single Date/POSIXct value", call. = FALSE)
    }
    tz <- attr(validated_df$time, "tzone")
    if (is.null(tz) || length(tz) == 0 || is.na(tz[1])) {
      tz <- ""
    } else {
      tz <- tz[1]
    }
    end_date <- as.POSIXct(end_date, tz = tz)
    if (is.na(end_date)) {
      stop("end_date must be convertible to POSIXct", call. = FALSE)
    }
  }

  tryCatch({
    sensor_wear_cpp(validated_df, reading_minutes, end_date, ndays)
  }, error = function(e) {
    stop("Error in sensor_wear: ", e$message, call. = FALSE)
  })
}

library(testthat)
library(cgmguru)
library(iglu)

data(example_data_5_subject)

test_that("sensor_wear matches iglu manual active_percent with per-id end dates", {
  cg <- sensor_wear(example_data_5_subject, ndays = 14, reading_minutes = 5)
  ig <- iglu::active_percent(
    example_data_5_subject,
    dt0 = 5,
    range_type = "manual",
    ndays = 14
  )

  cmp <- merge(
    cg[, c("id", "sensor_wear", "ndays", "start_date", "end_date")],
    ig[, c("id", "active_percent", "ndays", "start_date", "end_date")],
    by = "id",
    suffixes = c("_cg", "_ig")
  )

  expect_equal(cmp$sensor_wear, cmp$active_percent, tolerance = 1e-8)
  expect_equal(as.numeric(cmp$start_date_cg), as.numeric(cmp$start_date_ig),
               tolerance = 1e-8)
  expect_equal(as.numeric(cmp$end_date_cg), as.numeric(cmp$end_date_ig),
               tolerance = 1e-8)
})

test_that("sensor_wear matches iglu manual active_percent with a common end_date", {
  end_date <- max(example_data_5_subject$time, na.rm = TRUE)

  cg <- sensor_wear(
    example_data_5_subject,
    end_date = end_date,
    ndays = 14,
    reading_minutes = 5
  )
  ig <- iglu::active_percent(
    example_data_5_subject,
    dt0 = 5,
    range_type = "manual",
    ndays = 14,
    consistent_end_date = end_date
  )

  cmp <- merge(
    cg[, c("id", "sensor_wear", "start_date", "end_date")],
    ig[, c("id", "active_percent", "start_date", "end_date")],
    by = "id",
    suffixes = c("_cg", "_ig")
  )

  expect_equal(cmp$sensor_wear, cmp$active_percent, tolerance = 1e-8)
  expect_equal(as.numeric(cmp$start_date_cg), as.numeric(cmp$start_date_ig),
               tolerance = 1e-8)
  expect_equal(as.numeric(cmp$end_date_cg), as.numeric(cmp$end_date_ig),
               tolerance = 1e-8)
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

detect_all_events <- function(df, reading_minutes = NULL, sort_time = FALSE, inter_gap = 45, return_interpolated = FALSE) {
    .Call(`_cgmguru_detect_all_events`, df, reading_minutes, sort_time, inter_gap, return_interpolated)
}

detect_between_maxima <- function(df, transform_df) {
    .Call(`_cgmguru_detect_between_maxima`, df, transform_df)
}

detect_hyperglycemic_events <- function(df, reading_minutes = NULL, dur_length = 120, end_length = 15, start_gl = 250, end_gl = 180, sort_time = FALSE, inter_gap = 45, return_interpolated = TRUE, lv1_excl = FALSE) {
    .Call(`_cgmguru_detect_hyperglycemic_events`, df, reading_minutes, dur_length, end_length, start_gl, end_gl, sort_time, inter_gap, return_interpolated, lv1_excl)
}

detect_hypoglycemic_events <- function(df, reading_minutes = NULL, dur_length = 120, end_length = 15, start_gl = 70, sort_time = FALSE, inter_gap = 45, return_interpolated = TRUE, lv1_excl = FALSE) {
    .Call(`_cgmguru_detect_hypoglycemic_events`, df, reading_minutes, dur_length, end_length, start_gl, sort_time, inter_gap, return_interpolated, lv1_excl)
}

excursion <- function(df, gap = 15) {
    .Call(`_cgmguru_excursion`, df, gap)
}

find_local_maxima <- function(df) {
    .Call(`_cgmguru_find_local_maxima`, df)
}

find_max_after_hours <- function(df, start_point_df, hours) {
    .Call(`_cgmguru_find_max_after_hours`, df, start_point_df, hours)
}

find_max_before_hours <- function(df, start_point_df, hours) {
    .Call(`_cgmguru_find_max_before_hours`, df, start_point_df, hours)
}

find_min_after_hours <- function(df, start_point_df, hours) {
    .Call(`_cgmguru_find_min_after_hours`, df, start_point_df, hours)
}

find_min_before_hours <- function(df, start_point_df, hours) {
    .Call(`_cgmguru_find_min_before_hours`, df, start_point_df, hours)
}

find_new_maxima <- function(df, mod_grid_max_point_df, local_maxima_df) {
    .Call(`_cgmguru_find_new_maxima`, df, mod_grid_max_point_df, local_maxima_df)
}

grid <- function(df, gap = 15, threshold = 130) {
    .Call(`_cgmguru_grid`, df, gap, threshold)
}

interpolate_cgm_cpp <- function(df, reading_minutes = NULL, sort_time = FALSE, inter_gap = 45) {
    .Call(`_cgmguru_interpolate_cgm_cpp`, df, reading_minutes, sort_time, inter_gap)
}

maxima_grid <- function(df, threshold = 130, gap = 60, hours = 2) {
    .Call(`_cgmguru_maxima_grid`, df, threshold, gap, hours)
}

mod_grid <- function(df, grid_point_df, hours = 2, gap = 15) {
    .Call(`_cgmguru_mod_grid`, df, grid_point_df, hours, gap)
}

orderfast_cpp <- function(df) {
    .Call(`_cgmguru_orderfast_cpp`, df)
}

sensor_wear_cpp <- function(df, reading_minutes = NULL, end_date = NULL, ndays = 14.0) {
    .Call(`_cgmguru_sensor_wear_cpp`, df, reading_minutes, end_date, ndays)
}

start_finder <- function(df) {
    .Call(`_cgmguru_start_finder`, df)
}

transform_df <- function(grid_df, maxima_df) {
    .Call(`_cgmguru_transform_df`, grid_df, maxima_df)
}

#' Advanced Continuous Glucose Monitoring Analysis and GRID-Based Event Detection
#'
#' @description
#' A high-performance R package for comprehensive Continuous Glucose Monitoring (CGM) data
#' analysis with optimized C++ implementations. The package provides advanced tools for
#' CGM data analysis with two primary capabilities: GRID and postprandial peak detection,
#' and extended glycemic events detection aligned with international consensus CGM metrics.
#'
#' @details
#' The package implements several key algorithms for CGM analysis:
#' \itemize{
#'   \item \strong{GRID Algorithm}: Detects rapid glucose rate increases (commonly \eqn{\geq} 90-95 mg/dL/hour)
#'         with configurable thresholds and gaps for postprandial peak detection
#'   \item \strong{Postprandial Peak Detection}: Finds peak glucose after GRID points using
#'         local maxima and configurable time windows
#'   \item \strong{Consensus CGM Metrics Event Detection}: Level 1/2 hypo- and hyperglycemia
#'         detection with duration validation (default minimum 15 minutes) aligned with
#'         Battelino et al. (2023) international consensus
#'   \item \strong{Advanced Analysis Tools}: Local maxima finding, excursion analysis,
#'         and robust episode validation utilities
#' }
#'
#' Core algorithms are implemented in optimized C++ via 'Rcpp' for accurate and fast
#' analysis on large datasets, making the package suitable for both research and
#' clinical applications.
#'
#' @section Relationship to iglu:
#' cgmguru uses the \pkg{iglu} package as a formal methodological reference,
#' source of example datasets, and comparison target. Event preprocessing is an
#' independent C++ implementation of iglu-compatible semantics: a midnight-aligned
#' full-day grid, interpolation up to \code{inter_gap}, removal of gap-masked
#' rows, and segment-wise event classification. cgmguru does not call \pkg{iglu}
#' at runtime for its core algorithms.
#'
#' The event preprocessing grid applies only to event functions and
#' \code{\link{interpolate_cgm}}. GRID-family functions, including
#' \code{\link{grid}}, \code{\link{maxima_grid}}, and \code{\link{excursion}},
#' operate on the input timestamps and glucose values supplied by the user.
#'
#' @section Main Functions:
#' \describe{
#'   \item{\code{\link{grid}}}{GRID algorithm for detecting rapid glucose rate increases}
#'   \item{\code{\link{maxima_grid}}}{Combined maxima detection and GRID analysis for postprandial peaks}
#'   \item{\code{\link{detect_hyperglycemic_events}}}{Hyperglycemic event detection (Level 1/2/Extended)}
#'   \item{\code{\link{detect_hypoglycemic_events}}}{Hypoglycemic event detection (Level 1/2/Extended)}
#'   \item{\code{\link{detect_all_events}}}{Comprehensive detection of all glycemic event types}
#'   \item{\code{\link{find_local_maxima}}}{Local maxima identification in glucose time series}
#'   \item{\code{\link{interpolate_cgm}}}{C++ iglu-compatible event-grid interpolation}
#'   \item{\code{\link{orderfast}}}{Fast dataframe ordering utility}
#' }
#'
#' @section Data Requirements:
#' Input dataframes should contain:
#' \itemize{
#'   \item \code{id}: Patient identifier (character or factor)
#'   \item \code{time}: POSIXct timestamps
#'   \item \code{gl}: Glucose values in mg/dL
#' }
#'
#' All function arguments and return values are expected to be in tibble format.
#' For convenience, single-column parameters can be passed as vectors in R, which
#' will be automatically converted to single-column tibbles.
#'
#' @section Examples:
#' \preformatted{
#' # Basic GRID analysis
#' result <- grid(cgm_data, gap = 15, threshold = 130)
#'
#' # Postprandial peak detection (GRID-based)
#' maxima <- maxima_grid(cgm_data, threshold = 130, gap = 60, hours = 2)
#'
#' # Level 1 Hyperglycemic event detection
#' events <- detect_hyperglycemic_events(cgm_data, type = "lv1")
#'
#' # Comprehensive event detection; reading_minutes is calculated automatically
#' all_events <- detect_all_events(cgm_data)
#' }
#'
#' @author Sang Ho Park \email{shstat1729@gmail.com}
#'
#' @references
#' \itemize{
#'   \item Battelino, T., et al. "Continuous glucose monitoring and metrics for clinical trials: an international consensus statement." *The Lancet Diabetes & Endocrinology* 11.1 (2023): 42-57.
#'   \item Harvey, Rebecca A., et al. "Design of the glucose rate increase detector: a meal detection module for the health monitoring system." *Journal of diabetes science and technology* 8.2 (2014): 307-320.
#'   \item Adolfsson, Peter, et al. "Increased time in range and fewer missed bolus injections after introduction of a smart connected insulin pen." Diabetes Technology & Therapeutics 22.10 (2020): 709-718.
#'   \item Broll, S., Urbanek, J., Buchanan, D., Chun, E., Muschelli, J., Punjabi, N., and Gaynanova, I. "Interpreting blood glucose data with R package iglu." PLoS One 16.4 (2021): e0248560. doi:10.1371/journal.pone.0248560.
#'   \item Chun, E., Fernandes, J. N., and Gaynanova, I. "An Update on the iglu Software Package for Interpreting Continuous Glucose Monitoring Data." Diabetes Technology & Therapeutics 26.12 (2024): 939-950. doi:10.1089/dia.2024.0154.
#'   \item Park, Sang Ho, et al. "Identification of clinically meaningful automatically detected postprandial glucose excursions in individuals with type 1 diabetes using personal continuous glucose monitoring." Diabetes Research and Clinical Practice (2025): 112951.
#'   \item Park, Soojin, et al. "High-Amplitude and Prolonged Glucose Excursions as a Key Determinant of Discordance Between Glucose Management Indicator and Glycated Hemoglobin in Type 1 Diabetes." Diabetes Care (2026): dc252820. https://doi.org/10.2337/dc25-2820
#'   \item Edwards, Stephanie, et al. "Use of connected pen as a diagnostic tool to evaluate missed bolus dosing behavior in people with type 1 and type 2 diabetes." Diabetes Technology & Therapeutics 24.1 (2022): 61-66.
#' }
#'
#' For more information about the GRID algorithm and CGM analysis methodologies,
#' see the package vignette: \code{vignette("intro", package = "cgmguru")}
#'
#' @seealso
#' \code{\link{grid}}, \code{\link{maxima_grid}}, \code{\link{detect_hyperglycemic_events}}, \code{\link{detect_all_events}}
#'
#' @keywords internal
#' @useDynLib cgmguru, .registration = TRUE
#' @importFrom Rcpp evalCpp
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

---
title: "cgmguru: Complete CGM Analysis Workflow"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{cgmguru: Complete CGM Analysis Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", fig.width = 10, fig.height = 6)
library(cgmguru)
library(iglu)
library(ggplot2)
library(dplyr)
set.seed(123)
```

# cgmguru: Complete CGM Analysis Workflow

The `cgmguru` package provides comprehensive tools for analyzing Continuous Glucose Monitoring (CGM) data using the GRID (Glucose Rate Increase Detector) algorithm and related methodologies. This vignette demonstrates the complete workflow from basic analysis to advanced event detection.

For glycemic event calculation, cgmguru uses an independent C++/Rcpp
implementation of iglu-compatible preprocessing semantics: a midnight-aligned
full-day grid, interpolation up to `inter_gap`, removal of larger gap-masked
rows, and segment-wise event classification. This preprocessing is specific to
event functions and `interpolate_cgm()`. GRID-family functions (`grid()`,
`maxima_grid()`, and `excursion()`) operate on the input rows supplied by the
user and do not automatically use the full-day event grid.

If `reading_minutes` is omitted or `NULL`, event functions and
`interpolate_cgm()` calculate it automatically per subject from the median
positive timestamp spacing in the data.

## Core functions at a glance

- **grid**: Implements GRID to detect rapid glucose rate increases (meal starts); returns grid vector and episode summaries.
- **mod_grid**: Re-applies GRID logic with a defined gap and window to produce a modified grid for maxima mapping.
- **maxima_grid**: Combines GRID and local maxima to identify postprandial peaks within a time window and summarizes episodes.
- **detect_hyperglycemic_events**: Finds Level 1/Level 2/Extended hyperglycemia episodes using consensus thresholds and durations.
- **detect_hypoglycemic_events**: Finds Level 1/Level 2/Extended hypoglycemia episodes and duration below 54 mg/dL where applicable.
- **detect_all_events**: Unified interface summarizing hypo/hyper event types, counts, and durations by subject.
- **find_local_maxima**: Detects peaks where glucose rises for two points before and falls for two points after the peak.
- **find_max_after_hours / find_max_before_hours**: Finds maximum glucose within a time window after/before given start points; returns index and episode counts.
- **find_min_after_hours / find_min_before_hours**: Finds minimum glucose within a time window after/before given start points.
- **find_new_maxima**: Refines maxima around candidate grid points to improve peak identification.
- **detect_between_maxima**: Characterizes events occurring between identified maxima, including time-to-peak.
- **excursion**: Calculates excursions (>70 mg/dL rise within 2 hours, not preceded by <70 mg/dL), with counts and starts.
- **start_finder**: Locates R-based positions of episode starts in a 0/1 vector (1 after 0 or at start).
- **transform_df**: Maps GRID episode starts to maxima (within 4 hours) and merges results for downstream analysis.
- **orderfast**: Efficiently orders data by `id` and `time` for large CGM datasets.

Tip: See individual help pages for details and examples, for instance:

```{r, eval=FALSE}
?grid
?detect_all_events
```

## Loading Sample Data

We'll use two datasets from the `iglu` package to demonstrate different analysis scenarios:

```{r load-data}
# Load example datasets
data(example_data_5_subject)  # 5 subjects, 13,866 readings
data(example_data_hall)       # 19 subjects, 34,890 readings

# Display basic information about the datasets
cat("Dataset 1 (example_data_5_subject):\n")
cat("  Rows:", nrow(example_data_5_subject), "\n")
cat("  Subjects:", length(unique(example_data_5_subject$id)), "\n")
cat("  Time range:", as.character(range(example_data_5_subject$time)), "\n")
cat("  Glucose range:", range(example_data_5_subject$gl), "mg/dL\n\n")

cat("Dataset 2 (example_data_hall):\n")
cat("  Rows:", nrow(example_data_hall), "\n")
cat("  Subjects:", length(unique(example_data_hall$id)), "\n")
cat("  Time range:", as.character(range(example_data_hall$time)), "\n")
cat("  Glucose range:", range(example_data_hall$gl), "mg/dL\n")

# Show first few rows
head(example_data_5_subject)
```

## 1. Basic GRID Analysis

The GRID algorithm detects rapid glucose rate increases, which are often associated with meal consumption.

```{r grid-analysis}
# Perform GRID analysis on the smaller dataset
grid_result <- grid(example_data_5_subject, gap = 15, threshold = 130)

# Display results
cat("GRID Analysis Results:\n")
cat("  Detected grid points:", nrow(grid_result$grid_vector), "\n")
cat("  Episode counts:\n")
print(grid_result$episode_counts)

# Show first few detected grid points
cat("\nFirst few detected grid points:\n")
head(grid_result$grid_vector)
```

## 2. Hyperglycemic Event Detection

Detect different levels of hyperglycemic events according to clinical guidelines.

```{r hyperglycemic-events}
# Level 1 Hyperglycemic events (≥15 consecutive minutes >180 mg/dL)
hyper_lv1 <- detect_hyperglycemic_events(
  example_data_5_subject, 
  type = "lv1"
)

# Level 2 Hyperglycemic events (≥15 consecutive minutes >250 mg/dL)
hyper_lv2 <- detect_hyperglycemic_events(
  example_data_5_subject, 
  type = "lv2"
)

# Extended Hyperglycemic events (default parameters)
hyper_extended <- detect_hyperglycemic_events(example_data_5_subject, type = "extended")

cat("Hyperglycemic Event Detection Results:\n")
cat("Level 1 Events (>180 mg/dL):\n")
print(hyper_lv1$events_total)

cat("\nLevel 2 Events (>250 mg/dL):\n")
print(hyper_lv2$events_total)

cat("\nExtended Events (default):\n")
print(hyper_extended$events_total)

# Show detailed events for first subject
cat("\nDetailed Level 1 Events for Subject", hyper_lv1$events_detailed$id[1], ":\n")
head(hyper_lv1$events_detailed[hyper_lv1$events_detailed$id == hyper_lv1$events_detailed$id[1], ])
```

## 3. Hypoglycemic Event Detection

Detect hypoglycemic events using different thresholds.

```{r hypoglycemic-events}
# Level 1 Hypoglycemic events (≤70 mg/dL)
hypo_lv1 <- detect_hypoglycemic_events(
  example_data_5_subject, 
  type = "lv1"
)

# Level 2 Hypoglycemic events (≤54 mg/dL)
hypo_lv2 <- detect_hypoglycemic_events(
  example_data_5_subject, 
  type = "lv2"
)

cat("Hypoglycemic Event Detection Results:\n")
cat("Level 1 Events (≤70 mg/dL):\n")
print(hypo_lv1$events_total)

cat("\nLevel 2 Events (≤54 mg/dL):\n")
print(hypo_lv2$events_total)
```

## 4. Comprehensive Event Detection

Detect all types of glycemic events in one analysis.

```{r all-events}
# Detect all events; reading_minutes is calculated automatically when omitted
all_events <- detect_all_events(example_data_5_subject)

cat("Comprehensive Event Detection Results:\n")
print(all_events$subject_summary)
print(all_events$glycemic_event_summary)
```

## 5. Local Maxima Detection

Identify local maxima in glucose time series, which are important for postprandial peak analysis.

```{r local-maxima}
# Find local maxima
maxima_result <- find_local_maxima(example_data_5_subject)

cat("Local Maxima Detection Results:\n")
cat("  Total local maxima found:", nrow(maxima_result$local_maxima_vector), "\n")
cat("  Merged results:", nrow(maxima_result$merged_results), "\n")

# Show first few maxima
head(maxima_result$local_maxima_vector)
```

## 6. Maxima-GRID Combined Analysis

Combine maxima detection with GRID analysis for comprehensive postprandial peak detection.

```{r maxima-grid}
# Combined maxima and GRID analysis
maxima_grid_result <- maxima_grid(
  example_data_5_subject, 
  threshold = 130, 
  gap = 60, 
  hours = 2
)

cat("Maxima-GRID Combined Analysis Results:\n")
cat("  Detected maxima:", nrow(maxima_grid_result$results), "\n")
cat("  Episode counts:\n")
print(maxima_grid_result$episode_counts)

# Show first few results
head(maxima_grid_result$results)
```

## 7. Excursion Analysis

Analyze glucose excursions above a threshold.

```{r excursion-analysis}
# Excursion analysis
excursion_result <- excursion(example_data_5_subject, gap = 15)

cat("Excursion Analysis Results:\n")
cat("  Excursion vector length:", length(excursion_result$excursion_vector), "\n")
cat("  Episode counts:\n")
print(excursion_result$episode_counts)

# Show episode start information
head(excursion_result$episode_start)
```

## 8. Advanced Pipeline: Complete Workflow

Demonstrate the complete analysis pipeline using the larger dataset for more comprehensive results.
Note: This section may take longer to run on some machines.

```{r complete-pipeline}
# Use the larger dataset for comprehensive analysis
cat("Running complete analysis pipeline on example_data_hall...\n")

# Step 1: GRID analysis
cat("Step 1: GRID Analysis\n")
grid_pipeline <- grid(example_data_hall, gap = 15, threshold = 130)
cat("  Detected", nrow(grid_pipeline$grid_vector), "grid points\n")

# Step 2: Local maxima detection
cat("Step 2: Local Maxima Detection\n")
maxima_pipeline <- find_local_maxima(example_data_hall)
cat("  Found", nrow(maxima_pipeline$local_maxima_vector), "local maxima\n")

# Step 3: Modified GRID analysis
cat("Step 3: Modified GRID Analysis\n")
mod_grid_pipeline <- mod_grid(
  example_data_hall, 
  grid_pipeline$grid_vector, 
  hours = 2, 
  gap = 15
)
cat("  Modified grid points:", nrow(mod_grid_pipeline$mod_grid_vector), "\n")

# Step 4: Find maximum points after modified GRID points
cat("Step 4: Finding Maximum Points After GRID Points\n")
max_after_pipeline <- find_max_after_hours(
  example_data_hall,
  mod_grid_pipeline$mod_grid_vector,
  hours = 2
)
cat("  Maximum points found:", length(max_after_pipeline$max_index), "\n")

# Step 5: Find new maxima
cat("Step 5: Finding New Maxima\n")
new_maxima_pipeline <- find_new_maxima(
  example_data_hall,
  max_after_pipeline$max_index,
  maxima_pipeline$local_maxima_vector
)
cat("  New maxima identified:", nrow(new_maxima_pipeline), "\n")

# Step 6: Transform dataframes
cat("Step 6: Transforming Dataframes\n")
transformed_pipeline <- transform_df(
  grid_pipeline$episode_start, 
  new_maxima_pipeline
)
cat("  Transformed dataframe rows:", nrow(transformed_pipeline), "\n")

# Step 7: Detect between maxima
cat("Step 7: Detecting Between Maxima\n")
between_maxima_pipeline <- detect_between_maxima(
  example_data_hall, 
  transformed_pipeline
)
cat("  Between maxima analysis completed\n")

cat("\nComplete pipeline executed successfully!\n")
```

## 9. Time-Based Analysis Functions

Demonstrate functions that find maximum and minimum values within specific time windows.

```{r time-based-analysis}
# Create a subset for demonstration
subset_data <- example_data_5_subject[example_data_5_subject$id == unique(example_data_5_subject$id)[1], ][1:100, ]

# Create start points for time-based analysis
start_points <- subset_data[seq(1, nrow(subset_data), by = 20), ]

cat("Time-Based Analysis Functions:\n")

# Find maximum after 1 hour
max_after <- find_max_after_hours(subset_data, start_points, hours = 1)
cat("  Max after 1 hour:", length(max_after$max_index), "points\n")

# Find maximum before 1 hour
max_before <- find_max_before_hours(subset_data, start_points, hours = 1)
cat("  Max before 1 hour:", length(max_before$max_index), "points\n")

# Find minimum after 1 hour
min_after <- find_min_after_hours(subset_data, start_points, hours = 1)
cat("  Min after 1 hour:", length(min_after$min_index), "points\n")

# Find minimum before 1 hour
min_before <- find_min_before_hours(subset_data, start_points, hours = 1)
cat("  Min before 1 hour:", length(min_before$min_index), "points\n")
```

## 10. Data Ordering Utility

Demonstrate the fast dataframe ordering utility.

```{r ordering-utility}
# Create sample data with mixed order
sample_data <- data.frame(
  id = c("b", "a", "c", "a", "b"),
  time = as.POSIXct(c("2023-01-01 10:00:00", "2023-01-01 09:00:00", 
                      "2023-01-01 11:00:00", "2023-01-01 08:00:00", 
                      "2023-01-01 12:00:00"), tz = "UTC"),
  gl = c(120, 100, 140, 90, 130)
)

cat("Original data (unordered):\n")
print(sample_data)

# Order the data
ordered_data <- orderfast(sample_data)

cat("\nOrdered data:\n")
print(ordered_data)
```

## 11. Visualization Examples

Create visualizations to better understand the analysis results.

```{r visualization, fig.width=12, fig.height=8}
# Select one subject for visualization
subject_id <- unique(example_data_5_subject$id)[1]
subject_data <- example_data_5_subject[example_data_5_subject$id == subject_id, ]

# Create a comprehensive plot
p1 <- ggplot(subject_data, aes(x = time, y = gl)) +
  geom_line(color = "blue", alpha = 0.7, size = 0.5) +
  geom_hline(yintercept = 180, color = "red", linetype = "dashed", alpha = 0.8) +
  geom_hline(yintercept = 250, color = "darkred", linetype = "dashed", alpha = 0.8) +
  geom_hline(yintercept = 70, color = "orange", linetype = "dashed", alpha = 0.8) +
  geom_hline(yintercept = 54, color = "darkorange", linetype = "dashed", alpha = 0.8) +
  labs(title = paste("CGM Data for Subject", subject_id, "with Clinical Thresholds"),
       subtitle = "Red lines: Hyperglycemia thresholds (180, 250 mg/dL)\nOrange lines: Hypoglycemia thresholds (70, 54 mg/dL)",
       x = "Time", 
       y = "Glucose (mg/dL)") +
  theme_minimal() +
  theme(plot.title = element_text(size = 14, face = "bold"),
        plot.subtitle = element_text(size = 10))

print(p1)

# Create a summary plot showing event counts across subjects
event_summary <- hyper_lv1$events_total
event_summary$subject <- paste("Subject", event_summary$id)

p2 <- ggplot(event_summary, aes(x = subject, y = total_episodes)) +
  geom_col(fill = "steelblue", alpha = 0.7) +
  geom_text(aes(label = total_episodes), vjust = -0.5) +
  labs(title = "Level 1 Hyperglycemic Events by Subject",
       x = "Subject",
       y = "Number of Episodes") +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust = 1))

print(p2)
```

## 12. Performance Comparison

Compare performance between datasets of different sizes.

```{r performance-comparison}
# Function to measure execution time
measure_time <- function(expr) {
  start_time <- Sys.time()
  result <- eval(expr)
  end_time <- Sys.time()
  return(list(result = result, time = as.numeric(end_time - start_time, units = "secs")))
}

cat("Performance Comparison:\n")

# Test on smaller dataset
cat("Small dataset (5 subjects, 13,866 readings):\n")
small_time <- measure_time(grid(example_data_5_subject, gap = 15, threshold = 130))
cat("  GRID analysis time:", round(small_time$time, 3), "seconds\n")

small_maxima_time <- measure_time(find_local_maxima(example_data_5_subject))
cat("  Local maxima time:", round(small_maxima_time$time, 3), "seconds\n")

# Test on larger dataset
cat("\nLarge dataset (19 subjects, 34,890 readings):\n")
large_time <- measure_time(grid(example_data_hall, gap = 15, threshold = 130))
cat("  GRID analysis time:", round(large_time$time, 3), "seconds\n")

large_maxima_time <- measure_time(find_local_maxima(example_data_hall))
cat("  Local maxima time:", round(large_maxima_time$time, 3), "seconds\n")

# Calculate efficiency
efficiency_ratio <- (large_time$time / large_time$result$episode_counts$total_episodes) / 
                    (small_time$time / small_time$result$episode_counts$total_episodes)
cat("\nEfficiency ratio (large/small):", round(efficiency_ratio, 2))
```

## Summary

This vignette demonstrates the comprehensive capabilities of the `cgmguru` package:

### **Core Analysis Functions:**
- **GRID Analysis**: Detects rapid glucose rate increases
- **Event Detection**: Identifies hyperglycemic and hypoglycemic events
- **Local Maxima**: Finds peaks in glucose time series
- **Excursion Analysis**: Analyzes glucose excursions above thresholds

### **Advanced Pipeline Functions:**
- **Maxima-GRID Combination**: Integrated analysis for postprandial peaks
- **Time-Based Analysis**: Finds maximum/minimum values within time windows
- **Complete Workflow**: End-to-end analysis pipeline
- **Data Transformation**: Prepares data for downstream analysis

### **Utility Functions:**
- **Fast Ordering**: Efficient dataframe sorting
- **Input Validation**: Robust error handling and parameter validation

### **Key Features:**
- **High Performance**: C++ implementation for speed
- **Clinical Relevance**: Based on established CGM analysis methodologies
- **Comprehensive Coverage**: Handles various glycemic event types
- **Flexible Parameters**: Customizable thresholds and time windows
- **Robust Validation**: Input validation and error handling

The package is designed for both research and clinical applications, providing reliable and efficient tools for CGM data analysis. For more detailed function documentation, see `help(package = "cgmguru")`.

## References

- Battelino, T., et al. "Continuous glucose monitoring and metrics for clinical trials: an international consensus statement." *The Lancet Diabetes & Endocrinology* 11.1 (2023): 42-57.
- Harvey, Rebecca A., et al. "Design of the glucose rate increase detector: a meal detection module for the health monitoring system." *Journal of diabetes science and technology* 8.2 (2014): 307-320.
- Broll, S., Urbanek, J., Buchanan, D., Chun, E., Muschelli, J., Punjabi, N., and Gaynanova, I. "Interpreting blood glucose data with R package iglu." *PLoS One* 16.4 (2021): e0248560. doi:10.1371/journal.pone.0248560.
- Chun, E., Fernandes, J. N., and Gaynanova, I. "An Update on the iglu Software Package for Interpreting Continuous Glucose Monitoring Data." *Diabetes Technology & Therapeutics* 26.12 (2024): 939-950. doi:10.1089/dia.2024.0154.

---
title: "find_new_maxima function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{find_new_maxima function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
```

# find_new_maxima

Runs the documented examples for `find_new_maxima()`.

```{r}
example(find_new_maxima, package = "cgmguru", run.dontrun = FALSE)
```

---
title: "detect_between_maxima function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{detect_between_maxima function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
```

# detect_between_maxima

## Overview
Identifies and analyzes events occurring between detected maxima points to characterize glucose dynamics between peaks. Often used downstream of `grid()`, `mod_grid()`, `find_max_after_hours()`, and `find_new_maxima()`.

## Inputs
- **df**: CGM dataframe with `id`, `time` (POSIXct), `gl` (mg/dL)
- **transform_df**: Output from `transform_df()` mapping GRID starts to maxima

## Returns
- **results**: Tibble with `id`, `grid_time`, `grid_gl`, `maxima_time`, `maxima_glucose`, `time_to_peak`
- **episode_counts**: Counts per `id`

## Run documented examples
```{r}
example(detect_between_maxima, package = "cgmguru", run.dontrun = FALSE)
```

---
title: "orderfast function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{orderfast function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
```

# orderfast

## Overview
Fast ordering of a dataframe by `id` and `time`, optimized for large CGM datasets. Returns the input sorted while preserving columns.

## Inputs
- **df**: Dataframe containing `id` and `time`

## Returns
- Dataframe ordered by `id`, then `time`

## Run documented examples
```{r}
example(orderfast, package = "cgmguru", run.dontrun = FALSE)
```

---
title: "cgmguru: Examples Gallery"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Examples Gallery}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", fig.width = 10, fig.height = 6, warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
library(ggplot2)
library(dplyr)

# Use package root as working directory so we can reference ../examples
pkg_root <- normalizePath("..", winslash = "/", mustWork = TRUE)
examples_dir <- file.path(pkg_root, "inst", "examples")
stopifnot(dir.exists(examples_dir))
```

# Overview

This vignette renders and runs the example scripts provided in the package's `/inst/examples/` directory, so you can see inputs, outputs, and typical workflows in one HTML document.

If you are viewing this from RStudio Help, use the "Open in Browser" button for the best experience.

## Notes

- Code is executed by sourcing the example scripts directly from `/inst/examples/`.
- Some examples may take longer on large datasets.
- Figures may be produced where applicable.

---

## GRID

```{r ex-grid}
source(file.path(examples_dir, "grid.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Detect all events

```{r ex-detect-all-events}
source(file.path(examples_dir, "detect_all_events.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Detect hyperglycemic events

```{r ex-detect-hyper}
source(file.path(examples_dir, "detect_hyperglycemic_events.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Detect hypoglycemic events

```{r ex-detect-hypo}
source(file.path(examples_dir, "detect_hypoglycemic_events.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Excursion

```{r ex-excursion}
source(file.path(examples_dir, "excursion.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Local maxima

```{r ex-local-maxima}
source(file.path(examples_dir, "find_local_maxima.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Max after hours

```{r ex-max-after}
source(file.path(examples_dir, "find_max_after_hours.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Max before hours

```{r ex-max-before}
source(file.path(examples_dir, "find_max_before_hours.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Min after hours

```{r ex-min-after}
source(file.path(examples_dir, "find_min_after_hours.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Min before hours

```{r ex-min-before}
source(file.path(examples_dir, "find_min_before_hours.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Maxima + GRID combined

```{r ex-maxima-grid}
source(file.path(examples_dir, "maxima_grid.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Ordering utility

```{r ex-orderfast}
source(file.path(examples_dir, "orderfast.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Start finder

```{r ex-start-finder}
source(file.path(examples_dir, "start_finder.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

## Transform dataframe

```{r ex-transform-df}
source(file.path(examples_dir, "transform_df.R"), echo = TRUE, print.eval = TRUE, max.deparse.length = Inf)
```

---

## Function reference and help

For detailed function documentation with additional runnable `@examples`, see the package reference:

```{r, eval=FALSE}
help(package = "cgmguru")
```

You can also open the package vignettes index:

```{r, eval=FALSE}
browseVignettes("cgmguru")
```

---
title: "start_finder function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{start_finder function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE, message = FALSE)
library(cgmguru)
```

# start_finder

## Overview
Finds R-based (1-indexed) positions where 1s begin episodes in a 0/1 vector (1 after 0 or at start). Useful for identifying episode starts.

## Inputs
- **df**: Dataframe whose first column is an integer vector of 0/1

## Returns
- **start_index**: Tibble of episode start positions (R index)

## Run documented examples
```{r}
example(start_finder, package = "cgmguru", run.dontrun = FALSE)
```

---
title: "transform_df function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{transform_df function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
```

# transform_df

## Overview
Prepares data for downstream analysis by mapping GRID episode starts to maxima within a 4-hour window and merging the information.

## Inputs
- **grid_df**: From `grid()` (typically `episode_start`)
- **maxima_df**: From `find_new_maxima()` or local maxima results

## Returns
- Tibble with columns: `id`, `grid_time`, `grid_gl`, `maxima_time`, `maxima_gl`

## Run documented examples
```{r}
example(transform_df, package = "cgmguru", run.dontrun = FALSE)
```

---
title: "maxima_grid function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{maxima_grid function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
```

# maxima_grid

Runs the documented examples for `maxima_grid()`.

`maxima_grid()` runs the GRID and peak-mapping pipeline on the rows supplied in `df`. It does not automatically use `interpolate_cgm()` or the full-day event preprocessing grid unless you explicitly pass interpolated data to the function.

```{r}
example(maxima_grid, package = "cgmguru", run.dontrun = FALSE)
```

---
title: "find_max_after_hours function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{find_max_after_hours function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
```

# find_max_after_hours

Runs the documented examples for `find_max_after_hours()`.

```{r}
example(find_max_after_hours, package = "cgmguru", run.dontrun = FALSE)
```

---
title: "excursion function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{excursion function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
```

# excursion

## Overview
Calculates glucose excursions: >70 mg/dL rise within 2 hours, not preceded by a value <70 mg/dL.

## Inputs
- **df**: CGM dataframe with `id`, `time` (POSIXct), `gl` (mg/dL)
- **gap**: Minimum minutes separating excursions (default 15)

## Returns
- **excursion_vector**: Logical/int vector marking excursions
- **episode_counts**: Counts per `id`
- **episode_start**: Starts with `id`, `time`, `gl`, `index`

## Run documented examples
```{r}
example(excursion, package = "cgmguru", run.dontrun = FALSE)
```

---
title: "detect_all_events function: cgmguru and iglu-compatible event preprocessing"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{detect_all_events function: cgmguru and iglu-compatible event preprocessing}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4,
  message = FALSE,
  warning = FALSE
)

library(cgmguru)

# Availability flags used in chunk eval options
iglu_available <- requireNamespace("iglu", quietly = TRUE)
iglu_api_ok <- iglu_available
```

## Relationship to iglu

`cgmguru::detect_all_events()` is an independent C++/Rcpp implementation of
event calculation and CGM summary output. Its preprocessing is designed to be
compatible with the event grid semantics used by `iglu`: subject-specific
reading intervals, a midnight-aligned full-day grid, interpolation up to
`inter_gap`, removal of larger gap-masked rows, and segment-wise event
classification.

When `reading_minutes` is omitted or `NULL`, cgmguru calculates it
automatically per subject from the median positive timestamp spacing in the
input data.

The `iglu` package is used here as a formal reference, source of public example
datasets, and comparison target. cgmguru does not call `iglu` at runtime for its
core algorithms.

## Datasets

We use two CGM example datasets shipped with `iglu`:

- `example_data_5_subject`: 5 subjects
- `example_data_hall`: 19 subjects

```{r data-load, eval=iglu_available}
data(example_data_5_subject, package = "iglu")
data(example_data_hall, package = "iglu")

# Base-R summaries (no external dependencies)
summary_5 <- data.frame(
  rows = nrow(example_data_5_subject),
  subjects = length(unique(example_data_5_subject$id)),
  time_min = min(example_data_5_subject$time),
  time_max = max(example_data_5_subject$time),
  gl_min = min(example_data_5_subject$gl, na.rm = TRUE),
  gl_max = max(example_data_5_subject$gl, na.rm = TRUE)
)

summary_5
```

```{r data-load-missing, eval=!iglu_available}
cat("Note: The 'iglu' package is not available; vignette examples are skipped.\n")
```

## iglu: episode_calculation

`iglu::episode_calculation()` identifies hypo/hyperglycemia episodes.

```{r iglu-episodes-5, eval=iglu_available}
iglu_episodes_5 <- iglu::episode_calculation(
  data = example_data_5_subject
)
print(iglu_episodes_5)
```

```{r iglu-episodes-hall, eval=iglu_api_ok}
iglu_episodes_hall <- iglu::episode_calculation(
  data = example_data_hall
)
print(iglu_episodes_hall)
```

## cgmguru: detect_all_events

```{r cgmguru-all-5, eval=iglu_available}
all_events_5 <- detect_all_events(example_data_5_subject)
print(all_events_5)
```

```{r cgmguru-all-hall, eval=iglu_available}
all_events_hall <- detect_all_events(example_data_hall)
print(all_events_hall)
```

## Speed comparison

We compare performance using `microbenchmark` on both datasets. Each benchmark contrasts `iglu::episode_calculation()` with `cgmguru::detect_all_events()`.

```{r speed-microbenchmark, cache=FALSE, eval=iglu_available}
library(microbenchmark)
library(iglu)

# example_data_5_subject
bench_5 <- microbenchmark(
  episode_calculation = iglu::episode_calculation(example_data_5_subject),
  detect_all_events   = cgmguru::detect_all_events(example_data_5_subject),
  times = 10,
  unit = "ms"
)
print(bench_5)

# example_data_hall (all subjects)
bench_hall <- microbenchmark(
  episode_calculation = iglu::episode_calculation(example_data_hall),
  detect_all_events   = cgmguru::detect_all_events(example_data_hall),
  times = 10,
  unit = "ms"
)
print(bench_hall)
```

```{r iglu-api-mismatch, eval=iglu_available && !iglu_api_ok}
cat("Note: Installed 'iglu' version has a different 'episode_calculation' API; iglu examples are skipped.\n")
```

## References

- Broll S, Urbanek J, Buchanan D, Chun E, Muschelli J, Punjabi N, and Gaynanova I. Interpreting blood glucose data with R package iglu. *PLoS One*. 2021;16(4):e0248560. doi:10.1371/journal.pone.0248560.
- Chun E, Fernandes JN, and Gaynanova I. An Update on the iglu Software Package for Interpreting Continuous Glucose Monitoring Data. *Diabetes Technology & Therapeutics*. 2024;26(12):939-950. doi:10.1089/dia.2024.0154.
- iglu package site and citation guidance: <https://irinagain.github.io/iglu/> and <https://irinagain.github.io/iglu/authors.html>.

---
title: "excursion function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{excursion function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
```

# excursion

## Overview
Calculates glucose excursions: >70 mg/dL rise within 2 hours, not preceded by a value <70 mg/dL.

`excursion()` evaluates the rows supplied in `df`. It does not automatically call `interpolate_cgm()` and does not use the full-day event preprocessing grid unless you explicitly pass interpolated data to `excursion()`.

## Inputs
- **df**: CGM dataframe with `id`, `time` (POSIXct), `gl` (mg/dL)
- **gap**: Minimum minutes separating excursions (default 15)

## Returns
- **excursion_vector**: Logical/int vector marking excursions
- **episode_counts**: Counts per `id`
- **episode_start**: Starts with `id`, `time`, `gl`, `index`

## Run documented examples
```{r}
example(excursion, package = "cgmguru", run.dontrun = FALSE)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{mod_grid}
\alias{mod_grid}
\title{Modified GRID Analysis}
\usage{
mod_grid(df, grid_point_df, hours = 2, gap = 15)
}
\arguments{
\item{df}{A dataframe containing continuous glucose monitoring (CGM) data.
Must include columns:
\itemize{
  \item \code{id}: Subject identifier (string or factor)
  \item \code{time}: Time of measurement (POSIXct)
  \item \code{gl}: Glucose value (integer or numeric, mg/dL)
}}

\item{grid_point_df}{A dataframe with column \code{start_index} (start points for re-applied GRID)}

\item{hours}{Time window in hours for analysis (default: 2)}

\item{gap}{Gap threshold in minutes for event detection (default: 15).
This parameter defines the minimum time interval between consecutive GRID events.}
}
\value{
A list containing:
\itemize{
  \item \code{mod_grid_vector}: Tibble with modified GRID results (\code{mod_grid})
  \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
  \item \code{episode_start}: Tibble with all episode starts with columns:
    \itemize{
      \item \code{id}: Subject identifier
      \item \code{time}: Timestamp at which the event occurs; equivalent to \code{df$time[index]}
      \item \code{gl}: Glucose value at the event; equivalent to \code{df$gl[index]}
      \item \code{index}: R-based (1-indexed) row number(s) in \code{df} denoting where the event occurs
    }
}
}
\description{
Constructs a modified GRID series by reapplying the GRID logic with a designated
gap (e.g., 60 minutes) and analysis window in hours (e.g., 2 hours). It
reassigns GRID events under these constraints to produce a modified grid
suitable for downstream maxima mapping and episode analysis.
}
\section{Units and sampling}{

- \code{gap} is minutes; \code{hours} is hours; \code{time} is POSIXct.
}

\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# First, get grid points
grid_result <- grid(example_data_5_subject, gap = 60, threshold = 130)

# Perform modified GRID analysis
mod_result <- mod_grid(example_data_5_subject, grid_result$grid_vector, hours = 2, gap = 60)
print(paste("Modified grid points:", nrow(mod_result$mod_grid_vector)))

# Modified analysis with different parameters
mod_result_1h <- mod_grid(example_data_5_subject, grid_result$grid_vector, hours = 1, gap = 40)

# Analysis on larger dataset
large_grid <- grid(example_data_hall, gap = 60, threshold = 130)
large_mod_result <- mod_grid(example_data_hall, large_grid$grid_vector, hours = 2, gap = 60)
print(paste("Modified grid points in larger dataset:", nrow(large_mod_result$mod_grid_vector)))
}
\references{
Park, Sang Ho, et al. "Identification of clinically meaningful automatically detected postprandial glucose excursions in individuals with type 1 diabetes using personal continuous glucose monitoring." Diabetes Research and Clinical Practice (2025): 112951.

Park, Soojin, et al. "High-Amplitude and Prolonged Glucose Excursions as a Key Determinant of Discordance Between Glucose Management Indicator and Glycated Hemoglobin in Type 1 Diabetes." Diabetes Care (2026): dc252820. https://doi.org/10.2337/dc25-2820
}
\seealso{
\link{grid}, \link{find_max_after_hours}, \link{find_new_maxima}

Other GRID pipeline: 
\code{\link{detect_between_maxima}()},
\code{\link{find_local_maxima}()},
\code{\link{find_max_after_hours}()},
\code{\link{find_max_before_hours}()},
\code{\link{find_min_after_hours}()},
\code{\link{find_min_before_hours}()},
\code{\link{find_new_maxima}()},
\code{\link{grid}()},
\code{\link{maxima_grid}()},
\code{\link{start_finder}()},
\code{\link{transform_df}()}
}
\concept{GRID pipeline}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{excursion}
\alias{excursion}
\title{Calculate Glucose Excursions}
\usage{
excursion(df, gap = 15)
}
\arguments{
\item{df}{A dataframe containing continuous glucose monitoring (CGM) data.
Must include columns:
\itemize{
  \item \code{id}: Subject identifier (string or factor)
  \item \code{time}: Time of measurement (POSIXct)
  \item \code{gl}: Glucose value (integer or numeric, mg/dL)
}}

\item{gap}{Gap threshold in minutes for excursion calculation (default: 15).
This parameter defines the minimum time interval between consecutive GRID events.}
}
\value{
A list containing:
\itemize{
  \item \code{excursion_vector}: Tibble with excursion results (\code{excursion})
  \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
  \item \code{episode_start}: Tibble with all episode starts with columns:
    \itemize{
      \item \code{id}: Subject identifier
      \item \code{time}: Timestamp at which the event occurs; equivalent to \code{df$time[index]}
      \item \code{gl}: Glucose value at the event; equivalent to \code{df$gl[index]}
      \item \code{index}: R-based (1-indexed) row number(s) in \code{df} denoting where the event occurs
    }
}
}
\description{
Calculates glucose excursions in CGM data. An excursion is defined as
a \eqn{>} 70 mg/dL (\eqn{>} 3.9 mmol/L) rise within 2 hours, not preceded by a value
\eqn{<} 70 mg/dL (\eqn{<} 3.9 mmol/L).
}
\section{Notes}{

- \code{gap} is minutes; change to enforce minimum separation between excursions.
- This function operates on the rows supplied in \code{df}. It does not use
  \code{\link{interpolate_cgm}} or the full-day event preprocessing grid.
}

\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# Calculate glucose excursions
excursion_result <- excursion(example_data_5_subject, gap = 15)
print(paste("Excursion vector length:", length(excursion_result$excursion_vector)))
print(excursion_result$episode_counts)

# Excursion analysis with different gap
excursion_30min <- excursion(example_data_5_subject, gap = 30)

# Analysis on larger dataset
large_excursion <- excursion(example_data_hall, gap = 15)
print(paste("Excursion vector length in larger dataset:", length(large_excursion$excursion_vector)))
print(paste("Total episodes:", sum(large_excursion$episode_counts$episode_counts)))
}
\references{
Edwards, S., et al. (2022). Use of connected pen as a diagnostic tool to evaluate missed bolus dosing behavior in people with type 1 and type 2 diabetes. Diabetes Technology & Therapeutics, 24(1), 61-66.
}
\seealso{
\link{grid}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-package.R
\docType{package}
\name{cgmguru-package}
\alias{cgmguru}
\alias{cgmguru-package}
\title{Advanced Continuous Glucose Monitoring Analysis and GRID-Based Event Detection}
\description{
A high-performance R package for comprehensive Continuous Glucose Monitoring (CGM) data
analysis with optimized C++ implementations. The package provides advanced tools for
CGM data analysis with two primary capabilities: GRID and postprandial peak detection,
and extended glycemic events detection aligned with international consensus CGM metrics.
}
\details{
The package implements several key algorithms for CGM analysis:
\itemize{
  \item \strong{GRID Algorithm}: Detects rapid glucose rate increases (commonly \eqn{\geq} 90-95 mg/dL/hour)
        with configurable thresholds and gaps for postprandial peak detection
  \item \strong{Postprandial Peak Detection}: Finds peak glucose after GRID points using
        local maxima and configurable time windows
  \item \strong{Consensus CGM Metrics Event Detection}: Level 1/2 hypo- and hyperglycemia
        detection with duration validation (default minimum 15 minutes) aligned with
        Battelino et al. (2023) international consensus
  \item \strong{Advanced Analysis Tools}: Local maxima finding, excursion analysis,
        and robust episode validation utilities
}

Core algorithms are implemented in optimized C++ via 'Rcpp' for accurate and fast
analysis on large datasets, making the package suitable for both research and
clinical applications.
}
\section{Relationship to iglu}{

cgmguru uses the \pkg{iglu} package as a formal methodological reference,
source of example datasets, and comparison target. Event preprocessing is an
independent C++ implementation of iglu-compatible semantics: a midnight-aligned
full-day grid, interpolation up to \code{inter_gap}, removal of gap-masked
rows, and segment-wise event classification. cgmguru does not call \pkg{iglu}
at runtime for its core algorithms.

The event preprocessing grid applies only to event functions and
\code{\link{interpolate_cgm}}. GRID-family functions, including
\code{\link{grid}}, \code{\link{maxima_grid}}, and \code{\link{excursion}},
operate on the input timestamps and glucose values supplied by the user.
}

\section{Main Functions}{

\describe{
  \item{\code{\link{grid}}}{GRID algorithm for detecting rapid glucose rate increases}
  \item{\code{\link{maxima_grid}}}{Combined maxima detection and GRID analysis for postprandial peaks}
  \item{\code{\link{detect_hyperglycemic_events}}}{Hyperglycemic event detection (Level 1/2/Extended)}
  \item{\code{\link{detect_hypoglycemic_events}}}{Hypoglycemic event detection (Level 1/2/Extended)}
  \item{\code{\link{detect_all_events}}}{Comprehensive detection of all glycemic event types}
  \item{\code{\link{find_local_maxima}}}{Local maxima identification in glucose time series}
  \item{\code{\link{interpolate_cgm}}}{C++ iglu-compatible event-grid interpolation}
  \item{\code{\link{orderfast}}}{Fast dataframe ordering utility}
}
}

\section{Data Requirements}{

Input dataframes should contain:
\itemize{
  \item \code{id}: Patient identifier (character or factor)
  \item \code{time}: POSIXct timestamps
  \item \code{gl}: Glucose values in mg/dL
}

All function arguments and return values are expected to be in tibble format.
For convenience, single-column parameters can be passed as vectors in R, which
will be automatically converted to single-column tibbles.
}

\section{Examples}{

\preformatted{
# Basic GRID analysis
result <- grid(cgm_data, gap = 15, threshold = 130)

# Postprandial peak detection (GRID-based)
maxima <- maxima_grid(cgm_data, threshold = 130, gap = 60, hours = 2)

# Level 1 Hyperglycemic event detection
events <- detect_hyperglycemic_events(cgm_data, type = "lv1")

# Comprehensive event detection; reading_minutes is calculated automatically
all_events <- detect_all_events(cgm_data)
}
}

\references{
\itemize{
  \item Battelino, T., et al. "Continuous glucose monitoring and metrics for clinical trials: an international consensus statement." *The Lancet Diabetes & Endocrinology* 11.1 (2023): 42-57.
  \item Harvey, Rebecca A., et al. "Design of the glucose rate increase detector: a meal detection module for the health monitoring system." *Journal of diabetes science and technology* 8.2 (2014): 307-320.
  \item Adolfsson, Peter, et al. "Increased time in range and fewer missed bolus injections after introduction of a smart connected insulin pen." Diabetes Technology & Therapeutics 22.10 (2020): 709-718.
  \item Broll, S., Urbanek, J., Buchanan, D., Chun, E., Muschelli, J., Punjabi, N., and Gaynanova, I. "Interpreting blood glucose data with R package iglu." PLoS One 16.4 (2021): e0248560. doi:10.1371/journal.pone.0248560.
  \item Chun, E., Fernandes, J. N., and Gaynanova, I. "An Update on the iglu Software Package for Interpreting Continuous Glucose Monitoring Data." Diabetes Technology & Therapeutics 26.12 (2024): 939-950. doi:10.1089/dia.2024.0154.
  \item Park, Sang Ho, et al. "Identification of clinically meaningful automatically detected postprandial glucose excursions in individuals with type 1 diabetes using personal continuous glucose monitoring." Diabetes Research and Clinical Practice (2025): 112951.
  \item Park, Soojin, et al. "High-Amplitude and Prolonged Glucose Excursions as a Key Determinant of Discordance Between Glucose Management Indicator and Glycated Hemoglobin in Type 1 Diabetes." Diabetes Care (2026): dc252820. https://doi.org/10.2337/dc25-2820
  \item Edwards, Stephanie, et al. "Use of connected pen as a diagnostic tool to evaluate missed bolus dosing behavior in people with type 1 and type 2 diabetes." Diabetes Technology & Therapeutics 24.1 (2022): 61-66.
}

For more information about the GRID algorithm and CGM analysis methodologies,
see the package vignette: \code{vignette("intro", package = "cgmguru")}
}
\seealso{
\code{\link{grid}}, \code{\link{maxima_grid}}, \code{\link{detect_hyperglycemic_events}}, \code{\link{detect_all_events}}
}
\author{
Sang Ho Park \email{shstat1729@gmail.com}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{interpolate_cgm}
\alias{interpolate_cgm}
\title{Interpolate CGM Data}
\usage{
interpolate_cgm(df, reading_minutes = NULL, sort_time = FALSE,
 inter_gap = 45)
}
\arguments{
\item{df}{A dataframe containing CGM data with columns:
\itemize{
  \item \code{id}: Subject identifier
  \item \code{time}: POSIXct measurement timestamp
  \item \code{gl}: Glucose value in mg/dL
}}

\item{reading_minutes}{Time interval for the interpolation grid in minutes.
If omitted or \code{NULL}, it is calculated automatically per id as the
median positive time difference in the data.}

\item{sort_time}{Logical. If \code{TRUE}, sort rows within each id by
\code{time} in C++ before interpolation. Defaults to \code{FALSE}.}

\item{inter_gap}{Maximum gap in minutes to interpolate across. Defaults to
45; larger gaps split the event-detection grid.}
}
\value{
A tibble with columns \code{id}, interpolated \code{time}, and
  interpolated \code{gl}. Rows inside gaps larger than \code{inter_gap} are
  omitted.
}
\description{
Interpolates continuous glucose monitoring (CGM) data onto the same
iglu-compatible, midnight-aligned full-day grid used internally by
cgmguru's event-detection functions. The interpolation is implemented in C++
and is intended for users who want to inspect or reuse the preprocessed grid
behind \code{\link{detect_all_events}},
\code{\link{detect_hyperglycemic_events}}, and
\code{\link{detect_hypoglycemic_events}}.

For each subject, \code{interpolate_cgm()} builds an equally spaced grid at
\code{reading_minutes} intervals. If \code{reading_minutes} is omitted, it is
inferred per subject from the median positive timestamp difference. Glucose
values are linearly interpolated only across gaps up to \code{inter_gap};
larger gaps are treated as missing and removed from the returned data,
preserving segment boundaries used by event calculation.

The GRID-family functions \code{\link{grid}}, \code{\link{maxima_grid}}, and
\code{\link{excursion}} do not call this helper automatically; they operate
on the rows supplied by the user unless the caller explicitly passes an
interpolated dataset.
}
\examples{
df <- data.frame(
  id = "A",
  time = as.POSIXct(c("2026-01-01 00:15:00", "2026-01-01 00:25:00"),
                    tz = "UTC"),
  gl = c(100, 120)
)
interpolate_cgm(df)
}
\seealso{
\link{detect_all_events}, \link{detect_hyperglycemic_events},
  \link{detect_hypoglycemic_events}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R, R/orderfast.R
\name{orderfast}
\alias{orderfast}
\title{Fast Ordering Function}
\usage{
orderfast(df)
}
\arguments{
\item{df}{A dataframe with 'id' and 'time' columns}
}
\value{
A dataframe ordered by id and time

A dataframe ordered by id and time
}
\description{
Orders a dataframe by \code{id} and \code{time} columns using a C++
\code{std::sort} backend. Optimized for large CGM datasets, it returns the
input with rows sorted by subject then timestamp while preserving all columns.

Orders a dataframe by id and time columns using the C++ backend
}
\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# Shuffle without replacement, then order and compare to baseline
set.seed(123)
shuffled <- example_data_5_subject[sample(seq_len(nrow(example_data_5_subject)),
                                          replace = FALSE), ]
baseline <- orderfast(example_data_5_subject)
ordered_shuffled <- orderfast(shuffled)

# Compare results
print(paste("Identical after ordering:", identical(baseline, ordered_shuffled)))
head(baseline[, c("id", "time", "gl")])
head(ordered_shuffled[, c("id", "time", "gl")])

# Order larger dataset
ordered_large <- orderfast(example_data_hall)
print(paste("Ordered", nrow(ordered_large), "rows in larger dataset"))
df <- data.frame(id = c("b", "a", "a"), time = as.POSIXct(
  c("2024-01-01 01:00:00", "2024-01-01 00:00:00", "2024-01-01 01:00:00"), tz = "UTC"
))
orderfast(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\encoding{UTF-8}
\name{detect_hypoglycemic_events}
\alias{detect_hypoglycemic_events}
\title{Detect Hypoglycemic Events}
\usage{
detect_hypoglycemic_events(df, ..., type = "extended",
 reading_minutes = NULL, sort_time = FALSE, inter_gap = 45,
 return_interpolated = TRUE)
}
\arguments{
\item{df}{A dataframe containing continuous glucose monitoring (CGM) data.
Must include columns:
\itemize{
  \item \code{id}: Subject identifier (string or factor)
  \item \code{time}: Time of measurement (POSIXct)
  \item \code{gl}: Glucose value (integer or numeric, mg/dL)
}}

\item{...}{Custom event criteria supplied by name. Prefer \code{type} for
standard Level 1, Level 2, and Extended events. Supported custom criteria
are:
\itemize{
  \item \code{dur_length}: Minimum event duration in minutes required for
    an event to qualify.
  \item \code{end_length}: Required recovery duration in minutes before
    event termination is confirmed.
  \item \code{start_gl}: Glucose threshold in mg/dL used to qualify
    hypoglycemic readings. Hypoglycemic readings are below this value, and
    recovery is confirmed after glucose remains at or above this value for
    \code{end_length} minutes.
}}

\item{type}{Hypoglycemia event definition. One of \code{"extended"}
(default), \code{"lv1"}, \code{"lv2"}, or \code{"lv1_excl"}.}

\item{reading_minutes}{Time interval between readings in minutes (optional).
If omitted or \code{NULL}, the interval is calculated automatically per id
as the median positive time difference in the data.}

\item{sort_time}{Logical. If \code{TRUE}, sort rows within each id by
\code{time} in C++ before interpolation. Defaults to \code{FALSE}.}

\item{inter_gap}{Maximum gap in minutes to interpolate across. Defaults to
45; larger gaps split event-detection segments.}

\item{return_interpolated}{Logical. If \code{TRUE}, include the interpolated
grid data used for event detection in the returned list. Defaults to
\code{TRUE}.}
}
\value{
A list containing:
\itemize{
  \item \code{events_total}: Tibble with summary statistics per subject (id, total_episodes, avg_ep_per_day)
  \item \code{events_detailed}: Tibble with detailed event information (id, start_time, start_glucose, end_time, end_glucose, start_index, end_index, duration_below_54_minutes). End fields report the last dysglycemic reading before confirmed recovery starts. \code{start_index} and \code{end_index} are 1-based row positions in the returned \code{interpolated_data}.
  \item \code{interpolated_data}: Included by default with columns
    \code{id}, \code{time}, and \code{gl}; set
    \code{return_interpolated = FALSE} to omit it.
}
}
\description{
Identifies and segments hypoglycemic events in CGM data based on international consensus
CGM metrics (Battelino et al., 2023). Use \code{type} to select one of
three event definitions:
\itemize{
  \item \strong{Level 1}: \eqn{\geq} 15 consecutive min of \eqn{<} 70 mg/dL, ends with \eqn{\geq} 15 consecutive min \eqn{\geq} 70 mg/dL
  \item \strong{Level 2}: \eqn{\geq} 15 consecutive min of \eqn{<} 54 mg/dL, ends with \eqn{\geq} 15 consecutive min \eqn{\geq} 54 mg/dL
  \item \strong{Extended}: \eqn{>} 120 consecutive min of \eqn{<} 70 mg/dL, ends with \eqn{\geq} 15 consecutive min \eqn{\geq} 70 mg/dL
}
Events are counted only after glucose remains at or above the recovery
threshold for the specified end length. In \code{events_detailed},
\code{end_time}, \code{end_glucose}, and \code{end_index} report the last
hypoglycemic reading immediately before that confirmed recovery period starts.
}
\section{Methods}{

Hypoglycemic events can be detected using either the recommended
\code{type} argument or named custom threshold and duration criteria.

\strong{1. Preset method using \code{type} (recommended):}
Use \code{type} when you want the standard Level 1, Level 2, or Extended
hypoglycemia definitions without manually entering thresholds.
\itemize{
  \item \code{type = "lv1"} uses \code{start_gl = 70},
    \code{dur_length = 15}, and \code{end_length = 15}.
  \item \code{type = "lv2"} uses \code{start_gl = 54},
    \code{dur_length = 15}, and \code{end_length = 15}.
  \item \code{type = "extended"} uses \code{start_gl = 70},
    \code{dur_length = 120}, and \code{end_length = 15}.
  \item \code{type = "lv1_excl"} returns Level 1 episodes that do not
    overlap Level 2 episodes.
}

\strong{2. Custom criteria method:}
Supply \code{start_gl}, \code{dur_length}, and \code{end_length} directly
when using a custom definition, for example
\code{detect_hypoglycemic_events(df, start_gl = 70, dur_length = 15,
end_length = 15)} for Level 1 hypoglycemia. If an explicit \code{type} is
supplied together with custom numeric criteria, the function returns results
based on \code{type}; the custom criteria are ignored and a warning is
issued.
}

\section{Units and sampling}{

- \code{reading_minutes} can be a scalar (all rows) or a vector per-row.
- If \code{reading_minutes} is omitted or \code{NULL}, it is calculated
  automatically per id from timestamp spacing.
- Event classification uses cgmguru's independent C++ implementation of an
  iglu-compatible, midnight-aligned full-day grid. Data are linearly
  interpolated at the id-specific interval up to \code{inter_gap}; larger
  gaps are masked, removed from the event-classification data, and split
  segments.
- This preprocessing is specific to event calculation and does not affect
  \code{\link{grid}}, \code{\link{maxima_grid}}, or \code{\link{excursion}}.
}

\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# Level 1 Hypoglycemia Event (>=15 consecutive min of <70 mg/dL and event
# ends when there is >=15 consecutive min with a CGM sensor value of >=70 mg/dL)
hypo_lv1 <- detect_hypoglycemic_events(example_data_5_subject, type = "lv1")
print(hypo_lv1$events_total)

# Level 2 Hypoglycemia Event (>=15 consecutive min of <54 mg/dL and event
# ends when there is >=15 consecutive min with a CGM sensor value of >=54 mg/dL)
hypo_lv2 <- detect_hypoglycemic_events(example_data_5_subject, type = "lv2")

# Extended Hypoglycemia Event (>120 consecutive min of <70 mg/dL and event
# ends when there is >=15 consecutive min with a CGM sensor value of >=70 mg/dL)
hypo_extended <- detect_hypoglycemic_events(example_data_5_subject, type = "extended")
print(hypo_extended$events_total)

# Custom criteria method for the same standard definitions
hypo_lv1_custom <- detect_hypoglycemic_events(
  example_data_5_subject,
  start_gl = 70,
  dur_length = 15,
  end_length = 15
)
hypo_lv2_custom <- detect_hypoglycemic_events(
  example_data_5_subject,
  start_gl = 54,
  dur_length = 15,
  end_length = 15
)
hypo_extended_custom <- detect_hypoglycemic_events(
  example_data_5_subject,
  start_gl = 70,
  dur_length = 120,
  end_length = 15
)

# Compare event rates across levels
cat("Level 1 episodes:", sum(hypo_lv1$events_total$total_episodes), "\n")
cat("Level 2 episodes:", sum(hypo_lv2$events_total$total_episodes), "\n")
cat("Extended episodes:", sum(hypo_extended$events_total$total_episodes), "\n")

# Analysis on larger dataset with Level 1 criteria
large_hypo <- detect_hypoglycemic_events(example_data_hall, type = "lv1")
print(large_hypo$events_total)

# Analysis on larger dataset with Level 2 criteria
large_hypo_lv2 <- detect_hypoglycemic_events(example_data_hall, type = "lv2")
print(large_hypo_lv2$events_total)

# Analysis on larger dataset with Extended criteria
large_hypo_extended <- detect_hypoglycemic_events(example_data_hall, type = "extended")
print(large_hypo_extended$events_total)
}
\references{
Battelino, T., et al. (2023). Continuous glucose monitoring and metrics for clinical trials: an international consensus statement. The Lancet Diabetes & Endocrinology, 11(1), 42-57.
}
\seealso{
\link{detect_all_events}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{transform_df}
\alias{transform_df}
\title{Transform Dataframe for Analysis}
\usage{
transform_df(grid_df, maxima_df)
}
\arguments{
\item{grid_df}{A dataframe containing grid analysis results}

\item{maxima_df}{A dataframe containing maxima detection results}
}
\value{
A tibble with transformed data containing columns (\code{id}, \code{grid_time}, \code{grid_gl}, \code{maxima_time}, \code{maxima_gl})
}
\description{
Performs data transformations required for GRID analysis, including
mapping GRID episode starts to maxima within a 4-hour window and
merging grid and maxima information. This function prepares data
for downstream analysis by combining these results.
}
\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# Complete pipeline example with smaller dataset
threshold <- 130
gap <- 60
hours <- 2
# 1) Find GRID points
grid_result <- grid(example_data_5_subject, gap = gap, threshold = threshold)
# 2) Find modified GRID points before 2 hours minimum
mod_grid <- mod_grid(example_data_5_subject,
                     start_finder(grid_result$grid_vector),
                     hours = hours,
                     gap = gap)

# 3) Find maximum point 2 hours after mod_grid point
mod_grid_maxima <- find_max_after_hours(example_data_5_subject,
                                        start_finder(mod_grid$mod_grid_vector),
                                        hours = hours)

# 4) Identify local maxima around episodes/windows
local_maxima <- find_local_maxima(example_data_5_subject)

# 5) Among local maxima, find maximum point after two hours
final_maxima <- find_new_maxima(example_data_5_subject,
                                mod_grid_maxima$max_index,
                                local_maxima$local_maxima_vector)

# 6) Map GRID points to maximum points (within 4 hours)
transform_maxima <- transform_df(grid_result$episode_start, final_maxima)

# 7) Redistribute overlapping maxima between GRID points
final_between_maxima <- detect_between_maxima(example_data_5_subject, transform_maxima)
# Complete pipeline example with larger dataset (example_data_hall)
# This demonstrates the same workflow on a more comprehensive dataset
hall_threshold <- 130
hall_gap <- 60
hall_hours <- 2

# 1) Find GRID points on larger dataset
hall_grid_result <- grid(example_data_hall, gap = hall_gap, threshold = hall_threshold)

# 2) Find modified GRID points
hall_mod_grid <- mod_grid(example_data_hall,
                         start_finder(hall_grid_result$grid_vector),
                         hours = hall_hours,
                         gap = hall_gap)

# 3) Find maximum points after mod_grid
hall_mod_grid_maxima <- find_max_after_hours(example_data_hall,
                                            start_finder(hall_mod_grid$mod_grid_vector),
                                            hours = hall_hours)

# 4) Identify local maxima
hall_local_maxima <- find_local_maxima(example_data_hall)

# 5) Find new maxima
hall_final_maxima <- find_new_maxima(example_data_hall,
                                    hall_mod_grid_maxima$max_index,
                                    hall_local_maxima$local_maxima_vector)

# 6) Transform data
hall_transform_maxima <- transform_df(hall_grid_result$episode_start, hall_final_maxima)

# 7) Detect between maxima
hall_final_between_maxima <- detect_between_maxima(example_data_hall, hall_transform_maxima)
}
\seealso{
\link{grid}, \link{find_new_maxima}, \link{detect_between_maxima}

Other GRID pipeline: 
\code{\link{detect_between_maxima}()},
\code{\link{find_local_maxima}()},
\code{\link{find_max_after_hours}()},
\code{\link{find_max_before_hours}()},
\code{\link{find_min_after_hours}()},
\code{\link{find_min_before_hours}()},
\code{\link{find_new_maxima}()},
\code{\link{grid}()},
\code{\link{maxima_grid}()},
\code{\link{mod_grid}()},
\code{\link{start_finder}()}
}
\concept{GRID pipeline}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{find_max_before_hours}
\alias{find_max_before_hours}
\title{Find Maximum Glucose Before Specified Hours}
\usage{
find_max_before_hours(df, start_point_df, hours)
}
\arguments{
\item{df}{A dataframe containing continuous glucose monitoring (CGM) data.
Must include columns:
\itemize{
  \item \code{id}: Subject identifier (string or factor)
  \item \code{time}: Time of measurement (POSIXct)
  \item \code{gl}: Glucose value (integer or numeric, mg/dL)
}}

\item{start_point_df}{A dataframe with column \code{start_index} (R-based index into \code{df})}

\item{hours}{Number of hours to look back from the start point}
}
\value{
A list containing:
\itemize{
  \item \code{max_index}: Tibble with R-based (1-indexed) row numbers of maximum glucose (\code{max_index}).
    The corresponding occurrence time is \code{df$time[max_index]} and glucose is \code{df$gl[max_index]}.
  \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
  \item \code{episode_start}: Tibble with all episode starts with columns:
    \itemize{
      \item \code{id}: Subject identifier
      \item \code{time}: Timestamp at which the maximum occurs; equivalent to \code{df$time[index]}
      \item \code{gl}: Glucose value at the maximum; equivalent to \code{df$gl[index]}
      \item \code{index}: R-based (1-indexed) row number(s) in \code{df} denoting where the maximum occurs
    }
}
}
\description{
Identifies the maximum glucose value occurring within a specified time window
before a given start point. This function is useful for analyzing glucose
patterns preceding specific events or time points.
}
\section{Notes}{

- The search window is [\code{hours}, 0) hours before each start index.
}

\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# Create start points for demonstration (using row index)
start_index <- seq(1, nrow(example_data_5_subject), by = 100)
start_points <- data.frame(start_index = start_index)

# Find maximum glucose in previous 2 hours
max_before <- find_max_before_hours(example_data_5_subject, start_points, hours = 2)
print(paste("Found", length(max_before$max_index), "maximum points"))

# Find maximum glucose in previous 1 hour
max_before_1h <- find_max_before_hours(example_data_5_subject, start_points, hours = 1)

# Analysis on larger dataset
large_start_index <- seq(1, nrow(example_data_hall), by = 200)
large_start_points <- data.frame(start_index = large_start_index)
large_max_before <- find_max_before_hours(example_data_hall, large_start_points, hours = 2)
print(paste("Found", length(large_max_before$max_index), "maximum points in larger dataset"))
}
\seealso{
\link{mod_grid}, \link{find_local_maxima}, \link{find_new_maxima}

Other GRID pipeline: 
\code{\link{detect_between_maxima}()},
\code{\link{find_local_maxima}()},
\code{\link{find_max_after_hours}()},
\code{\link{find_min_after_hours}()},
\code{\link{find_min_before_hours}()},
\code{\link{find_new_maxima}()},
\code{\link{grid}()},
\code{\link{maxima_grid}()},
\code{\link{mod_grid}()},
\code{\link{start_finder}()},
\code{\link{transform_df}()}
}
\concept{GRID pipeline}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{sensor_wear}
\alias{sensor_wear}
\title{Calculate Sensor Wear}
\usage{
sensor_wear(df, end_date = NULL, ndays = 14,
 reading_minutes = NULL)
}
\arguments{
\item{df}{A dataframe containing CGM data with columns:
\itemize{
  \item \code{id}: Subject identifier
  \item \code{time}: POSIXct measurement timestamp
  \item \code{gl}: Glucose value in mg/dL
}}

\item{end_date}{End timestamp for the calculation window. If \code{NULL},
each subject's last valid timestamp is used. \code{Date} values are
converted with \code{as.POSIXct()}, matching iglu's manual active-percent
behavior.}

\item{ndays}{Number of days in the retrospective window. Defaults to 14.}

\item{reading_minutes}{Reading interval in minutes. If \code{NULL}, it is
inferred per id from the median positive difference between valid readings.}
}
\value{
A tibble with columns \code{id}, \code{sensor_wear}, \code{ndays},
  \code{start_date}, and \code{end_date}.
}
\description{
Calculates the percent of expected CGM readings observed during a fixed
retrospective window. This C++ implementation follows the manual-range logic
used by \code{iglu::active_percent(range_type = "manual")}: valid readings
in \code{[end_date - ndays, end_date]} are divided by the expected number of
readings over \code{ndays} days.

If \code{end_date = NULL}, each subject's last valid timestamp defines that
subject's retrospective window. If \code{end_date} is supplied, the same
endpoint is used for all subjects, which is useful for a common study cutoff
or report date. Duplicate timestamps within a subject are de-duplicated after
sorting, and rows with missing \code{time} or \code{gl} do not count as
observed readings.
}
\examples{
library(iglu)
data(example_data_5_subject)
sensor_wear(example_data_5_subject, ndays = 14, reading_minutes = 5)
}
\seealso{
\link{detect_all_events}, \code{\link[iglu:active_percent]{iglu::active_percent}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{detect_between_maxima}
\alias{detect_between_maxima}
\title{Detect Events Between Maxima}
\usage{
detect_between_maxima(df, transform_df)
}
\arguments{
\item{df}{A dataframe containing continuous glucose monitoring (CGM) data.
Must include columns:
\itemize{
  \item \code{id}: Subject identifier (string or factor)
  \item \code{time}: Time of measurement (POSIXct)
  \item \code{gl}: Glucose value (integer or numeric, mg/dL)
}}

\item{transform_df}{A dataframe containing summary information from previous transformations}
}
\value{
A list containing:
\itemize{
  \item \code{results}: Tibble with events between maxima (\code{id}, \code{grid_time}, \code{grid_gl}, \code{maxima_time}, \code{maxima_glucose}, \code{time_to_peak})
  \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
}
}
\description{
Identifies and analyzes events occurring between detected maxima points,
providing detailed episode information for GRID analysis. This function
helps characterize the glucose dynamics between identified peaks.
}
\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# Complete pipeline to get transform_df
grid_result <- grid(example_data_5_subject, gap = 60, threshold = 130)
maxima_result <- find_local_maxima(example_data_5_subject)
mod_result <- mod_grid(example_data_5_subject, grid_result$grid_vector, hours = 2, gap = 60)
max_after <- find_max_after_hours(example_data_5_subject, mod_result$mod_grid_vector, hours = 2)
new_maxima <- find_new_maxima(example_data_5_subject,
                              max_after$max_index,
                              maxima_result$local_maxima_vector)
transformed <- transform_df(grid_result$episode_start, new_maxima)

# Detect events between maxima
between_events <- detect_between_maxima(example_data_5_subject, transformed)
print(paste("Events between maxima:", length(between_events)))

# Analysis on larger dataset
large_grid <- grid(example_data_hall, gap = 60, threshold = 130)
large_maxima <- find_local_maxima(example_data_hall)
large_mod <- mod_grid(example_data_hall, large_grid$grid_vector, hours = 2, gap = 60)
large_max_after <- find_max_after_hours(example_data_hall, large_mod$mod_grid_vector, hours = 2)
large_new_maxima <- find_new_maxima(example_data_hall,
                                    large_max_after$max_index,
                                    large_maxima$local_maxima_vector)
large_transformed <- transform_df(large_grid$episode_start, large_new_maxima)
large_between <- detect_between_maxima(example_data_hall, large_transformed)
print(paste("Events between maxima in larger dataset:", length(large_between)))
}
\seealso{
\link{grid}, \link{mod_grid}, \link{find_new_maxima}, \link{transform_df}

Other GRID pipeline: 
\code{\link{find_local_maxima}()},
\code{\link{find_max_after_hours}()},
\code{\link{find_max_before_hours}()},
\code{\link{find_min_after_hours}()},
\code{\link{find_min_before_hours}()},
\code{\link{find_new_maxima}()},
\code{\link{grid}()},
\code{\link{maxima_grid}()},
\code{\link{mod_grid}()},
\code{\link{start_finder}()},
\code{\link{transform_df}()}
}
\concept{GRID pipeline}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{start_finder}
\alias{start_finder}
\title{Find Start Points for Event Analysis}
\usage{
start_finder(df)
}
\arguments{
\item{df}{A dataframe with the first column containing an integer vector of 0s and 1s}
}
\value{
A tibble containing start_index with R-based (1-indexed) positions where episodes start
Note: These index refer to positions in the provided input vector/dataframe, not necessarily rows of the original CGM \code{df} unless that vector was derived directly from \code{df} in row order.
}
\description{
Finds R-based (1-indexed) positions where the value is 1 in an integer vector
of 0s and 1s, specifically identifying episode start points. This function
looks for positions where a 1 follows a 0 or is at the beginning of the vector,
which is useful for identifying the start of glycemic events or episodes.
}
\section{Notes}{

- Returns R-based \code{start_index} positions relative to the provided input vector/dataframe.
- If used on vectors derived from a CGM \code{df}, index map directly to \code{df} rows.
}

\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# Create a binary vector indicating episode starts
binary_vector <- c(0, 0, 1, 1, 0, 1, 0, 0, 1, 1)
df <- data.frame(episode_starts = binary_vector)

# Find R-based index where episodes start
start_points <- start_finder(df)
print(paste("Start index:", paste(start_points$start_index, collapse = ", ")))

# Use with actual GRID results
grid_result <- grid(example_data_5_subject, gap = 15, threshold = 130)
grid_starts <- start_finder(grid_result$grid_vector)
print(paste("GRID episode starts:", length(grid_starts$start_index)))

# Analysis on larger dataset
large_grid <- grid(example_data_hall, gap = 15, threshold = 130)
large_starts <- start_finder(large_grid$grid_vector)
print(paste("GRID episode starts in larger dataset:", length(large_starts$start_index)))
}
\seealso{
\link{grid}, \link{mod_grid}

Other GRID pipeline: 
\code{\link{detect_between_maxima}()},
\code{\link{find_local_maxima}()},
\code{\link{find_max_after_hours}()},
\code{\link{find_max_before_hours}()},
\code{\link{find_min_after_hours}()},
\code{\link{find_min_before_hours}()},
\code{\link{find_new_maxima}()},
\code{\link{grid}()},
\code{\link{maxima_grid}()},
\code{\link{mod_grid}()},
\code{\link{transform_df}()}
}
\concept{GRID pipeline}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{find_local_maxima}
\alias{find_local_maxima}
\title{Find Local Maxima in Glucose Time Series}
\usage{
find_local_maxima(df)
}
\arguments{
\item{df}{A dataframe containing continuous glucose monitoring (CGM) data.
Must include columns:
\itemize{
  \item \code{id}: Subject identifier (string or factor)
  \item \code{time}: Time of measurement (POSIXct)
  \item \code{gl}: Glucose value (integer or numeric, mg/dL)
}}
}
\value{
A list containing:
\itemize{
  \item \code{local_maxima_vector}: Tibble with R-based (1-indexed) row numbers of local maxima (\code{local_maxima}). The corresponding occurrence time is \code{df$time[local_maxima]} and glucose is \code{df$gl[local_maxima]}.
  \item \code{merged_results}: Tibble with local maxima details (\code{id}, \code{time}, \code{gl})
}
}
\description{
Identifies local maxima (peaks) in glucose concentration time series data.
Uses a difference-based algorithm to detect peaks where glucose values
increase or remain constant for two consecutive points before the peak point,
and decrease or remain constant for two consecutive points after the peak point.
}
\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# Find local maxima
maxima_result <- find_local_maxima(example_data_5_subject)
print(paste("Found", nrow(maxima_result$local_maxima_vector), "local maxima"))

# Find maxima on larger dataset
large_maxima <- find_local_maxima(example_data_hall)
print(paste("Found", nrow(large_maxima$local_maxima_vector), "local maxima in larger dataset"))

# View first few maxima
head(maxima_result$local_maxima_vector)

# View merged results
head(maxima_result$merged_results)
}
\seealso{
\link{grid}, \link{mod_grid}, \link{find_new_maxima}

Other GRID pipeline: 
\code{\link{detect_between_maxima}()},
\code{\link{find_max_after_hours}()},
\code{\link{find_max_before_hours}()},
\code{\link{find_min_after_hours}()},
\code{\link{find_min_before_hours}()},
\code{\link{find_new_maxima}()},
\code{\link{grid}()},
\code{\link{maxima_grid}()},
\code{\link{mod_grid}()},
\code{\link{start_finder}()},
\code{\link{transform_df}()}
}
\concept{GRID pipeline}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{detect_all_events}
\alias{detect_all_events}
\title{Detect All Glycemic Events}
\usage{
detect_all_events(df, reading_minutes = NULL, sort_time = FALSE,
 inter_gap = 45, return_interpolated = FALSE)
}
\arguments{
\item{df}{A dataframe containing continuous glucose monitoring (CGM) data.
Must include columns:
\itemize{
  \item \code{id}: Subject identifier (string or factor)
  \item \code{time}: Time of measurement (POSIXct)
  \item \code{gl}: Glucose value (integer or numeric, mg/dL)
}}

\item{reading_minutes}{Time interval between readings in minutes (optional).
Can be a single integer/numeric value (applied to all subjects), a vector
matching data length, or \code{NULL}. If omitted or \code{NULL}, the
interval is calculated automatically per id as the median positive time
difference in the data.}

\item{sort_time}{Logical. If \code{TRUE}, sort rows within each id by
\code{time} in C++ before interpolation. Defaults to \code{FALSE}.}

\item{inter_gap}{Maximum gap in minutes to interpolate across. Defaults to
45; larger gaps split event-detection segments.}

\item{return_interpolated}{Logical. Retained for backward compatibility and
ignored; \code{detect_all_events()} does not return interpolated data.}
}
\value{
A list with two tibbles:
\itemize{
  \item \code{events_long_df}: One row per subject, event type, and event
    level. Contains the full event summary: \code{id}, \code{type},
    \code{level}, \code{total_episodes}, \code{avg_ep_per_day}, and
    \code{avg_episode_duration_below_54}.
  \item \code{summary_df}: One row per subject. CGM summary metric columns
    are calculated on the interpolated glucose grid, and event summaries are
    included as wide \code{*_event_count} columns only.
}
\code{summary_df} includes:
\itemize{
  \item \code{id}: Subject identifier
  \item \code{TIR}: Time in range 70-180 mg/dL, percent
  \item \code{TITR}: Time in tight range 70-140 mg/dL, percent
  \item \code{TBR70}, \code{TBR54}: Time below 70 and 54 mg/dL, percent
  \item \code{TAR180}, \code{TAR250}: Time above 180 and 250 mg/dL, percent
  \item \code{CV}: Glucose standard deviation divided by mean glucose
  \item \code{SD}: Glucose standard deviation
  \item \code{mean_glucose}: Mean glucose in mg/dL
  \item \code{GMI}: \eqn{3.31 + 0.02392 * mean_glucose}
  \item \code{uGMI}: \eqn{1 / (15.36 / mean_glucose + 0.0425)}
  \item \code{GRI}: Glycemia Risk Index,
    \eqn{3.0 * VLow + 2.4 * Low + 1.6 * VHigh + 0.8 * High}, where
    \code{VLow} is percent time \eqn{<}54 mg/dL, \code{Low} is 54-\eqn{<}70
    mg/dL, \code{VHigh} is \eqn{>}250 mg/dL, and \code{High} is
    \eqn{>}180-\eqn{\leq}250 mg/dL
  \item \code{sensor_wear}: Percent of expected CGM readings observed,
    calculated from the original timestamps using the same automatic range
    method as \code{iglu::active_percent()}
}
Event counts are returned in wide columns for each hypo/hyper level
combination, for example \code{hypo_lv1_event_count}. The same pattern is
repeated for \code{lv1}, \code{lv2}, \code{extended}, and \code{lv1_excl}
within both \code{hypo} and \code{hyper}.
}
\description{
Comprehensive function to detect all types of glycemic events aligned with
international consensus CGM metrics (Battelino et al., 2023). This function
provides a unified interface for detecting multiple event types including
Level 1/2/Extended hypo- and hyperglycemia, and Level 1 excluded events.
Events are counted only after the required recovery condition is confirmed;
duration summaries use the event boundary immediately before recovery starts.
Event preprocessing uses cgmguru's independent C++ implementation of an
iglu-compatible day-based grid: each subject is interpolated from the first
observed day's midnight plus one reading interval, rather than from the first
observed timestamp. Larger gaps are masked and removed before event
classification, preserving gap-based segment boundaries. This preprocessing
is specific to event calculation and does not affect \code{\link{grid}},
\code{\link{maxima_grid}}, or \code{\link{excursion}}.
}
\section{Event types}{

- Hypoglycemia: lv1 (\eqn{<} 70 mg/dL, \eqn{\geq} 15 min), lv2 (\eqn{<} 54 mg/dL, \eqn{\geq} 15 min), extended (\eqn{<} 70 mg/dL, \eqn{\geq} 120 min).
- Hyperglycemia: lv1 (\eqn{>} 180 mg/dL, \eqn{\geq} 15 min), lv2 (\eqn{>} 250 mg/dL, \eqn{\geq} 15 min), extended (\eqn{>} 250 mg/dL, \eqn{\geq} 90 min in 120 min, end \eqn{\leq} 180 mg/dL for \eqn{\geq} 15 min).
}

\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# Detect all glycemic events; reading_minutes is calculated automatically
# from the timestamp spacing when omitted
all_outputs <- detect_all_events(example_data_5_subject)
print(all_outputs$summary_df)
print(all_outputs$events_long_df)

# Detect all events on larger dataset
large_outputs <- detect_all_events(example_data_hall)
print(paste("Total subjects analyzed:", nrow(large_outputs$summary_df)))
}
\references{
Battelino, T., et al. (2023). Continuous glucose monitoring and metrics for clinical trials: an international consensus statement. The Lancet Diabetes & Endocrinology, 11(1), 42-57.
}
\seealso{
\link{detect_hyperglycemic_events}, \link{detect_hypoglycemic_events}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{find_new_maxima}
\alias{find_new_maxima}
\title{Find New Maxima Around Grid Points}
\usage{
find_new_maxima(df, mod_grid_max_point_df, local_maxima_df)
}
\arguments{
\item{df}{A dataframe containing continuous glucose monitoring (CGM) data.
Must include columns:
\itemize{
  \item \code{id}: Subject identifier (string or factor)
  \item \code{time}: Time of measurement (POSIXct)
  \item \code{gl}: Glucose value (integer or numeric, mg/dL)
}}

\item{mod_grid_max_point_df}{A dataframe with column \code{index} (candidate maxima index)}

\item{local_maxima_df}{A dataframe with column \code{local_maxima} (index of local peaks)}
}
\value{
A tibble with updated maxima information containing columns (\code{id}, \code{time}, \code{gl}, \code{index})
The \code{index} column contains R-based (1-indexed) row number(s) in \code{df}; thus, \code{time == df$time[index]} and \code{gl == df$gl[index]}.
}
\description{
Identifies new maxima in the vicinity of previously identified grid points,
useful for refining maxima detection in GRID analysis. This function helps
improve the accuracy of peak detection by searching around known event points.
}
\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# First, get grid points and local maxima
grid_result <- grid(example_data_5_subject, gap = 15, threshold = 130)
maxima_result <- find_local_maxima(example_data_5_subject)

# Create modified grid points (simplified for example)
mod_grid_indices <- data.frame(index = grid_result$episode_start$index[1:10])

# Find new maxima around grid points
new_maxima <- find_new_maxima(example_data_5_subject,
                              mod_grid_indices,
                              maxima_result$local_maxima_vector)
print(paste("Found", nrow(new_maxima), "new maxima"))

# Analysis on larger dataset
large_grid <- grid(example_data_hall, gap = 15, threshold = 130)
large_maxima <- find_local_maxima(example_data_hall)
large_mod_grid <- data.frame(index = large_grid$episode_start$index[1:20])
large_new_maxima <- find_new_maxima(example_data_hall,
                                    large_mod_grid,
                                    large_maxima$local_maxima_vector)
print(paste("Found", nrow(large_new_maxima), "new maxima in larger dataset"))
}
\seealso{
\link{find_local_maxima}, \link{find_max_after_hours}, \link{transform_df}

Other GRID pipeline: 
\code{\link{detect_between_maxima}()},
\code{\link{find_local_maxima}()},
\code{\link{find_max_after_hours}()},
\code{\link{find_max_before_hours}()},
\code{\link{find_min_after_hours}()},
\code{\link{find_min_before_hours}()},
\code{\link{grid}()},
\code{\link{maxima_grid}()},
\code{\link{mod_grid}()},
\code{\link{start_finder}()},
\code{\link{transform_df}()}
}
\concept{GRID pipeline}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{maxima_grid}
\alias{maxima_grid}
\title{Combined Maxima Detection and GRID Analysis}
\usage{
maxima_grid(df, threshold = 130, gap = 60, hours = 2)
}
\arguments{
\item{df}{A dataframe containing continuous glucose monitoring (CGM) data.
Must include columns:
\itemize{
  \item \code{id}: Subject identifier (string or factor)
  \item \code{time}: Time of measurement (POSIXct)
  \item \code{gl}: Glucose value (integer or numeric, mg/dL)
}}

\item{threshold}{GRID slope threshold in mg/dL/hour for event classification (default: 130)}

\item{gap}{Gap threshold in minutes for event detection (default: 60).
This parameter defines the minimum time interval between consecutive GRID events.}

\item{hours}{Time window in hours for maxima analysis (default: 2)}
}
\value{
A list containing:
\itemize{
  \item \code{results}: Tibble with combined maxima and GRID analysis results, with columns:
    \itemize{
      \item \code{id}: Subject identifier
      \item \code{grid_time}: Timestamp of GRID event detection (POSIXct)
      \item \code{grid_gl}: Glucose value at GRID event (mg/dL)
      \item \code{maxima_time}: Timestamp of peak glucose (POSIXct)
      \item \code{maxima_glucose}: Peak glucose value (mg/dL)
      \item \code{time_to_peak_min}: Time from GRID event to peak in minutes
      \item \code{grid_index}: R-based (1-indexed) row number of GRID event; \code{grid_time == df$time[grid_index]}, \code{grid_gl == df$gl[grid_index]}
      \item \code{maxima_index}: R-based (1-indexed) row number of peak; \code{maxima_time == df$time[maxima_index]}, \code{maxima_glucose == df$gl[maxima_index]}
    }
  \item \code{episode_counts}: Tibble with episode counts per subject (\code{id}, \code{episode_counts})
}
}
\description{
Fast method for postprandial glucose peak detection combining GRID algorithm with local maxima analysis.
Detects meal-induced glucose peaks by identifying GRID events (rapid glucose increases) and mapping
them to corresponding local maxima within a search window. Local maxima are defined as points where
glucose values increase or remain constant for two consecutive points before the peak, and decrease
or remain constant for two consecutive points after the peak.

The 7-step algorithm:
(1) finds GRID points indicating meal starts
(2) identifies modified GRID points after minimum duration
(3) locates maximum glucose within the subsequent time window
(4) detects all local maxima using the two-consecutive-point criteria
(5) refines peaks from local maxima candidates
(6) maps GRID points to peaks within 4-hour constraint
(7) redistributes overlapping peaks.
}
\section{Algorithm (7 steps)}{

1) GRID -> 2) modified GRID -> 3) window maxima -> 4) local maxima -> 5) refine peaks ->
6) map GRID to peaks (\eqn{\leq} 4h) -> 7) redistribute overlapping peaks.
}

\section{Input grid}{

This function operates on the rows supplied in \code{df}. It does not use
\code{\link{interpolate_cgm}} or the full-day event preprocessing grid unless
the caller explicitly supplies interpolated data.
}

\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# Combined analysis on smaller dataset
maxima_result <- maxima_grid(example_data_5_subject, threshold = 130, gap = 60, hours = 2)
print(maxima_result$episode_counts)
print(maxima_result$results)

# More sensitive analysis
sensitive_maxima <- maxima_grid(example_data_5_subject, threshold = 120, gap = 30, hours = 1)
print(sensitive_maxima$episode_counts)
print(sensitive_maxima$results)

# Analysis on larger dataset
large_maxima <- maxima_grid(example_data_hall, threshold = 130, gap = 60, hours = 2)
print(large_maxima$episode_counts)
print(large_maxima$results)
}
\references{
Park, Sang Ho, et al. "Identification of clinically meaningful automatically detected postprandial glucose excursions in individuals with type 1 diabetes using personal continuous glucose monitoring." Diabetes Research and Clinical Practice (2025): 112951.

Park, Soojin, et al. "High-Amplitude and Prolonged Glucose Excursions as a Key Determinant of Discordance Between Glucose Management Indicator and Glycated Hemoglobin in Type 1 Diabetes." Diabetes Care (2026): dc252820. https://doi.org/10.2337/dc25-2820
}
\seealso{
\link{grid}, \link{mod_grid}, \link{find_local_maxima}, \link{find_new_maxima}, \link{transform_df}

Other GRID pipeline: 
\code{\link{detect_between_maxima}()},
\code{\link{find_local_maxima}()},
\code{\link{find_max_after_hours}()},
\code{\link{find_max_before_hours}()},
\code{\link{find_min_after_hours}()},
\code{\link{find_min_before_hours}()},
\code{\link{find_new_maxima}()},
\code{\link{grid}()},
\code{\link{mod_grid}()},
\code{\link{start_finder}()},
\code{\link{transform_df}()}
}
\concept{GRID pipeline}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgmguru-functions-docs.R
\name{grid}
\alias{grid}
\title{GRID Algorithm for Glycemic Event Detection}
\usage{
grid(df, gap = 15, threshold = 130)
}
\arguments{
\item{df}{A dataframe containing continuous glucose monitoring (CGM) data.
Must include columns:
\itemize{
  \item \code{id}: Subject identifier (string or factor)
  \item \code{time}: Time of measurement (POSIXct)
  \item \code{gl}: Glucose value (integer or numeric, mg/dL)
}}

\item{gap}{Gap threshold in minutes for event detection (default: 15).
This parameter defines the minimum time interval between consecutive GRID events. For example, if gap is set to 60, only one GRID event can be detected within any one-hour window; subsequent events within the gap interval are not counted as new events.}

\item{threshold}{GRID slope threshold in mg/dL/hour for event classification (default: 130)}
}
\value{
A list containing:
\itemize{
  \item \code{grid_vector}: A tibble with the results of the GRID analysis. Contains a \code{grid} column (0/1 values; 1 denotes a detected GRID event) and all relevant input columns.
  \item \code{episode_counts}: A tibble summarizing the number of GRID events per subject (\code{id}) as \code{episode_counts}.
  \item \code{episode_start}: A tibble listing the start of each GRID episode, with columns:
    \itemize{
      \item \code{id}: Subject ID.
      \item \code{time}: The timestamp (POSIXct) at which the GRID event was detected.
      \item \code{gl}: The glucose value (mg/dL; integer or numeric) at the GRID event.
      \item \code{index}: R-based (1-indexed) row number(s) in the original dataframe where the GRID event occurs. The occurrence time equals \code{df$time[index]} and glucose equals \code{df$gl[index]}.
    }
}
}
\description{
Implements the GRID (Glucose Rate Increase Detector) algorithm for detecting rapid glucose rate increases in continuous glucose monitoring (CGM) data.
This algorithm identifies rapid glucose changes using specific rate-based criteria, and is commonly applied for meal detection.
Meals are detected when the CGM value is \eqn{\geq} 7.2 mmol/L (\eqn{\geq} 130 mg/dL) and the rate-of-change is \eqn{\geq} 5.3 mmol/L/h [\eqn{\geq} 95 mg/dL/h] for the last two consecutive readings, or \eqn{\geq} 5.0 mmol/L/h [\eqn{\geq} 90 mg/dL/h] for two of the last three readings.
}
\section{Algorithm}{

- Flags points where \code{gl >= 130 mg/dL} and rate-of-change meets the GRID criteria (see references).
- Enforces a minimum \code{gap} in minutes between detected events to avoid duplicates.
}

\section{Units and sampling}{

- \code{gl} is mg/dL; \code{time} is POSIXct; \code{gap} is minutes.
- The effective sampling interval is derived from \code{time} deltas.
- This function operates on the rows supplied in \code{df}. It does not use
  \code{\link{interpolate_cgm}} or the full-day event preprocessing grid.
}

\examples{
# Load sample data
library(iglu)
data(example_data_5_subject)
data(example_data_hall)

# Basic GRID analysis on smaller dataset
grid_result <- grid(example_data_5_subject, gap = 15, threshold = 130)
print(grid_result$episode_counts)
print(grid_result$episode_start)
print(grid_result$grid_vector)

# More sensitive GRID analysis
sensitive_result <- grid(example_data_5_subject, gap = 10, threshold = 120)

# GRID analysis on larger dataset
large_grid <- grid(example_data_hall, gap = 15, threshold = 130)
print(paste("Detected", sum(large_grid$episode_counts$episode_counts), "episodes"))
print(large_grid$episode_start)
print(large_grid$grid_vector)

}
\references{
Harvey, R. A., et al. (2014). Design of the glucose rate increase detector: a meal detection module for the health monitoring system. Journal of Diabetes Science and Technology, 8(2), 307-320.

Adolfsson, Peter, et al. "Increased time in range and fewer missed bolus injections after introduction of a smart connected insulin pen." Diabetes technology & therapeutics 22.10 (2020): 709-718.
}
\seealso{
\link{mod_grid}, \link{maxima_grid}, \link{find_local_maxima}, \link{detect_between_maxima}

Other GRID pipeline: 
\code{\link{detect_between_maxima}()},
\code{\link{find_local_maxima}()},
\code{\link{find_max_after_hours}()},
\code{\link{find_max_before_hours}()},
\code{\link{find_min_after_hours}()},
\code{\link{find_min_before_hours}()},
\code{\link{find_new_maxima}()},
\code{\link{maxima_grid}()},
\code{\link{mod_grid}()},
\code{\link{start_finder}()},
\code{\link{transform_df}()}
}
\concept{GRID pipeline}
